@@ -4,22 +4,23 @@ mod quantile;
 mod sum;
 mod variance;
 
-use super::*;
-use crate::utils::CustomIterTools;
+use std::fmt::Debug;
+
 use arrow::array::PrimitiveArray;
 use arrow::datatypes::DataType;
 use arrow::types::NativeType;
-use num::{Float, NumCast};
-#[cfg(feature = "serde")]
-use serde::{Deserialize, Serialize};
-use std::fmt::Debug;
-
 pub use mean::*;
 pub use min_max::*;
+use num::{Float, NumCast};
 pub use quantile::*;
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
 pub use sum::*;
 pub use variance::*;
 
+use super::*;
+use crate::utils::CustomIterTools;
+
 pub trait RollingAggWindowNoNulls<'a, T: NativeType> {
     fn new(slice: &'a [T], start: usize, end: usize) -> Self;
 
@@ -54,7 +55,7 @@ where
         })
         .collect_trusted::<Vec<_>>();
 
-    let validity = create_validity(min_periods, len as usize, window_size, det_offsets_fn);
+    let validity = create_validity(min_periods, len, window_size, det_offsets_fn);
     Box::new(PrimitiveArray::from_data(
         T::PRIMITIVE.into(),
         out.into(),
@@ -102,7 +103,7 @@ where
         })
         .collect_trusted::<Vec<T>>();
 
-    let validity = create_validity(min_periods, len as usize, window_size, det_offsets_fn);
+    let validity = create_validity(min_periods, len, window_size, det_offsets_fn);
     Box::new(PrimitiveArray::from_data(
         DataType::from(T::PRIMITIVE),
         out.into(),
