@@ -64,6 +64,15 @@ where
             DataType::Categorical(_) => {
                 Ok(CategoricalChunked::full_null(self.name(), self.len()).into_series())
             }
+            #[cfg(feature = "dtype-struct")]
+            DataType::Struct(fields) => {
+                // cast to first field dtype
+                let fld = &fields[0];
+                let dtype = &fld.dtype;
+                let name = &fld.name;
+                let s = cast_impl_inner(name, &self.chunks, dtype, true)?;
+                Ok(StructChunked::new_unchecked(self.name(), &[s]).into_series())
+            }
             _ => cast_impl_inner(self.name(), &self.chunks, data_type, checked).map(|mut s| {
                 // maintain sorted if data types remain signed
                 // this may still fail with overflow?
@@ -113,6 +122,17 @@ impl ChunkCast for Utf8Chunked {
     }
 }
 
+#[cfg(feature = "dtype-binary")]
+impl ChunkCast for BinaryChunked {
+    fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
+        cast_impl(self.name(), &self.chunks, data_type)
+    }
+
+    fn cast_unchecked(&self, data_type: &DataType) -> PolarsResult<Series> {
+        self.cast(data_type)
+    }
+}
+
 fn boolean_to_utf8(ca: &BooleanChunked) -> Utf8Chunked {
     ca.into_iter()
         .map(|opt_b| match opt_b {
@@ -160,17 +180,36 @@ impl ChunkCast for ListChunked {
         match data_type {
             DataType::List(child_type) => {
                 let phys_child = child_type.to_physical();
-                let mut ca = if child_type.to_physical() != self.inner_dtype().to_physical() {
-                    let chunks = self
-                        .downcast_iter()
-                        .map(|list| cast_inner_list_type(list, &phys_child))
-                        .collect::<PolarsResult<_>>()?;
-                    ListChunked::from_chunks(self.name(), chunks)
+
+                if phys_child.is_primitive() {
+                    let mut ca = if child_type.to_physical() != self.inner_dtype().to_physical() {
+                        let chunks = self
+                            .downcast_iter()
+                            .map(|list| cast_inner_list_type(list, &phys_child))
+                            .collect::<PolarsResult<_>>()?;
+                        ListChunked::from_chunks(self.name(), chunks)
+                    } else {
+                        self.clone()
+                    };
+                    ca.set_inner_dtype(*child_type.clone());
+                    Ok(ca.into_series())
                 } else {
-                    self.clone()
-                };
-                ca.set_inner_dtype(*child_type.clone());
-                Ok(ca.into_series())
+                    let ca = self.rechunk();
+                    let arr = ca.downcast_iter().next().unwrap();
+                    let s = Series::try_from(("", arr.values().clone())).unwrap();
+                    let new_inner = s.cast(child_type)?;
+                    let new_values = new_inner.array_ref(0).clone();
+
+                    let data_type =
+                        ListArray::<i64>::default_datatype(new_values.data_type().clone());
+                    let new_arr = ListArray::<i64>::new(
+                        data_type,
+                        arr.offsets().clone(),
+                        new_values,
+                        arr.validity().cloned(),
+                    );
+                    Series::try_from((self.name(), Box::new(new_arr) as ArrayRef))
+                }
             }
             _ => Err(PolarsError::ComputeError("Cannot cast list type".into())),
         }
