@@ -30,7 +30,7 @@ impl<T> ChunkFilter<T> for ChunkedArray<T>
 where
     T: PolarsNumericType,
 {
-    fn filter(&self, filter: &BooleanChunked) -> Result<ChunkedArray<T>> {
+    fn filter(&self, filter: &BooleanChunked) -> PolarsResult<ChunkedArray<T>> {
         // broadcast
         if filter.len() == 1 {
             return match filter.get(0) {
@@ -51,7 +51,7 @@ where
 }
 
 impl ChunkFilter<BooleanType> for BooleanChunked {
-    fn filter(&self, filter: &BooleanChunked) -> Result<ChunkedArray<BooleanType>> {
+    fn filter(&self, filter: &BooleanChunked) -> PolarsResult<ChunkedArray<BooleanType>> {
         // broadcast
         if filter.len() == 1 {
             return match filter.get(0) {
@@ -72,7 +72,7 @@ impl ChunkFilter<BooleanType> for BooleanChunked {
 }
 
 impl ChunkFilter<Utf8Type> for Utf8Chunked {
-    fn filter(&self, filter: &BooleanChunked) -> Result<ChunkedArray<Utf8Type>> {
+    fn filter(&self, filter: &BooleanChunked) -> PolarsResult<ChunkedArray<Utf8Type>> {
         // broadcast
         if filter.len() == 1 {
             return match filter.get(0) {
@@ -93,8 +93,31 @@ impl ChunkFilter<Utf8Type> for Utf8Chunked {
     }
 }
 
+#[cfg(feature = "dtype-binary")]
+impl ChunkFilter<BinaryType> for BinaryChunked {
+    fn filter(&self, filter: &BooleanChunked) -> PolarsResult<ChunkedArray<BinaryType>> {
+        // broadcast
+        if filter.len() == 1 {
+            return match filter.get(0) {
+                Some(true) => Ok(self.clone()),
+                _ => Ok(BinaryChunked::full_null(self.name(), 0)),
+            };
+        }
+        check_filter_len!(self, filter);
+        let (left, filter) = align_chunks_binary(self, filter);
+
+        let chunks = left
+            .downcast_iter()
+            .zip(filter.downcast_iter())
+            .map(|(left, mask)| filter_fn(left, mask).unwrap())
+            .collect::<Vec<_>>();
+
+        Ok(self.copy_with_chunks(chunks, true))
+    }
+}
+
 impl ChunkFilter<ListType> for ListChunked {
-    fn filter(&self, filter: &BooleanChunked) -> Result<ListChunked> {
+    fn filter(&self, filter: &BooleanChunked) -> PolarsResult<ListChunked> {
         // broadcast
         if filter.len() == 1 {
             return match filter.get(0) {
@@ -126,7 +149,7 @@ impl<T> ChunkFilter<ObjectType<T>> for ObjectChunked<T>
 where
     T: PolarsObject,
 {
-    fn filter(&self, filter: &BooleanChunked) -> Result<ChunkedArray<ObjectType<T>>>
+    fn filter(&self, filter: &BooleanChunked) -> PolarsResult<ChunkedArray<ObjectType<T>>>
     where
         Self: Sized,
     {
