@@ -1,12 +1,14 @@
-#[cfg(feature = "object")]
-use crate::chunked_array::object::ObjectArray;
-use crate::prelude::downcast::Chunks;
-use crate::prelude::*;
+use std::convert::TryFrom;
+
 use arrow::array::{Array, BooleanArray, ListArray, PrimitiveArray, Utf8Array};
 use arrow::bitmap::utils::get_bit_unchecked;
 use arrow::bitmap::Bitmap;
 use polars_arrow::is_valid::*;
-use std::convert::TryFrom;
+
+#[cfg(feature = "object")]
+use crate::chunked_array::object::ObjectArray;
+use crate::prelude::downcast::Chunks;
+use crate::prelude::*;
 
 macro_rules! take_random_get {
     ($self:ident, $index:ident) => {{
@@ -230,6 +232,75 @@ impl<'a> IntoTakeRandom<'a> for &'a Utf8Chunked {
     }
 }
 
+#[cfg(feature = "dtype-binary")]
+pub struct BinaryTakeRandom<'a> {
+    pub(crate) chunks: Chunks<'a, BinaryArray<i64>>,
+    pub(crate) chunk_lens: Vec<IdxSize>,
+}
+
+#[cfg(feature = "dtype-binary")]
+impl<'a> TakeRandom for BinaryTakeRandom<'a> {
+    type Item = &'a [u8];
+
+    #[inline]
+    fn get(&self, index: usize) -> Option<Self::Item> {
+        take_random_get!(self, index)
+    }
+
+    #[inline]
+    unsafe fn get_unchecked(&self, index: usize) -> Option<Self::Item> {
+        take_random_get_unchecked!(self, index)
+    }
+}
+
+#[cfg(feature = "dtype-binary")]
+pub struct BinaryTakeRandomSingleChunk<'a> {
+    pub(crate) arr: &'a BinaryArray<i64>,
+}
+
+#[cfg(feature = "dtype-binary")]
+impl<'a> TakeRandom for BinaryTakeRandomSingleChunk<'a> {
+    type Item = &'a [u8];
+
+    #[inline]
+    fn get(&self, index: usize) -> Option<Self::Item> {
+        take_random_get_single!(self, index)
+    }
+
+    #[inline]
+    unsafe fn get_unchecked(&self, index: usize) -> Option<Self::Item> {
+        if self.arr.is_valid_unchecked(index) {
+            Some(self.arr.value_unchecked(index))
+        } else {
+            None
+        }
+    }
+}
+
+#[cfg(feature = "dtype-binary")]
+impl<'a> IntoTakeRandom<'a> for &'a BinaryChunked {
+    type Item = &'a [u8];
+    type TakeRandom = TakeRandBranch2<BinaryTakeRandomSingleChunk<'a>, BinaryTakeRandom<'a>>;
+
+    fn take_rand(&self) -> Self::TakeRandom {
+        match self.chunks.len() {
+            1 => {
+                let arr = self.downcast_iter().next().unwrap();
+                let t = BinaryTakeRandomSingleChunk { arr };
+                TakeRandBranch2::Single(t)
+            }
+            _ => {
+                let chunks = self.downcast_chunks();
+                let t = BinaryTakeRandom {
+                    chunks,
+                    chunk_lens: self.chunks.iter().map(|a| a.len() as IdxSize).collect(),
+                };
+                TakeRandBranch2::Multi(t)
+            }
+        }
+    }
+}
+
 impl<'a> IntoTakeRandom<'a> for &'a BooleanChunked {
     type Item = bool;
     type TakeRandom = TakeRandBranch2<BoolTakeRandomSingleChunk<'a>, BoolTakeRandom<'a>>;
