@@ -71,7 +71,11 @@ impl<T> ChunkZip<T> for ChunkedArray<T>
 where
     T: PolarsNumericType,
 {
-    fn zip_with(&self, mask: &BooleanChunked, other: &ChunkedArray<T>) -> Result<ChunkedArray<T>> {
+    fn zip_with(
+        &self,
+        mask: &BooleanChunked,
+        other: &ChunkedArray<T>,
+    ) -> PolarsResult<ChunkedArray<T>> {
         // broadcasting path
         if self.len() != mask.len() || other.len() != mask.len() {
             impl_ternary_broadcast!(self, self.len(), other.len(), other, mask, T)
@@ -86,14 +90,18 @@ where
                     let arr = if_then_else(&mask_c, left_c, right_c)?;
                     Ok(arr)
                 })
-                .collect::<Result<Vec<_>>>()?;
+                .collect::<PolarsResult<Vec<_>>>()?;
             Ok(ChunkedArray::from_chunks(self.name(), chunks))
         }
     }
 }
 
 impl ChunkZip<BooleanType> for BooleanChunked {
-    fn zip_with(&self, mask: &BooleanChunked, other: &BooleanChunked) -> Result<BooleanChunked> {
+    fn zip_with(
+        &self,
+        mask: &BooleanChunked,
+        other: &BooleanChunked,
+    ) -> PolarsResult<BooleanChunked> {
         // broadcasting path
         if self.len() != mask.len() || other.len() != mask.len() {
             impl_ternary_broadcast!(self, self.len(), other.len(), other, mask, BooleanType)
@@ -108,14 +116,14 @@ impl ChunkZip<BooleanType> for BooleanChunked {
                     let arr = if_then_else(&mask_c, left_c, right_c)?;
                     Ok(arr)
                 })
-                .collect::<Result<Vec<_>>>()?;
+                .collect::<PolarsResult<Vec<_>>>()?;
             Ok(ChunkedArray::from_chunks(self.name(), chunks))
         }
     }
 }
 
 impl ChunkZip<Utf8Type> for Utf8Chunked {
-    fn zip_with(&self, mask: &BooleanChunked, other: &Utf8Chunked) -> Result<Utf8Chunked> {
+    fn zip_with(&self, mask: &BooleanChunked, other: &Utf8Chunked) -> PolarsResult<Utf8Chunked> {
         if self.len() != mask.len() || other.len() != mask.len() {
             impl_ternary_broadcast!(self, self.len(), other.len(), other, mask, Utf8Type)
         } else {
@@ -129,17 +137,44 @@ impl ChunkZip<Utf8Type> for Utf8Chunked {
                     let arr = if_then_else(&mask_c, left_c, right_c)?;
                     Ok(arr)
                 })
-                .collect::<Result<Vec<_>>>()?;
+                .collect::<PolarsResult<Vec<_>>>()?;
             Ok(ChunkedArray::from_chunks(self.name(), chunks))
         }
     }
 }
+
+#[cfg(feature = "dtype-binary")]
+impl ChunkZip<BinaryType> for BinaryChunked {
+    fn zip_with(
+        &self,
+        mask: &BooleanChunked,
+        other: &BinaryChunked,
+    ) -> PolarsResult<BinaryChunked> {
+        if self.len() != mask.len() || other.len() != mask.len() {
+            impl_ternary_broadcast!(self, self.len(), other.len(), other, mask, BinaryType)
+        } else {
+            let (left, right, mask) = align_chunks_ternary(self, other, mask);
+            let chunks = left
+                .downcast_iter()
+                .zip(right.downcast_iter())
+                .zip(mask.downcast_iter())
+                .map(|((left_c, right_c), mask_c)| {
+                    let mask_c = prepare_mask(mask_c);
+                    let arr = if_then_else(&mask_c, left_c, right_c)?;
+                    Ok(arr)
+                })
+                .collect::<PolarsResult<Vec<_>>>()?;
+            Ok(ChunkedArray::from_chunks(self.name(), chunks))
+        }
+    }
+}
+
 impl ChunkZip<ListType> for ListChunked {
     fn zip_with(
         &self,
         mask: &BooleanChunked,
         other: &ChunkedArray<ListType>,
-    ) -> Result<ChunkedArray<ListType>> {
+    ) -> PolarsResult<ChunkedArray<ListType>> {
         let (left, right, mask) = align_chunks_ternary(self, other, mask);
         let chunks = left
             .downcast_iter()
@@ -150,7 +185,7 @@ impl ChunkZip<ListType> for ListChunked {
                 let arr = if_then_else(&mask_c, left_c, right_c)?;
                 Ok(arr)
             })
-            .collect::<Result<Vec<_>>>()?;
+            .collect::<PolarsResult<Vec<_>>>()?;
         Ok(ChunkedArray::from_chunks(self.name(), chunks))
     }
 }
@@ -161,7 +196,7 @@ impl<T: PolarsObject> ChunkZip<ObjectType<T>> for ObjectChunked<T> {
         &self,
         mask: &BooleanChunked,
         other: &ChunkedArray<ObjectType<T>>,
-    ) -> Result<ChunkedArray<ObjectType<T>>> {
+    ) -> PolarsResult<ChunkedArray<ObjectType<T>>> {
         let (left, right, mask) = align_chunks_ternary(self, other, mask);
         let mut ca: Self = left
             .as_ref()
