@@ -16,8 +16,7 @@ use crate::chunked_array::builder::{
 #[cfg(feature = "object")]
 use crate::chunked_array::object::ObjectArray;
 use crate::prelude::*;
-use crate::utils::NoNull;
-use crate::utils::{get_iter_capacity, CustomIterTools};
+use crate::utils::{get_iter_capacity, CustomIterTools, NoNull};
 
 impl<T: PolarsDataType> Default for ChunkedArray<T> {
     fn default() -> Self {
@@ -134,6 +133,41 @@ where
     }
 }
 
+// FromIterator for BinaryChunked variants.
+#[cfg(feature = "dtype-binary")]
+impl<Ptr> FromIterator<Option<Ptr>> for BinaryChunked
+where
+    Ptr: AsRef<[u8]>,
+{
+    fn from_iter<I: IntoIterator<Item = Option<Ptr>>>(iter: I) -> Self {
+        let arr = BinaryArray::<i64>::from_iter(iter);
+        Self::from_chunks("", vec![Box::new(arr)])
+    }
+}
+
+#[cfg(feature = "dtype-binary")]
+impl PolarsAsRef<[u8]> for Vec<u8> {}
+
+#[cfg(feature = "dtype-binary")]
+impl PolarsAsRef<[u8]> for &[u8] {}
+
+#[cfg(feature = "dtype-binary")]
+impl PolarsAsRef<[u8]> for &&[u8] {}
+
+#[cfg(feature = "dtype-binary")]
+impl<'a> PolarsAsRef<[u8]> for Cow<'a, [u8]> {}
+
+#[cfg(feature = "dtype-binary")]
+impl<Ptr> FromIterator<Ptr> for BinaryChunked
+where
+    Ptr: PolarsAsRef<[u8]>,
+{
+    fn from_iter<I: IntoIterator<Item = Ptr>>(iter: I) -> Self {
+        let arr = BinaryArray::<i64>::from_iter_values(iter.into_iter());
+        Self::from_chunks("", vec![Box::new(arr)])
+    }
+}
+
 impl<Ptr> FromIterator<Ptr> for ListChunked
 where
     Ptr: Borrow<Series>,
@@ -273,8 +307,7 @@ impl FromIterator<Option<Box<dyn Array>>> for ListChunked {
 #[cfg(feature = "object")]
 impl<T: PolarsObject> FromIterator<Option<T>> for ObjectChunked<T> {
     fn from_iter<I: IntoIterator<Item = Option<T>>>(iter: I) -> Self {
-        use arrow::bitmap::Bitmap;
-        use arrow::bitmap::MutableBitmap;
+        use arrow::bitmap::{Bitmap, MutableBitmap};
 
         let iter = iter.into_iter();
         let size = iter.size_hint().0;
