@@ -17,11 +17,13 @@ pub enum DataType {
     Float64,
     /// String data
     Utf8,
+    #[cfg(feature = "dtype-binary")]
+    Binary,
     /// A 32-bit date representing the elapsed time since UNIX epoch (1970-01-01)
     /// in days (32 bits).
     Date,
     /// A 64-bit date representing the elapsed time since UNIX epoch (1970-01-01)
-    /// in milliseconds (64 bits).
+    /// in the given timeunit (64 bits).
     Datetime(TimeUnit, Option<TimeZone>),
     // 64-bit integer representing difference between times in milliseconds or nanoseconds
     Duration(TimeUnit),
@@ -132,6 +134,21 @@ impl DataType {
         matches!(self, Date | Datetime(_, _) | Duration(_) | Time)
     }
 
+    /// Check if datatype is a primitive type. By that we mean that
+    /// it is not a container type.
+    pub fn is_primitive(&self) -> bool {
+        #[cfg(feature = "dtype-binary")]
+        {
+            self.is_numeric()
+                | matches!(self, DataType::Boolean | DataType::Utf8 | DataType::Binary)
+        }
+
+        #[cfg(not(feature = "dtype-binary"))]
+        {
+            self.is_numeric() | matches!(self, DataType::Boolean | DataType::Utf8)
+        }
+    }
+
     /// Check if this [`DataType`] is a numeric type
     pub fn is_numeric(&self) -> bool {
         // allow because it cannot be replaced when object feature is activated
@@ -144,10 +161,14 @@ impl DataType {
             | DataType::Duration(_)
             | DataType::Boolean
             | DataType::Null => false,
+            #[cfg(feature = "dtype-binary")]
+            DataType::Binary => false,
             #[cfg(feature = "object")]
             DataType::Object(_) => false,
             #[cfg(feature = "dtype-categorical")]
             DataType::Categorical(_) => false,
+            #[cfg(feature = "dtype-struct")]
+            DataType::Struct(_) => false,
             _ => true,
         }
     }
@@ -156,6 +177,10 @@ impl DataType {
         matches!(self, DataType::Float32 | DataType::Float64)
     }
 
+    pub fn is_integer(&self) -> bool {
+        self.is_numeric() && !matches!(self, DataType::Float32 | DataType::Float64)
+    }
+
     pub fn is_signed(&self) -> bool {
         // allow because it cannot be replaced when object feature is activated
         #[allow(clippy::match_like_matches_macro)]
@@ -188,6 +213,8 @@ impl DataType {
             Float32 => ArrowDataType::Float32,
             Float64 => ArrowDataType::Float64,
             Utf8 => ArrowDataType::LargeUtf8,
+            #[cfg(feature = "dtype-binary")]
+            Binary => ArrowDataType::LargeBinary,
             Date => ArrowDataType::Date32,
             Datetime(unit, tz) => ArrowDataType::Timestamp(unit.to_arrow(), tz.clone()),
             Duration(unit) => ArrowDataType::Duration(unit.to_arrow()),
@@ -239,6 +266,8 @@ impl Display for DataType {
             DataType::Float32 => "f32",
             DataType::Float64 => "f64",
             DataType::Utf8 => "str",
+            #[cfg(feature = "dtype-binary")]
+            DataType::Binary => "binary",
             DataType::Date => "date",
             DataType::Datetime(tu, tz) => {
                 let s = match tz {
