@@ -1,21 +1,22 @@
-use super::*;
-use crate::frame::groupby::hashing::HASHMAP_INIT_SIZE;
-use crate::utils::{split_ca, split_df};
-use crate::vector_hasher::{df_rows_to_hashes_threaded, AsU64};
-use crate::POOL;
-use ahash::RandomState;
-use arrow::types::NativeType;
-use num::Zero;
-use rayon::prelude::*;
 use std::fmt::Debug;
 use std::hash::Hash;
 use std::ops::Sub;
 
+use ahash::RandomState;
+use arrow::types::NativeType;
+use num::Zero;
+use rayon::prelude::*;
+
+use super::*;
+use crate::frame::groupby::hashing::HASHMAP_INIT_SIZE;
 #[cfg(feature = "dtype-categorical")]
 use crate::frame::hash_join::check_categorical_src;
 use crate::frame::hash_join::{
     create_probe_table, get_hash_tbl_threaded_join_partitioned, multiple_keys as mk, prepare_strs,
 };
+use crate::utils::{split_ca, split_df};
+use crate::vector_hasher::{df_rows_to_hashes_threaded, AsU64};
+use crate::POOL;
 
 pub(super) unsafe fn join_asof_backward_with_indirection_and_tolerance<
     T: PartialOrd + Copy + Sub<Output = T> + Debug,
@@ -60,6 +61,35 @@ pub(super) unsafe fn join_asof_backward_with_indirection_and_tolerance<
     }
 }
 
+pub(super) unsafe fn join_asof_forward_with_indirection_and_tolerance<
+    T: PartialOrd + Copy + Sub<Output = T> + Debug,
+>(
+    val_l: T,
+    right: &[T],
+    offsets: &[IdxSize],
+    tolerance: T,
+) -> (Option<IdxSize>, usize) {
+    if offsets.is_empty() {
+        return (None, 0);
+    }
+    let last_offset = *offsets.get_unchecked(offsets.len() - 1);
+    let last_value = *right.get_unchecked(last_offset as usize);
+    if val_l <= last_value {
+        for (idx, &offset) in offsets.iter().enumerate() {
+            let val_r = *right.get_unchecked(offset as usize);
+            if val_r >= val_l {
+                let dist = val_r - val_l;
+                return if dist > tolerance {
+                    (None, idx)
+                } else {
+                    (Some(offset), idx)
+                };
+            }
+        }
+    }
+    (None, offsets.len())
+}
+
 pub(super) unsafe fn join_asof_backward_with_indirection<T: PartialOrd + Copy + Debug>(
     val_l: T,
     right: &[T],
@@ -86,6 +116,29 @@ pub(super) unsafe fn join_asof_backward_with_indirection<T: PartialOrd + Copy +
     }
 }
 
+pub(super) unsafe fn join_asof_forward_with_indirection<T: PartialOrd + Copy + Debug>(
+    val_l: T,
+    right: &[T],
+    offsets: &[IdxSize],
+    // only there to have the same function signature
+    _: T,
+) -> (Option<IdxSize>, usize) {
+    if offsets.is_empty() {
+        return (None, 0);
+    }
+    let last_offset = *offsets.get_unchecked(offsets.len() - 1);
+    let last_value = *right.get_unchecked(last_offset as usize);
+    if val_l <= last_value {
+        for (idx, &offset) in offsets.iter().enumerate() {
+            let val_r = *right.get_unchecked(offset as usize);
+            if val_r >= val_l {
+                return (Some(offset), idx);
+            }
+        }
+    }
+    (None, offsets.len())
+}
+
 // process the group taken by the `by` operation and keep track of the offset.
 // we don't process a group at once but per `index_left` we find the `right_index` and keep track
 // of the offsets we have already processed in a separate hashmap. Then on a next iteration we can
@@ -143,7 +196,8 @@ fn asof_join_by_numeric<T, S>(
     left_asof: &ChunkedArray<T>,
     right_asof: &ChunkedArray<T>,
     tolerance: Option<AnyValue<'static>>,
-) -> Vec<Option<IdxSize>>
+    strategy: AsofStrategy,
+) -> PolarsResult<Vec<Option<IdxSize>>>
 where
     T: PolarsNumericType,
     S: PolarsNumericType,
@@ -153,18 +207,27 @@ where
     let (join_asof_fn, tolerance): (
         unsafe fn(T::Native, &[T::Native], &[IdxSize], T::Native) -> (Option<IdxSize>, usize),
         _,
-    ) = match tolerance {
-        Some(tolerance) => {
+    ) = match (tolerance, strategy) {
+        (Some(tolerance), AsofStrategy::Backward) => {
             let tol = tolerance.extract::<T::Native>().unwrap();
             (join_asof_backward_with_indirection_and_tolerance, tol)
         }
-        None => (join_asof_backward_with_indirection, T::Native::zero()),
+        (None, AsofStrategy::Backward) => (join_asof_backward_with_indirection, T::Native::zero()),
+        (Some(tolerance), AsofStrategy::Forward) => {
+            let tol = tolerance.extract::<T::Native>().unwrap();
+            (join_asof_forward_with_indirection_and_tolerance, tol)
+        }
+        (None, AsofStrategy::Forward) => (join_asof_forward_with_indirection, T::Native::zero()),
     };
+
     let left_asof = left_asof.rechunk();
-    let left_asof = left_asof.cont_slice().unwrap();
+    let err = |_: PolarsError| {
+        PolarsError::ComputeError("Keys are not allowed to have null values in asof join.".into())
+    };
+    let left_asof = left_asof.cont_slice().map_err(err)?;
 
     let right_asof = right_asof.rechunk();
-    let right_asof = right_asof.cont_slice().unwrap();
+    let right_asof = right_asof.cont_slice().map_err(err)?;
 
     let n_threads = POOL.current_num_threads();
     let splitted_left = split_ca(by_left, n_threads).unwrap();
@@ -196,7 +259,7 @@ where
     debug_assert!(n_tables.is_power_of_two());
 
     // next we probe the right relation
-    POOL.install(|| {
+    Ok(POOL.install(|| {
         vals_left
             .into_par_iter()
             .zip(offsets)
@@ -244,7 +307,7 @@ where
             })
             .flatten()
             .collect()
-    })
+    }))
 }
 
 fn asof_join_by_utf8<T>(
@@ -253,6 +316,7 @@ fn asof_join_by_utf8<T>(
     left_asof: &ChunkedArray<T>,
     right_asof: &ChunkedArray<T>,
     tolerance: Option<AnyValue<'static>>,
+    strategy: AsofStrategy,
 ) -> Vec<Option<IdxSize>>
 where
     T: PolarsNumericType,
@@ -261,12 +325,17 @@ where
     let (join_asof_fn, tolerance): (
         unsafe fn(T::Native, &[T::Native], &[IdxSize], T::Native) -> (Option<IdxSize>, usize),
         _,
-    ) = match tolerance {
-        Some(tolerance) => {
+    ) = match (tolerance, strategy) {
+        (Some(tolerance), AsofStrategy::Backward) => {
             let tol = tolerance.extract::<T::Native>().unwrap();
             (join_asof_backward_with_indirection_and_tolerance, tol)
         }
-        None => (join_asof_backward_with_indirection, T::Native::zero()),
+        (None, AsofStrategy::Backward) => (join_asof_backward_with_indirection, T::Native::zero()),
+        (Some(tolerance), AsofStrategy::Forward) => {
+            let tol = tolerance.extract::<T::Native>().unwrap();
+            (join_asof_forward_with_indirection_and_tolerance, tol)
+        }
+        (None, AsofStrategy::Forward) => (join_asof_forward_with_indirection, T::Native::zero()),
     };
 
     let left_asof = left_asof.rechunk();
@@ -354,11 +423,12 @@ where
 // TODO! optimize this. This does a full scan backwards. Use the same strategy as in the single `by`
 // implementations
 fn asof_join_by_multiple<T>(
-    a: &DataFrame,
-    b: &DataFrame,
+    a: &mut DataFrame,
+    b: &mut DataFrame,
     left_asof: &ChunkedArray<T>,
     right_asof: &ChunkedArray<T>,
     tolerance: Option<AnyValue<'static>>,
+    strategy: AsofStrategy,
 ) -> Vec<Option<IdxSize>>
 where
     T: PolarsNumericType,
@@ -367,12 +437,17 @@ where
     let (join_asof_fn, tolerance): (
         unsafe fn(T::Native, &[T::Native], &[IdxSize], T::Native) -> (Option<IdxSize>, usize),
         _,
-    ) = match tolerance {
-        Some(tolerance) => {
+    ) = match (tolerance, strategy) {
+        (Some(tolerance), AsofStrategy::Backward) => {
             let tol = tolerance.extract::<T::Native>().unwrap();
             (join_asof_backward_with_indirection_and_tolerance, tol)
         }
-        None => (join_asof_backward_with_indirection, T::Native::zero()),
+        (None, AsofStrategy::Backward) => (join_asof_backward_with_indirection, T::Native::zero()),
+        (Some(tolerance), AsofStrategy::Forward) => {
+            let tol = tolerance.extract::<T::Native>().unwrap();
+            (join_asof_forward_with_indirection_and_tolerance, tol)
+        }
+        (None, AsofStrategy::Forward) => (join_asof_forward_with_indirection, T::Native::zero()),
     };
     let left_asof = left_asof.rechunk();
     let left_asof = left_asof.cont_slice().unwrap();
@@ -384,8 +459,8 @@ where
     let dfs_a = split_df(a, n_threads).unwrap();
     let dfs_b = split_df(b, n_threads).unwrap();
 
-    let (build_hashes, random_state) = df_rows_to_hashes_threaded(&dfs_b, None);
-    let (probe_hashes, _) = df_rows_to_hashes_threaded(&dfs_a, Some(random_state));
+    let (build_hashes, random_state) = df_rows_to_hashes_threaded(&dfs_b, None).unwrap();
+    let (probe_hashes, _) = df_rows_to_hashes_threaded(&dfs_a, Some(random_state)).unwrap();
 
     let hash_tbls = mk::create_probe_table(&build_hashes, b);
     // early drop to reduce memory pressure
@@ -470,24 +545,21 @@ impl DataFrame {
         strategy: AsofStrategy,
         tolerance: Option<AnyValue<'static>>,
         slice: Option<(i64, usize)>,
-    ) -> Result<DataFrame>
+    ) -> PolarsResult<DataFrame>
     where
         I: IntoIterator<Item = S>,
         S: AsRef<str>,
     {
-        if let AsofStrategy::Forward = strategy {
-            panic!("forward strategy + groupby not yet implemented");
-        }
-
         use DataType::*;
         let left_asof = self.column(left_on)?;
         let right_asof = other.column(right_on)?;
         let right_asof_name = right_asof.name();
+        let left_asof_name = left_asof.name();
 
         check_asof_columns(left_asof, right_asof)?;
 
-        let left_by = self.select(left_by)?;
-        let right_by = other.select(right_by)?;
+        let mut left_by = self.select(left_by)?;
+        let mut right_by = other.select(right_by)?;
 
         let left_by_s = &left_by.get_columns()[0];
         let right_by_s = &right_by.get_columns()[0];
@@ -507,20 +579,21 @@ impl DataFrame {
                         left_asof,
                         right_asof,
                         tolerance,
+                        strategy,
                     ),
                     _ => {
                         if left_by_s.bit_repr_is_large() {
                             let left_by = left_by_s.bit_repr_large();
                             let right_by = right_by_s.bit_repr_large();
                             asof_join_by_numeric(
-                                &left_by, &right_by, left_asof, right_asof, tolerance,
-                            )
+                                &left_by, &right_by, left_asof, right_asof, tolerance, strategy,
+                            )?
                         } else {
                             let left_by = left_by_s.bit_repr_small();
                             let right_by = right_by_s.bit_repr_small();
                             asof_join_by_numeric(
-                                &left_by, &right_by, left_asof, right_asof, tolerance,
-                            )
+                                &left_by, &right_by, left_asof, right_asof, tolerance, strategy,
+                            )?
                         }
                     }
                 }
@@ -530,7 +603,14 @@ impl DataFrame {
                     #[cfg(feature = "dtype-categorical")]
                     check_categorical_src(lhs.dtype(), rhs.dtype())?;
                 }
-                asof_join_by_multiple(&left_by, &right_by, left_asof, right_asof, tolerance)
+                asof_join_by_multiple(
+                    &mut left_by,
+                    &mut right_by,
+                    left_asof,
+                    right_asof,
+                    tolerance,
+                    strategy,
+                )
             }
         } else {
             // we cannot use bit repr as that loses ordering
@@ -547,30 +627,40 @@ impl DataFrame {
                         left_asof,
                         right_asof,
                         tolerance,
+                        strategy,
                     ),
                     _ => {
                         if left_by_s.bit_repr_is_large() {
                             let left_by = left_by_s.bit_repr_large();
                             let right_by = right_by_s.bit_repr_large();
                             asof_join_by_numeric(
-                                &left_by, &right_by, left_asof, right_asof, tolerance,
-                            )
+                                &left_by, &right_by, left_asof, right_asof, tolerance, strategy,
+                            )?
                         } else {
                             let left_by = left_by_s.bit_repr_small();
                             let right_by = right_by_s.bit_repr_small();
                             asof_join_by_numeric(
-                                &left_by, &right_by, left_asof, right_asof, tolerance,
-                            )
+                                &left_by, &right_by, left_asof, right_asof, tolerance, strategy,
+                            )?
                         }
                     }
                 }
             } else {
-                asof_join_by_multiple(&left_by, &right_by, left_asof, right_asof, tolerance)
+                asof_join_by_multiple(
+                    &mut left_by,
+                    &mut right_by,
+                    left_asof,
+                    right_asof,
+                    tolerance,
+                    strategy,
+                )
             }
         };
 
         let mut drop_these = right_by.get_column_names();
-        drop_these.push(right_asof_name);
+        if left_asof_name == right_asof_name {
+            drop_these.push(right_asof_name);
+        }
 
         let cols = other
             .get_columns()
@@ -603,7 +693,7 @@ impl DataFrame {
             )
         };
 
-        self.finish_join(left, right_df, None)
+        _finish_join(left, right_df, None)
     }
 
     /// This is similar to a left-join except that we match on nearest key rather than equal keys.
@@ -620,7 +710,7 @@ impl DataFrame {
         right_by: I,
         strategy: AsofStrategy,
         tolerance: Option<AnyValue<'static>>,
-    ) -> Result<DataFrame>
+    ) -> PolarsResult<DataFrame>
     where
         I: IntoIterator<Item = S>,
         S: AsRef<str>,
@@ -636,7 +726,7 @@ mod test {
     use super::*;
 
     #[test]
-    fn test_asof_by() -> Result<()> {
+    fn test_asof_by() -> PolarsResult<()> {
         let a = df![
         "a" => [-1, 2, 3, 3, 3, 4],
         "b" => ["a", "b", "c", "d", "e", "f"]
@@ -660,7 +750,7 @@ mod test {
     }
 
     #[test]
-    fn test_asof_by2() -> Result<()> {
+    fn test_asof_by2() -> PolarsResult<()> {
         let trades = df![
             "time" => [23i64, 38, 48, 48, 48],
             "ticker" => ["MSFT", "MSFT", "GOOG", "GOOG", "AAPL"],
@@ -714,4 +804,143 @@ mod test {
 
         Ok(())
     }
+
+    #[test]
+    fn test_asof_by3() -> PolarsResult<()> {
+        let a = df![
+        "a" => [ -1,   2,   2,   3,   3,   3,   4],
+        "b" => ["a", "a", "b", "c", "d", "e", "f"]
+        ]?;
+
+        let b = df![
+                 "a" => [  1,   3,   2,   3,   2],
+                 "b" => ["a", "a", "b", "c", "d"],
+        "right_vals" => [  1,   3,   2,   3,   4]
+        ]?;
+
+        let out = a.join_asof_by(&b, "a", "a", ["b"], ["b"], AsofStrategy::Forward, None)?;
+        assert_eq!(out.get_column_names(), &["a", "b", "right_vals"]);
+        let out = out.column("right_vals").unwrap();
+        let out = out.i32().unwrap();
+        assert_eq!(
+            Vec::from(out),
+            &[Some(1), Some(3), Some(2), Some(3), None, None, None]
+        );
+
+        let out = a.join_asof_by(
+            &b,
+            "a",
+            "a",
+            ["b"],
+            ["b"],
+            AsofStrategy::Forward,
+            Some(AnyValue::Int32(1)),
+        )?;
+        assert_eq!(out.get_column_names(), &["a", "b", "right_vals"]);
+        let out = out.column("right_vals").unwrap();
+        let out = out.i32().unwrap();
+        assert_eq!(
+            Vec::from(out),
+            &[None, Some(3), Some(2), Some(3), None, None, None]
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_asof_by4() -> PolarsResult<()> {
+        let trades = df![
+            "time" =>    [23i64,     38,     48,     48,     48],
+            "ticker" => ["MSFT", "MSFT", "GOOG", "GOOG", "AAPL"],
+            "groups_numeric" => [1, 1, 2, 2, 3],
+            "bid" => [51.95, 51.95, 720.77, 720.92, 98.0]
+        ]?;
+
+        let quotes = df![
+            "time" =>    [23i64,     23,     30,     41,     48,     49,     72,     75],
+            "ticker" => ["GOOG", "MSFT", "MSFT", "MSFT", "GOOG", "AAPL", "GOOG", "MSFT"],
+            "bid" =>     [720.5,  51.95,  51.97,  51.99,  720.5,  97.99,  720.5,  52.01],
+            "groups_numeric" => [2, 1, 1, 1, 2, 3, 2, 1],
+
+        ]?;
+        /*
+        trades:
+        shape: (5, 4)
+        ┌──────┬────────┬────────────────┬────────┐
+        │ time ┆ ticker ┆ groups_numeric ┆ bid    │
+        │ ---  ┆ ---    ┆ ---            ┆ ---    │
+        │ i64  ┆ str    ┆ i32            ┆ f64    │
+        ╞══════╪════════╪════════════════╪════════╡
+        │ 23   ┆ MSFT   ┆ 1              ┆ 51.95  │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 38   ┆ MSFT   ┆ 1              ┆ 51.95  │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 48   ┆ GOOG   ┆ 2              ┆ 720.77 │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 48   ┆ GOOG   ┆ 2              ┆ 720.92 │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 48   ┆ AAPL   ┆ 3              ┆ 98.0   │
+        └──────┴────────┴────────────────┴────────┘
+        quotes:
+        shape: (8, 4)
+        ┌──────┬────────┬───────┬────────────────┐
+        │ time ┆ ticker ┆ bid   ┆ groups_numeric │
+        │ ---  ┆ ---    ┆ ---   ┆ ---            │
+        │ i64  ┆ str    ┆ f64   ┆ i32            │
+        ╞══════╪════════╪═══════╪════════════════╡
+        │ 23   ┆ GOOG   ┆ 720.5 ┆ 2              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 23   ┆ MSFT   ┆ 51.95 ┆ 1              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 30   ┆ MSFT   ┆ 51.97 ┆ 1              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 41   ┆ MSFT   ┆ 51.99 ┆ 1              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 48   ┆ GOOG   ┆ 720.5 ┆ 2              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 49   ┆ AAPL   ┆ 97.99 ┆ 3              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 72   ┆ GOOG   ┆ 720.5 ┆ 2              │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ 75   ┆ MSFT   ┆ 52.01 ┆ 1              │
+        └──────┴────────┴───────┴────────────────┘
+        */
+
+        let out = trades.join_asof_by(
+            &quotes,
+            "time",
+            "time",
+            ["ticker"],
+            ["ticker"],
+            AsofStrategy::Forward,
+            None,
+        )?;
+        let a = out.column("bid_right").unwrap();
+        let a = a.f64().unwrap();
+        let expected = &[
+            Some(51.95),
+            Some(51.99),
+            Some(720.5),
+            Some(720.5),
+            Some(97.99),
+        ];
+
+        assert_eq!(Vec::from(a), expected);
+
+        let out = trades.join_asof_by(
+            &quotes,
+            "time",
+            "time",
+            ["groups_numeric"],
+            ["groups_numeric"],
+            AsofStrategy::Forward,
+            None,
+        )?;
+        let a = out.column("bid_right").unwrap();
+        let a = a.f64().unwrap();
+
+        assert_eq!(Vec::from(a), expected);
+
+        Ok(())
+    }
 }
