@@ -10,7 +10,7 @@ pub(super) mod sort_merge;
 
 #[cfg(feature = "chunked_ids")]
 use std::borrow::Cow;
-use std::fmt::Debug;
+use std::fmt::{Debug, Display, Formatter};
 use std::hash::{BuildHasher, Hash, Hasher};
 
 use ahash::RandomState;
@@ -45,7 +45,7 @@ use crate::frame::hash_join::multiple_keys::{
 use crate::frame::hash_join::multiple_keys::{left_anti_multiple_keys, left_semi_multiple_keys};
 use crate::prelude::*;
 use crate::utils::series::to_physical_and_bit_repr;
-use crate::utils::{set_partition_size, slice_slice, split_ca};
+use crate::utils::{_set_partition_size, slice_slice, split_ca};
 use crate::vector_hasher::{
     create_hash_and_keys_threaded_vectorized, prepare_hashed_relation_threaded, this_partition,
     AsU64, BytesHash,
@@ -65,6 +65,7 @@ pub type JoinOptIds = Vec<Option<IdxSize>>;
 #[cfg(not(feature = "chunked_ids"))]
 pub type JoinIds = Vec<IdxSize>;
 
+/// [ChunkIdx, DfIdx]
 pub type ChunkId = [IdxSize; 2];
 
 pub fn default_join_ids() -> JoinOptIds {
@@ -97,6 +98,8 @@ macro_rules! det_hash_prone_order {
 }
 
 pub(super) use det_hash_prone_order;
+#[cfg(feature = "performant")]
+use polars_arrow::conversion::primitive_to_vec;
 
 use crate::series::IsSorted;
 
@@ -107,7 +110,8 @@ pub(crate) fn check_categorical_src(l: &DataType, r: &DataType) -> PolarsResult<
     match (l, r) {
         (DataType::Categorical(Some(l)), DataType::Categorical(Some(r))) => {
             if !l.same_src(r) {
-                return Err(PolarsError::ComputeError("joins/or comparisons on categorical dtypes can only happen if they are created under the same global string cache".into()));
+                return Err(PolarsError::ComputeError("Joins/or comparisons on categorical dtypes can only happen if they are created under the same global string cache.\
+                  Hint: set a global StringCache".into()));
             }
             Ok(())
         }
@@ -115,14 +119,13 @@ pub(crate) fn check_categorical_src(l: &DataType, r: &DataType) -> PolarsResult<
     }
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, PartialEq, Eq)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub enum JoinType {
     Left,
     Inner,
     Outer,
     #[cfg(feature = "asof_join")]
-    #[cfg_attr(feature = "serde", serde(skip))]
     AsOf(AsOfOptions),
     Cross,
     #[cfg(feature = "semi_anti_join")]
@@ -131,6 +134,31 @@ pub enum JoinType {
     Anti,
 }
 
+impl Display for JoinType {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        use JoinType::*;
+        let val = match self {
+            Left => "LEFT",
+            Inner => "INNER",
+            Outer => "OUTER",
+            #[cfg(feature = "asof_join")]
+            AsOf(_) => "ASOF",
+            Cross => "CROSS",
+            #[cfg(feature = "semi_anti_join")]
+            Semi => "SEMI",
+            #[cfg(feature = "semi_anti_join")]
+            Anti => "ANTI",
+        };
+        write!(f, "{}", val)
+    }
+}
+
+impl Debug for JoinType {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self)
+    }
+}
+
 pub(crate) unsafe fn get_hash_tbl_threaded_join_partitioned<Item>(
     h: u64,
     hash_tables: &[Item],
@@ -233,6 +261,8 @@ macro_rules! impl_zip_outer_join {
 }
 impl_zip_outer_join!(BooleanChunked);
 impl_zip_outer_join!(Utf8Chunked);
+#[cfg(feature = "dtype-binary")]
+impl_zip_outer_join!(BinaryChunked);
 
 impl ZipOuterJoinColumn for Float32Chunked {
     fn zip_outer_join_column(
@@ -264,38 +294,38 @@ impl ZipOuterJoinColumn for Float64Chunked {
     }
 }
 
-impl DataFrame {
-    /// Utility method to finish a join.
-    pub(crate) fn finish_join(
-        &self,
-        mut df_left: DataFrame,
-        mut df_right: DataFrame,
-        suffix: Option<String>,
-    ) -> PolarsResult<DataFrame> {
-        let mut left_names = PlHashSet::with_capacity(df_left.width());
+/// Utility method to finish a join.
+#[doc(hidden)]
+pub fn _finish_join(
+    mut df_left: DataFrame,
+    mut df_right: DataFrame,
+    suffix: Option<&str>,
+) -> PolarsResult<DataFrame> {
+    let mut left_names = PlHashSet::with_capacity(df_left.width());
 
-        df_left.columns.iter().for_each(|series| {
-            left_names.insert(series.name());
-        });
+    df_left.columns.iter().for_each(|series| {
+        left_names.insert(series.name());
+    });
 
-        let mut rename_strs = Vec::with_capacity(df_right.width());
+    let mut rename_strs = Vec::with_capacity(df_right.width());
 
-        df_right.columns.iter().for_each(|series| {
-            if left_names.contains(series.name()) {
-                rename_strs.push(series.name().to_owned())
-            }
-        });
-        let suffix = suffix.as_deref().unwrap_or("_right");
-
-        for name in rename_strs {
-            df_right.rename(&name, &format!("{}{}", name, suffix))?;
+    df_right.columns.iter().for_each(|series| {
+        if left_names.contains(series.name()) {
+            rename_strs.push(series.name().to_owned())
         }
+    });
+    let suffix = suffix.unwrap_or("_right");
 
-        drop(left_names);
-        df_left.hstack_mut(&df_right.columns)?;
-        Ok(df_left)
+    for name in rename_strs {
+        df_right.rename(&name, &format!("{}{}", name, suffix))?;
     }
 
+    drop(left_names);
+    df_left.hstack_mut(&df_right.columns)?;
+    Ok(df_left)
+}
+
+impl DataFrame {
     /// # Safety
     /// Join tuples must be in bounds
     #[cfg(feature = "chunked_ids")]
@@ -350,17 +380,23 @@ impl DataFrame {
     ) -> PolarsResult<DataFrame> {
         #[cfg(feature = "cross_join")]
         if let JoinType::Cross = how {
-            return self.cross_join(other, suffix, slice);
+            return self.cross_join(other, suffix.as_deref(), slice);
         }
 
         #[cfg(feature = "chunked_ids")]
         {
-            if _check_rechunk {
+            // a left join create chunked-ids
+            // the others not yet.
+            // TODO! change this to other join types once they support chunked-id joins
+            if _check_rechunk
+                && !(matches!(how, JoinType::Left)
+                    || std::env::var("POLARS_NO_CHUNKED_JOIN").is_ok())
+            {
                 let mut left = Cow::Borrowed(self);
                 let mut right = Cow::Borrowed(other);
                 if self.should_rechunk() {
                     if _verbose {
-                        eprintln!("join triggered a rechunk of the left dataframe: {} columns are affected", self.width());
+                        eprintln!("{:?} join triggered a rechunk of the left dataframe: {} columns are affected", how, self.width());
                     }
 
                     let mut tmp_left = self.clone();
@@ -369,7 +405,7 @@ impl DataFrame {
                 }
                 if other.should_rechunk() {
                     if _verbose {
-                        eprintln!("join triggered a rechunk of the right dataframe: {} columns are affected", other.width());
+                        eprintln!("{:?} join triggered a rechunk of the right dataframe: {} columns are affected", how, other.width());
                     }
                     let mut tmp_right = other.clone();
                     tmp_right.as_single_chunk_par();
@@ -383,7 +419,7 @@ impl DataFrame {
                     suffix,
                     slice,
                     false,
-                    false,
+                    _verbose,
                 );
             }
         }
@@ -412,9 +448,11 @@ impl DataFrame {
             let s_right = other.column(selected_right[0].name())?;
             return match how {
                 JoinType::Inner => {
-                    self.inner_join_from_series(other, s_left, s_right, suffix, slice)
+                    self.inner_join_from_series(other, s_left, s_right, suffix, slice, _verbose)
+                }
+                JoinType::Left => {
+                    self.left_join_from_series(other, s_left, s_right, suffix, slice, _verbose)
                 }
-                JoinType::Left => self.left_join_from_series(other, s_left, s_right, suffix, slice),
                 JoinType::Outer => {
                     self.outer_join_from_series(other, s_left, s_right, suffix, slice)
                 }
@@ -505,7 +543,7 @@ impl DataFrame {
                             ._take_unchecked_slice(join_idx_right, true)
                     },
                 );
-                self.finish_join(df_left, df_right, suffix)
+                _finish_join(df_left, df_right, suffix.as_deref())
             }
             JoinType::Left => {
                 let mut left = DataFrame::new_no_checks(selected_left_physical);
@@ -553,7 +591,7 @@ impl DataFrame {
                 }
                 keys.extend_from_slice(df_left.get_columns());
                 let df_left = DataFrame::new_no_checks(keys);
-                self.finish_join(df_left, df_right, suffix)
+                _finish_join(df_left, df_right, suffix.as_deref())
             }
             #[cfg(feature = "asof_join")]
             JoinType::AsOf(_) => Err(PolarsError::ComputeError(
@@ -626,7 +664,7 @@ impl DataFrame {
     {
         #[cfg(feature = "cross_join")]
         if let JoinType::Cross = how {
-            return self.cross_join(other, suffix, None);
+            return self.cross_join(other, suffix.as_deref(), None);
         }
         let selected_left = self.select_series(left_on)?;
         let selected_right = other.select_series(right_on)?;
@@ -672,22 +710,12 @@ impl DataFrame {
         s_right: &Series,
         suffix: Option<String>,
         slice: Option<(i64, usize)>,
+        verbose: bool,
     ) -> PolarsResult<DataFrame> {
         #[cfg(feature = "dtype-categorical")]
         check_categorical_src(s_left.dtype(), s_right.dtype())?;
-
-        let ((join_tuples_left, join_tuples_right), sorted) = if use_sort_merge(s_left, s_right) {
-            #[cfg(feature = "performant")]
-            {
-                (par_sorted_merge_inner(s_left, s_right), true)
-            }
-            #[cfg(not(feature = "performant"))]
-            {
-                s_left.hash_join_inner(s_right)
-            }
-        } else {
-            s_left.hash_join_inner(s_right)
-        };
+        let ((join_tuples_left, join_tuples_right), sorted) =
+            sort_or_hash_inner(s_left, s_right, verbose);
 
         let mut join_tuples_left = &*join_tuples_left;
         let mut join_tuples_right = &*join_tuples_right;
@@ -707,7 +735,7 @@ impl DataFrame {
                     ._take_unchecked_slice(join_tuples_right, true)
             },
         );
-        self.finish_join(df_left, df_right, suffix)
+        _finish_join(df_left, df_right, suffix.as_deref())
     }
 
     /// Perform a left join on two DataFrames
@@ -787,7 +815,7 @@ impl DataFrame {
         };
         let (df_left, df_right) = POOL.join(materialize_left, materialize_right);
 
-        self.finish_join(df_left, df_right, suffix)
+        _finish_join(df_left, df_right, suffix.as_deref())
     }
 
     #[cfg(feature = "chunked_ids")]
@@ -838,7 +866,7 @@ impl DataFrame {
         };
         let (df_left, df_right) = POOL.join(materialize_left, materialize_right);
 
-        self.finish_join(df_left, df_right, suffix)
+        _finish_join(df_left, df_right, suffix.as_deref())
     }
 
     pub(crate) fn left_join_from_series(
@@ -848,33 +876,26 @@ impl DataFrame {
         s_right: &Series,
         suffix: Option<String>,
         slice: Option<(i64, usize)>,
+        verbose: bool,
     ) -> PolarsResult<DataFrame> {
         #[cfg(feature = "dtype-categorical")]
         check_categorical_src(s_left.dtype(), s_right.dtype())?;
 
-        let ids = if use_sort_merge(s_left, s_right) {
-            #[cfg(feature = "performant")]
-            {
-                let (left_idx, right_idx) = par_sorted_merge_left(s_left, s_right);
-                #[cfg(feature = "chunked_ids")]
-                {
-                    (Either::Left(left_idx), Either::Left(right_idx))
-                }
-
-                #[cfg(not(feature = "chunked_ids"))]
-                {
-                    (left_idx, right_idx)
-                }
-            }
-            #[cfg(not(feature = "performant"))]
-            {
-                s_left.hash_join_left(s_right)
-            }
-        } else {
-            s_left.hash_join_left(s_right)
-        };
-
-        self.finish_left_join(ids, &other.drop(s_right.name()).unwrap(), suffix, slice)
+        // ensure that the chunks are aligned otherwise we go OOB
+        let mut left = self.clone();
+        let mut s_left = s_left.clone();
+        let mut right = other.clone();
+        let mut s_right = s_right.clone();
+        if left.should_rechunk() {
+            left.as_single_chunk_par();
+            s_left = s_left.rechunk();
+        }
+        if right.should_rechunk() {
+            right.as_single_chunk_par();
+            s_right = s_right.rechunk();
+        }
+        let ids = sort_or_hash_left(&s_left, &s_right, verbose);
+        left.finish_left_join(ids, &right.drop(s_right.name()).unwrap(), suffix, slice)
     }
 
     #[cfg(feature = "semi_anti_join")]
@@ -995,7 +1016,7 @@ impl DataFrame {
         };
 
         df_left.get_columns_mut().insert(join_column_index, s);
-        self.finish_join(df_left, df_right, suffix)
+        _finish_join(df_left, df_right, suffix.as_deref())
     }
 }
 
@@ -1003,6 +1024,8 @@ impl DataFrame {
 mod test {
     use crate::df;
     use crate::prelude::*;
+    #[cfg(feature = "dtype-categorical")]
+    use crate::{reset_string_cache, IUseStringCache};
 
     fn create_frames() -> (DataFrame, DataFrame) {
         let s0 = Series::new("days", &[0, 1, 2]);
@@ -1198,14 +1221,12 @@ mod test {
             .join(&df_b, ["a", "b"], ["foo", "bar"], JoinType::Left, None)
             .unwrap();
         let ca = joined.column("ham").unwrap().utf8().unwrap();
-        dbg!(&df_a, &df_b);
         assert_eq!(Vec::from(ca), correct_ham);
         let joined_inner_hack = df_a.inner_join(&df_b, ["dummy"], ["dummy"]).unwrap();
         let joined_inner = df_a
             .join(&df_b, ["a", "b"], ["foo", "bar"], JoinType::Inner, None)
             .unwrap();
 
-        dbg!(&joined_inner_hack, &joined_inner);
         assert!(joined_inner_hack
             .column("ham")
             .unwrap()
@@ -1265,8 +1286,8 @@ mod test {
         df_a.try_apply("b", |s| s.cast(&DataType::Categorical(None)))
             .unwrap();
         // create a new cache
-        toggle_string_cache(false);
-        toggle_string_cache(true);
+        reset_string_cache();
+        let sc = IUseStringCache::new();
 
         df_b.try_apply("bar", |s| s.cast(&DataType::Categorical(None)))
             .unwrap();
