@@ -259,7 +259,7 @@ fn is_nested_null(av: &AnyValue) -> bool {
     }
 }
 
-// nested dtypes that are all null, will be set as null leaf dtype
+// nested dtypes that are all null, will be set as null leave dtype
 fn infer_dtype_dynamic(av: &AnyValue) -> DataType {
     match av {
         AnyValue::List(s) if s.null_count() == s.len() => DataType::List(Box::new(DataType::Null)),
@@ -379,6 +379,8 @@ pub enum AnyValueBuffer<'a> {
         TimeUnit,
         Option<TimeZone>,
     ),
+    #[cfg(feature = "dtype-duration")]
+    Duration(PrimitiveChunkedBuilder<Int64Type>, TimeUnit),
     #[cfg(feature = "dtype-time")]
     Time(PrimitiveChunkedBuilder<Int64Type>),
     Float32(PrimitiveChunkedBuilder<Float32Type>),
@@ -419,6 +421,13 @@ impl<'a> AnyValueBuffer<'a> {
                 let v = convert_time_units(v, tu_r, *tu_l);
                 builder.append_value(v)
             }
+            #[cfg(feature = "dtype-duration")]
+            (Duration(builder, _), AnyValue::Null) => builder.append_null(),
+            #[cfg(feature = "dtype-duration")]
+            (Duration(builder, tu_l), AnyValue::Duration(v, tu_r)) => {
+                let v = convert_time_units(v, tu_r, *tu_l);
+                builder.append_value(v)
+            }
             #[cfg(feature = "dtype-time")]
             (Time(builder), AnyValue::Time(v)) => builder.append_value(v),
             #[cfg(feature = "dtype-time")]
@@ -428,13 +437,13 @@ impl<'a> AnyValueBuffer<'a> {
             (Float32(builder), val) => builder.append_value(val.extract()?),
             (Float64(builder), val) => builder.append_value(val.extract()?),
             (Utf8(builder), AnyValue::Utf8(v)) => builder.append_value(v),
+            (Utf8(builder), AnyValue::Utf8Owned(v)) => builder.append_value(v),
             (Utf8(builder), AnyValue::Null) => builder.append_null(),
             // Struct and List can be recursive so use anyvalues for that
             (All(_, vals), v) => vals.push(v),
 
             // dynamic types
             (Utf8(builder), av) => match av {
-                AnyValue::Utf8(v) => builder.append_value(v),
                 AnyValue::Int64(v) => builder.append_value(&format!("{}", v)),
                 AnyValue::Float64(v) => builder.append_value(&format!("{}", v)),
                 AnyValue::Boolean(true) => builder.append_value("true"),
@@ -465,6 +474,8 @@ impl<'a> AnyValueBuffer<'a> {
             Date(b) => b.finish().into_date().into_series(),
             #[cfg(feature = "dtype-datetime")]
             Datetime(b, tu, tz) => b.finish().into_datetime(tu, tz).into_series(),
+            #[cfg(feature = "dtype-duration")]
+            Duration(b, tu) => b.finish().into_duration(tu).into_series(),
             #[cfg(feature = "dtype-time")]
             Time(b) => b.finish().into_time().into_series(),
             Float32(b) => b.finish().into_series(),
@@ -496,6 +507,8 @@ impl From<(&DataType, usize)> for AnyValueBuffer<'_> {
             Datetime(tu, tz) => {
                 AnyValueBuffer::Datetime(PrimitiveChunkedBuilder::new("", len), *tu, tz.clone())
             }
+            #[cfg(feature = "dtype-duration")]
+            Duration(tu) => AnyValueBuffer::Duration(PrimitiveChunkedBuilder::new("", len), *tu),
             #[cfg(feature = "dtype-time")]
             Time => AnyValueBuffer::Time(PrimitiveChunkedBuilder::new("", len)),
             Float32 => AnyValueBuffer::Float32(PrimitiveChunkedBuilder::new("", len)),
