@@ -24,11 +24,10 @@ impl Debug for Schema {
 
 impl<I, J> From<I> for Schema
 where
-    I: IntoIterator<Item = J>,
+    I: Iterator<Item = J>,
     J: Into<Field>,
 {
-    fn from(flds: I) -> Self {
-        let iter = flds.into_iter();
+    fn from(iter: I) -> Self {
         let mut map: PlIndexMap<_, _> =
             IndexMap::with_capacity_and_hasher(iter.size_hint().0, ahash::RandomState::default());
         for fld in iter {
@@ -44,15 +43,15 @@ where
     J: Into<Field>,
 {
     fn from_iter<I: IntoIterator<Item = J>>(iter: I) -> Self {
-        Schema::from(iter)
+        Schema::from(iter.into_iter())
     }
 }
 
 impl Schema {
     // could not implement TryFrom
-    pub fn try_from_fallible<I>(flds: I) -> Result<Self>
+    pub fn try_from_fallible<I>(flds: I) -> PolarsResult<Self>
     where
-        I: IntoIterator<Item = Result<Field>>,
+        I: IntoIterator<Item = PolarsResult<Field>>,
     {
         let iter = flds.into_iter();
         let mut map: PlIndexMap<_, _> =
@@ -114,6 +113,11 @@ impl Schema {
         self.inner.get(name)
     }
 
+    pub fn try_get(&self, name: &str) -> PolarsResult<&DataType> {
+        self.get(name)
+            .ok_or_else(|| PolarsError::NotFound(name.to_string().into()))
+    }
+
     pub fn get_full(&self, name: &str) -> Option<(usize, &String, &DataType)> {
         self.inner.get_full(name)
     }
@@ -124,10 +128,21 @@ impl Schema {
             .map(|dtype| Field::new(name, dtype.clone()))
     }
 
+    pub fn try_get_field(&self, name: &str) -> PolarsResult<Field> {
+        self.inner
+            .get(name)
+            .ok_or_else(|| PolarsError::NotFound(name.to_string().into()))
+            .map(|dtype| Field::new(name, dtype.clone()))
+    }
+
     pub fn get_index(&self, index: usize) -> Option<(&String, &DataType)> {
         self.inner.get_index(index)
     }
 
+    pub fn get_index_mut(&mut self, index: usize) -> Option<(&mut String, &mut DataType)> {
+        self.inner.get_index_mut(index)
+    }
+
     pub fn coerce_by_name(&mut self, name: &str, dtype: DataType) -> Option<()> {
         *self.inner.get_mut(name)? = dtype;
         Some(())
@@ -175,13 +190,22 @@ impl Schema {
 
 pub type SchemaRef = Arc<Schema>;
 
+impl IntoIterator for Schema {
+    type Item = (String, DataType);
+    type IntoIter = <PlIndexMap<String, DataType> as IntoIterator>::IntoIter;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.inner.into_iter()
+    }
+}
+
 /// This trait exists to be unify the API of polars Schema and arrows Schema
 #[cfg(feature = "private")]
 pub trait IndexOfSchema: Debug {
     /// Get the index of column by name.
     fn index_of(&self, name: &str) -> Option<usize>;
 
-    fn try_index_of(&self, name: &str) -> Result<usize> {
+    fn try_index_of(&self, name: &str) -> PolarsResult<usize> {
         self.index_of(name).ok_or_else(|| {
             PolarsError::SchemaMisMatch(
                 format!(
