@@ -1,7 +1,7 @@
 use super::*;
 
 pub trait NumOpsDispatch: Debug {
-    fn subtract(&self, rhs: &Series) -> Result<Series> {
+    fn subtract(&self, rhs: &Series) -> PolarsResult<Series> {
         Err(PolarsError::InvalidOperation(
             format!(
                 "subtraction operation not supported for {:?} and {:?}",
@@ -10,7 +10,7 @@ pub trait NumOpsDispatch: Debug {
             .into(),
         ))
     }
-    fn add_to(&self, rhs: &Series) -> Result<Series> {
+    fn add_to(&self, rhs: &Series) -> PolarsResult<Series> {
         Err(PolarsError::InvalidOperation(
             format!(
                 "addition operation not supported for {:?} and {:?}",
@@ -19,7 +19,7 @@ pub trait NumOpsDispatch: Debug {
             .into(),
         ))
     }
-    fn multiply(&self, rhs: &Series) -> Result<Series> {
+    fn multiply(&self, rhs: &Series) -> PolarsResult<Series> {
         Err(PolarsError::InvalidOperation(
             format!(
                 "multiplication operation not supported for {:?} and {:?}",
@@ -28,7 +28,7 @@ pub trait NumOpsDispatch: Debug {
             .into(),
         ))
     }
-    fn divide(&self, rhs: &Series) -> Result<Series> {
+    fn divide(&self, rhs: &Series) -> PolarsResult<Series> {
         Err(PolarsError::InvalidOperation(
             format!(
                 "division operation not supported for {:?} and {:?}",
@@ -37,7 +37,7 @@ pub trait NumOpsDispatch: Debug {
             .into(),
         ))
     }
-    fn remainder(&self, rhs: &Series) -> Result<Series> {
+    fn remainder(&self, rhs: &Series) -> PolarsResult<Series> {
         Err(PolarsError::InvalidOperation(
             format!(
                 "remainder operation not supported for {:?} and {:?}",
@@ -53,7 +53,7 @@ where
     T: PolarsNumericType,
     ChunkedArray<T>: IntoSeries,
 {
-    fn subtract(&self, rhs: &Series) -> Result<Series> {
+    fn subtract(&self, rhs: &Series) -> PolarsResult<Series> {
         // Safety:
         // There will be UB if a ChunkedArray is alive with the wrong datatype.
         // we now only create the potentially wrong dtype for a short time.
@@ -63,28 +63,28 @@ where
         let out = self - rhs;
         Ok(out.into_series())
     }
-    fn add_to(&self, rhs: &Series) -> Result<Series> {
+    fn add_to(&self, rhs: &Series) -> PolarsResult<Series> {
         // Safety:
         // see subtract
         let rhs = unsafe { self.unpack_series_matching_physical_type(rhs) };
         let out = self + rhs;
         Ok(out.into_series())
     }
-    fn multiply(&self, rhs: &Series) -> Result<Series> {
+    fn multiply(&self, rhs: &Series) -> PolarsResult<Series> {
         // Safety:
         // see subtract
         let rhs = unsafe { self.unpack_series_matching_physical_type(rhs) };
         let out = self * rhs;
         Ok(out.into_series())
     }
-    fn divide(&self, rhs: &Series) -> Result<Series> {
+    fn divide(&self, rhs: &Series) -> PolarsResult<Series> {
         // Safety:
         // see subtract
         let rhs = unsafe { self.unpack_series_matching_physical_type(rhs) };
         let out = self / rhs;
         Ok(out.into_series())
     }
-    fn remainder(&self, rhs: &Series) -> Result<Series> {
+    fn remainder(&self, rhs: &Series) -> PolarsResult<Series> {
         // Safety:
         // see subtract
         let rhs = unsafe { self.unpack_series_matching_physical_type(rhs) };
@@ -94,7 +94,16 @@ where
 }
 
 impl NumOpsDispatch for Utf8Chunked {
-    fn add_to(&self, rhs: &Series) -> Result<Series> {
+    fn add_to(&self, rhs: &Series) -> PolarsResult<Series> {
+        let rhs = self.unpack_series_matching_type(rhs)?;
+        let out = self + rhs;
+        Ok(out.into_series())
+    }
+}
+
+#[cfg(feature = "dtype-binary")]
+impl NumOpsDispatch for BinaryChunked {
+    fn add_to(&self, rhs: &Series) -> PolarsResult<Series> {
         let rhs = self.unpack_series_matching_type(rhs)?;
         let out = self + rhs;
         Ok(out.into_series())
@@ -110,7 +119,7 @@ pub mod checked {
 
     pub trait NumOpsDispatchChecked: Debug {
         /// Checked integer division. Computes self / rhs, returning None if rhs == 0 or the division results in overflow.
-        fn checked_div(&self, rhs: &Series) -> Result<Series> {
+        fn checked_div(&self, rhs: &Series) -> PolarsResult<Series> {
             Err(PolarsError::InvalidOperation(
                 format!(
                     "checked division operation not supported for {:?} and {:?}",
@@ -119,7 +128,7 @@ pub mod checked {
                 .into(),
             ))
         }
-        fn checked_div_num<T: ToPrimitive>(&self, _rhs: T) -> Result<Series> {
+        fn checked_div_num<T: ToPrimitive>(&self, _rhs: T) -> PolarsResult<Series> {
             Err(PolarsError::InvalidOperation(
                 format!(
                     "checked division by number operation not supported for {:?}",
@@ -137,7 +146,7 @@ pub mod checked {
             CheckedDiv<Output = T::Native> + CheckedDiv<Output = T::Native> + num::Zero + num::One,
         ChunkedArray<T>: IntoSeries,
     {
-        fn checked_div(&self, rhs: &Series) -> Result<Series> {
+        fn checked_div(&self, rhs: &Series) -> PolarsResult<Series> {
             // Safety:
             // There will be UB if a ChunkedArray is alive with the wrong datatype.
             // we now only create the potentially wrong dtype for a short time.
@@ -166,7 +175,7 @@ pub mod checked {
     }
 
     impl NumOpsDispatchChecked for Float32Chunked {
-        fn checked_div(&self, rhs: &Series) -> Result<Series> {
+        fn checked_div(&self, rhs: &Series) -> PolarsResult<Series> {
             // Safety:
             // see check_div for chunkedarray<T>
             let rhs = unsafe { self.unpack_series_matching_physical_type(rhs) };
@@ -198,7 +207,7 @@ pub mod checked {
     }
 
     impl NumOpsDispatchChecked for Float64Chunked {
-        fn checked_div(&self, rhs: &Series) -> Result<Series> {
+        fn checked_div(&self, rhs: &Series) -> PolarsResult<Series> {
             // Safety:
             // see check_div
             let rhs = unsafe { self.unpack_series_matching_physical_type(rhs) };
@@ -230,12 +239,12 @@ pub mod checked {
     }
 
     impl NumOpsDispatchChecked for Series {
-        fn checked_div(&self, rhs: &Series) -> Result<Series> {
+        fn checked_div(&self, rhs: &Series) -> PolarsResult<Series> {
             let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
             lhs.as_ref().as_ref().checked_div(rhs.as_ref())
         }
 
-        fn checked_div_num<T: ToPrimitive>(&self, rhs: T) -> Result<Series> {
+        fn checked_div_num<T: ToPrimitive>(&self, rhs: T) -> PolarsResult<Series> {
             use DataType::*;
             let s = self.to_physical_repr();
 
@@ -322,7 +331,7 @@ pub mod checked {
 pub(crate) fn coerce_lhs_rhs<'a>(
     lhs: &'a Series,
     rhs: &'a Series,
-) -> Result<(Cow<'a, Series>, Cow<'a, Series>)> {
+) -> PolarsResult<(Cow<'a, Series>, Cow<'a, Series>)> {
     if let Some(result) = coerce_time_units(lhs, rhs) {
         return Ok(result);
     }
@@ -396,12 +405,51 @@ fn coerce_time_units<'a>(
     }
 }
 
+#[cfg(feature = "dtype-struct")]
+fn struct_arithmetic<F: FnMut(&Series, &Series) -> Series>(
+    s: &Series,
+    rhs: &Series,
+    mut func: F,
+) -> Series {
+    let s = s.struct_().unwrap();
+    let rhs = rhs.struct_().unwrap();
+    let s_fields = s.fields();
+    let rhs_fields = rhs.fields();
+    match (s_fields.len(), rhs_fields.len()) {
+        (_, 1) => {
+            let rhs = &rhs.fields()[0];
+            s.apply_fields(|s| func(s, rhs)).into_series()
+        }
+        (1, _) => {
+            let s = &s.fields()[0];
+            rhs.apply_fields(|rhs| func(s, rhs)).into_series()
+        }
+        _ => {
+            let mut rhs_iter = rhs.fields().iter();
+
+            s.apply_fields(|s| match rhs_iter.next() {
+                Some(rhs) => func(s, rhs),
+                None => s.clone(),
+            })
+            .into_series()
+        }
+    }
+}
+
 impl ops::Sub for &Series {
     type Output = Series;
 
     fn sub(self, rhs: Self) -> Self::Output {
-        let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
-        lhs.subtract(rhs.as_ref()).expect("data types don't match")
+        match (self.dtype(), rhs.dtype()) {
+            #[cfg(feature = "dtype-struct")]
+            (DataType::Struct(_), DataType::Struct(_)) => {
+                struct_arithmetic(self, rhs, |a, b| a.sub(b))
+            }
+            _ => {
+                let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
+                lhs.subtract(rhs.as_ref()).expect("data types don't match")
+            }
+        }
     }
 }
 
@@ -409,8 +457,16 @@ impl ops::Add for &Series {
     type Output = Series;
 
     fn add(self, rhs: Self) -> Self::Output {
-        let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
-        lhs.add_to(rhs.as_ref()).expect("data types don't match")
+        match (self.dtype(), rhs.dtype()) {
+            #[cfg(feature = "dtype-struct")]
+            (DataType::Struct(_), DataType::Struct(_)) => {
+                struct_arithmetic(self, rhs, |a, b| a.add(b))
+            }
+            _ => {
+                let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
+                lhs.add_to(rhs.as_ref()).expect("data types don't match")
+            }
+        }
     }
 }
 
@@ -423,8 +479,16 @@ impl ops::Mul for &Series {
     /// let out = &s * &s;
     /// ```
     fn mul(self, rhs: Self) -> Self::Output {
-        let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
-        lhs.multiply(rhs.as_ref()).expect("data types don't match")
+        match (self.dtype(), rhs.dtype()) {
+            #[cfg(feature = "dtype-struct")]
+            (DataType::Struct(_), DataType::Struct(_)) => {
+                struct_arithmetic(self, rhs, |a, b| a.mul(b))
+            }
+            _ => {
+                let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
+                lhs.multiply(rhs.as_ref()).expect("data types don't match")
+            }
+        }
     }
 }
 
@@ -437,8 +501,16 @@ impl ops::Div for &Series {
     /// let out = &s / &s;
     /// ```
     fn div(self, rhs: Self) -> Self::Output {
-        let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
-        lhs.divide(rhs.as_ref()).expect("data types don't match")
+        match (self.dtype(), rhs.dtype()) {
+            #[cfg(feature = "dtype-struct")]
+            (DataType::Struct(_), DataType::Struct(_)) => {
+                struct_arithmetic(self, rhs, |a, b| a.div(b))
+            }
+            _ => {
+                let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
+                lhs.divide(rhs.as_ref()).expect("data types don't match")
+            }
+        }
     }
 }
 
@@ -451,8 +523,16 @@ impl ops::Rem for &Series {
     /// let out = &s / &s;
     /// ```
     fn rem(self, rhs: Self) -> Self::Output {
-        let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
-        lhs.remainder(rhs.as_ref()).expect("data types don't match")
+        match (self.dtype(), rhs.dtype()) {
+            #[cfg(feature = "dtype-struct")]
+            (DataType::Struct(_), DataType::Struct(_)) => {
+                struct_arithmetic(self, rhs, |a, b| a.rem(b))
+            }
+            _ => {
+                let (lhs, rhs) = coerce_lhs_rhs(self, rhs).expect("cannot coerce datatypes");
+                lhs.remainder(rhs.as_ref()).expect("data types don't match")
+            }
+        }
     }
 }
 
