@@ -1,9 +1,5 @@
 //! Comparison operations on Series.
 
-use super::Series;
-use crate::apply_method_physical_numeric;
-use crate::prelude::*;
-use crate::series::arithmetic::coerce_lhs_rhs;
 #[cfg(any(
     feature = "dtype-duration",
     feature = "dtype-datetime",
@@ -13,6 +9,11 @@ use crate::series::arithmetic::coerce_lhs_rhs;
 ))]
 use std::ops::Deref;
 
+use super::Series;
+use crate::apply_method_physical_numeric;
+use crate::prelude::*;
+use crate::series::arithmetic::coerce_lhs_rhs;
+
 macro_rules! impl_compare {
     ($self:expr, $rhs:expr, $method:ident) => {{
         let (lhs, rhs) = coerce_lhs_rhs($self, $rhs).expect("cannot coerce datatypes");
@@ -21,6 +22,8 @@ macro_rules! impl_compare {
         match lhs.dtype() {
             DataType::Boolean => lhs.bool().unwrap().$method(rhs.bool().unwrap()),
             DataType::Utf8 => lhs.utf8().unwrap().$method(rhs.utf8().unwrap()),
+            #[cfg(feature = "dtype-binary")]
+            DataType::Binary => lhs.binary().unwrap().$method(rhs.binary().unwrap()),
             DataType::UInt8 => lhs.u8().unwrap().$method(rhs.u8().unwrap()),
             DataType::UInt16 => lhs.u16().unwrap().$method(rhs.u16().unwrap()),
             DataType::UInt32 => lhs.u32().unwrap().$method(rhs.u32().unwrap()),
@@ -64,9 +67,9 @@ fn compare_cat_to_str_value<Compare>(
     name: &str,
     compare: Compare,
     fill_value: bool,
-) -> Result<BooleanChunked>
+) -> PolarsResult<BooleanChunked>
 where
-    Compare: Fn(&Series, u32) -> Result<BooleanChunked>,
+    Compare: Fn(&Series, u32) -> PolarsResult<BooleanChunked>,
 {
     let cat = cat.categorical().expect("should be categorical");
     let cat_map = cat.get_rev_map();
@@ -86,9 +89,9 @@ fn compare_cat_to_str_series<Compare>(
     name: &str,
     compare: Compare,
     fill_value: bool,
-) -> Result<BooleanChunked>
+) -> PolarsResult<BooleanChunked>
 where
-    Compare: Fn(&Series, u32) -> Result<BooleanChunked>,
+    Compare: Fn(&Series, u32) -> PolarsResult<BooleanChunked>,
 {
     match string.utf8()?.get(0) {
         None => Ok(cat.is_null()),
@@ -96,7 +99,7 @@ where
     }
 }
 
-fn validate_types(left: &DataType, right: &DataType) -> Result<()> {
+fn validate_types(left: &DataType, right: &DataType) -> PolarsResult<()> {
     use DataType::*;
     #[cfg(feature = "dtype-categorical")]
     {
@@ -124,9 +127,9 @@ fn validate_types(left: &DataType, right: &DataType) -> Result<()> {
 }
 
 impl ChunkCompare<&Series> for Series {
-    type Item = Result<BooleanChunked>;
+    type Item = PolarsResult<BooleanChunked>;
 
-    fn eq_missing(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn eq_missing(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         #[cfg(feature = "dtype-categorical")]
         use DataType::*;
@@ -171,7 +174,7 @@ impl ChunkCompare<&Series> for Series {
     }
 
     /// Create a boolean mask by checking for equality.
-    fn equal(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn equal(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         #[cfg(feature = "dtype-categorical")]
         use DataType::*;
@@ -216,7 +219,7 @@ impl ChunkCompare<&Series> for Series {
     }
 
     /// Create a boolean mask by checking for inequality.
-    fn not_equal(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn not_equal(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         #[cfg(feature = "dtype-categorical")]
         use DataType::*;
@@ -261,7 +264,7 @@ impl ChunkCompare<&Series> for Series {
     }
 
     /// Create a boolean mask by checking if self > rhs.
-    fn gt(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn gt(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         let mut out = impl_compare!(self, rhs, gt);
         out.rename(self.name());
@@ -269,7 +272,7 @@ impl ChunkCompare<&Series> for Series {
     }
 
     /// Create a boolean mask by checking if self >= rhs.
-    fn gt_eq(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn gt_eq(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         let mut out = impl_compare!(self, rhs, gt_eq);
         out.rename(self.name());
@@ -277,7 +280,7 @@ impl ChunkCompare<&Series> for Series {
     }
 
     /// Create a boolean mask by checking if self < rhs.
-    fn lt(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn lt(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         let mut out = impl_compare!(self, rhs, lt);
         out.rename(self.name());
@@ -285,7 +288,7 @@ impl ChunkCompare<&Series> for Series {
     }
 
     /// Create a boolean mask by checking if self <= rhs.
-    fn lt_eq(&self, rhs: &Series) -> Result<BooleanChunked> {
+    fn lt_eq(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), rhs.dtype())?;
         let mut out = impl_compare!(self, rhs, lt_eq);
         out.rename(self.name());
@@ -297,45 +300,45 @@ impl<Rhs> ChunkCompare<Rhs> for Series
 where
     Rhs: NumericNative,
 {
-    type Item = Result<BooleanChunked>;
+    type Item = PolarsResult<BooleanChunked>;
 
-    fn eq_missing(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn eq_missing(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         // just take any numeric dtype for rhs
         validate_types(self.dtype(), &DataType::Int8)?;
         self.equal(rhs)
     }
 
-    fn equal(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn equal(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, equal, rhs))
     }
 
-    fn not_equal(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn not_equal(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, not_equal, rhs))
     }
 
-    fn gt(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn gt(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, gt, rhs))
     }
 
-    fn gt_eq(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn gt_eq(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, gt_eq, rhs))
     }
 
-    fn lt(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn lt(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, lt, rhs))
     }
 
-    fn lt_eq(&self, rhs: Rhs) -> Result<BooleanChunked> {
+    fn lt_eq(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, lt_eq, rhs))
@@ -343,13 +346,13 @@ where
 }
 
 impl ChunkCompare<&str> for Series {
-    type Item = Result<BooleanChunked>;
-    fn eq_missing(&self, rhs: &str) -> Result<BooleanChunked> {
+    type Item = PolarsResult<BooleanChunked>;
+    fn eq_missing(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         self.equal(rhs)
     }
 
-    fn equal(&self, rhs: &str) -> Result<BooleanChunked> {
+    fn equal(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         use DataType::*;
         match self.dtype() {
@@ -362,7 +365,7 @@ impl ChunkCompare<&str> for Series {
         }
     }
 
-    fn not_equal(&self, rhs: &str) -> Result<BooleanChunked> {
+    fn not_equal(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         use DataType::*;
         match self.dtype() {
@@ -379,7 +382,7 @@ impl ChunkCompare<&str> for Series {
         }
     }
 
-    fn gt(&self, rhs: &str) -> Result<BooleanChunked> {
+    fn gt(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         if let Ok(a) = self.utf8() {
             Ok(a.gt(rhs))
@@ -394,7 +397,7 @@ impl ChunkCompare<&str> for Series {
         }
     }
 
-    fn gt_eq(&self, rhs: &str) -> Result<BooleanChunked> {
+    fn gt_eq(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         if let Ok(a) = self.utf8() {
             Ok(a.gt_eq(rhs))
@@ -409,7 +412,7 @@ impl ChunkCompare<&str> for Series {
         }
     }
 
-    fn lt(&self, rhs: &str) -> Result<BooleanChunked> {
+    fn lt(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         if let Ok(a) = self.utf8() {
             Ok(a.lt(rhs))
@@ -424,7 +427,7 @@ impl ChunkCompare<&str> for Series {
         }
     }
 
-    fn lt_eq(&self, rhs: &str) -> Result<BooleanChunked> {
+    fn lt_eq(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
         validate_types(self.dtype(), &DataType::Utf8)?;
         if let Ok(a) = self.utf8() {
             Ok(a.lt_eq(rhs))
