@@ -11,7 +11,7 @@ use crate::frame::groupby::*;
 use crate::frame::hash_join::*;
 use crate::prelude::*;
 
-impl IntoSeries for DatetimeChunked {
+unsafe impl IntoSeries for DatetimeChunked {
     fn into_series(self) -> Series {
         Series(Arc::new(SeriesWrap(self)))
     }
@@ -73,8 +73,9 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
         })
     }
 
-    fn vec_hash(&self, random_state: RandomState) -> PolarsResult<Vec<u64>> {
-        Ok(self.0.vec_hash(random_state))
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+        self.0.vec_hash(random_state, buf);
+        Ok(())
     }
 
     fn vec_hash_combine(&self, build_hasher: RandomState, hashes: &mut [u64]) -> PolarsResult<()> {
@@ -103,24 +104,6 @@ impl private::PrivateSeries for SeriesWrap<DatetimeChunked> {
             .unwrap()
     }
 
-    unsafe fn agg_quantile(
-        &self,
-        groups: &GroupsProxy,
-        quantile: f64,
-        interpol: QuantileInterpolOptions,
-    ) -> Series {
-        self.0
-            .agg_quantile(groups, quantile, interpol)
-            .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
-            .into_series()
-    }
-
-    unsafe fn agg_median(&self, groups: &GroupsProxy) -> Series {
-        self.0
-            .agg_median(groups)
-            .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
-            .into_series()
-    }
     fn zip_outer_join_column(
         &self,
         right_column: &Series,
@@ -214,14 +197,6 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
         }
     }
 
-    #[cfg(feature = "interpolate")]
-    fn interpolate(&self) -> Series {
-        self.0
-            .interpolate()
-            .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
-            .into_series()
-    }
-
     fn rename(&mut self, name: &str) {
         self.0.rename(name);
     }
@@ -369,9 +344,9 @@ impl SeriesTrait for SeriesWrap<DatetimeChunked> {
             .into_series()
     }
 
-    fn expand_at_index(&self, index: usize, length: usize) -> Series {
+    fn new_from_index(&self, index: usize, length: usize) -> Series {
         self.0
-            .expand_at_index(index, length)
+            .new_from_index(index, length)
             .into_datetime(self.0.time_unit(), self.0.time_zone().clone())
             .into_series()
     }
