@@ -117,7 +117,7 @@ pub(crate) mod private {
         fn into_partial_ord_inner<'a>(&'a self) -> Box<dyn PartialOrdInner + 'a> {
             invalid_operation_panic!(self)
         }
-        fn vec_hash(&self, _build_hasher: RandomState) -> PolarsResult<Vec<u64>> {
+        fn vec_hash(&self, _build_hasher: RandomState, _buf: &mut Vec<u64>) -> PolarsResult<()> {
             invalid_operation!(self)
         }
         fn vec_hash_combine(
@@ -198,10 +198,6 @@ pub trait SeriesTrait:
         IsSorted::Not
     }
 
-    #[cfg(feature = "interpolate")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "interpolate")))]
-    fn interpolate(&self) -> Series;
-
     /// Rename the Series.
     fn rename(&mut self, name: &str);
 
@@ -381,17 +377,17 @@ pub trait SeriesTrait:
         None
     }
 
-    /// Create a new Series filled with values at that index.
+    /// Create a new Series filled with values from the given index.
     ///
     /// # Example
     ///
     /// ```rust
     /// use polars_core::prelude::*;
     /// let s = Series::new("a", [0i32, 1, 8]);
-    /// let expanded = s.expand_at_index(2, 4);
-    /// assert_eq!(Vec::from(expanded.i32().unwrap()), &[Some(8), Some(8), Some(8), Some(8)])
+    /// let s2 = s.new_from_index(2, 4);
+    /// assert_eq!(Vec::from(s2.i32().unwrap()), &[Some(8), Some(8), Some(8), Some(8)])
     /// ```
-    fn expand_at_index(&self, _index: usize, _length: usize) -> Series {
+    fn new_from_index(&self, _index: usize, _length: usize) -> Series {
         invalid_operation_panic!(self)
     }
 
@@ -581,15 +577,15 @@ pub trait SeriesTrait:
     }
     /// Get the median of the Series as a new Series of length 1.
     fn median_as_series(&self) -> Series {
-        invalid_operation_panic!(self)
+        Series::full_null(self.name(), 1, self.dtype())
     }
     /// Get the variance of the Series as a new Series of length 1.
     fn var_as_series(&self, _ddof: u8) -> Series {
-        invalid_operation_panic!(self)
+        Series::full_null(self.name(), 1, self.dtype())
     }
     /// Get the standard deviation of the Series as a new Series of length 1.
     fn std_as_series(&self, _ddof: u8) -> Series {
-        invalid_operation_panic!(self)
+        Series::full_null(self.name(), 1, self.dtype())
     }
     /// Get the quantile of the ChunkedArray as a new Series of length 1.
     fn quantile_as_series(
@@ -597,7 +593,7 @@ pub trait SeriesTrait:
         _quantile: f64,
         _interpol: QuantileInterpolOptions,
     ) -> PolarsResult<Series> {
-        invalid_operation_panic!(self)
+        Ok(Series::full_null(self.name(), 1, self.dtype()))
     }
 
     fn fmt_list(&self) -> String {
