@@ -1,11 +1,14 @@
 use std::convert::TryInto;
 use std::hash::{BuildHasher, BuildHasherDefault, Hash, Hasher};
 
-use ahash::{CallHasher, RandomState};
+use ahash::RandomState;
 use arrow::bitmap::utils::get_bit_unchecked;
-use hashbrown::{hash_map::RawEntryMut, HashMap};
+use hashbrown::hash_map::RawEntryMut;
+use hashbrown::HashMap;
 use polars_arrow::utils::CustomIterTools;
+use polars_utils::HashSingle;
 use rayon::prelude::*;
+use xxhash_rust::xxh3::xxh3_64_with_seed;
 
 use crate::datatypes::UInt64Chunked;
 use crate::prelude::*;
@@ -20,7 +23,7 @@ pub trait VecHash {
     /// Compute the hash for all values in the array.
     ///
     /// This currently only works with the AHash RandomState hasher builder.
-    fn vec_hash(&self, _random_state: RandomState) -> Vec<u64> {
+    fn vec_hash(&self, _random_state: RandomState, _buf: &mut Vec<u64>) {
         unimplemented!()
     }
 
@@ -40,103 +43,195 @@ pub(crate) fn get_null_hash_value(random_state: RandomState) -> u64 {
     hasher.finish()
 }
 
-impl<T> VecHash for ChunkedArray<T>
+macro_rules! fx_hash_8_bit {
+    ($val: expr, $k: expr ) => {{
+        let val = std::mem::transmute::<_, u8>($val);
+        (val as u64).wrapping_mul($k)
+    }};
+}
+macro_rules! fx_hash_16_bit {
+    ($val: expr, $k: expr ) => {{
+        let val = std::mem::transmute::<_, u16>($val);
+        (val as u64).wrapping_mul($k)
+    }};
+}
+macro_rules! fx_hash_32_bit {
+    ($val: expr, $k: expr ) => {{
+        let val = std::mem::transmute::<_, u32>($val);
+        (val as u64).wrapping_mul($k)
+    }};
+}
+macro_rules! fx_hash_64_bit {
+    ($val: expr, $k: expr ) => {{
+        ($val as u64).wrapping_mul($k)
+    }};
+}
+
+fn finish_vec_hash<T>(ca: &ChunkedArray<T>, random_state: RandomState, buf: &mut Vec<u64>)
 where
     T: PolarsIntegerType,
-    T::Native: Hash + CallHasher,
+    T::Native: Hash,
 {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        // Note that we don't use the no null branch! This can break in unexpected ways.
-        // for instance with threading we split an array in n_threads, this may lead to
-        // splits that have no nulls and splits that have nulls. Then one array is hashed with
-        // Option<T> and the other array with T.
-        // Meaning that they cannot be compared. By always hashing on Option<T> the random_state is
-        // the only deterministic seed.
-        let mut av = Vec::with_capacity(self.len());
+    let null_h = get_null_hash_value(random_state);
+    let hashes = buf.as_mut_slice();
 
-        self.downcast_iter().for_each(|arr| {
-            av.extend(
-                arr.values()
-                    .as_slice()
-                    .iter()
-                    .map(|v| T::Native::get_hash(v, &random_state)),
-            );
-        });
+    let mut offset = 0;
+    ca.downcast_iter().for_each(|arr| {
+        if arr.null_count() > 0 {
+            let validity = arr.validity().unwrap();
+            let (slice, byte_offset, _) = validity.as_slice();
+            (0..validity.len())
+                .map(|i| unsafe { get_bit_unchecked(slice, i + byte_offset) })
+                .zip(&mut hashes[offset..])
+                .for_each(|(valid, h)| {
+                    *h = [null_h, *h][valid as usize];
+                })
+        }
+        offset += arr.len();
+    });
+}
 
-        let null_h = get_null_hash_value(random_state);
-        let hashes = av.as_mut_slice();
+fn integer_vec_hash_combine<T>(ca: &ChunkedArray<T>, random_state: RandomState, hashes: &mut [u64])
+where
+    T: PolarsIntegerType,
+    T::Native: Hash,
+{
+    let null_h = get_null_hash_value(random_state.clone());
 
-        let mut offset = 0;
-        self.downcast_iter().for_each(|arr| {
-            if let Some(validity) = arr.validity() {
+    let mut offset = 0;
+    ca.downcast_iter().for_each(|arr| {
+        match arr.null_count() {
+            0 => arr
+                .values()
+                .as_slice()
+                .iter()
+                .zip(&mut hashes[offset..])
+                .for_each(|(v, h)| {
+                    let l = random_state.hash_single(v);
+                    *h = _boost_hash_combine(l, *h)
+                }),
+            _ => {
+                let validity = arr.validity().unwrap();
                 let (slice, byte_offset, _) = validity.as_slice();
                 (0..validity.len())
                     .map(|i| unsafe { get_bit_unchecked(slice, i + byte_offset) })
                     .zip(&mut hashes[offset..])
-                    .for_each(|(valid, h)| {
-                        *h = [null_h, *h][valid as usize];
-                    })
+                    .zip(arr.values().as_slice())
+                    .for_each(|((valid, h), l)| {
+                        *h = _boost_hash_combine(
+                            [null_h, random_state.hash_single(l)][valid as usize],
+                            *h,
+                        )
+                    });
             }
-            offset += arr.len();
-        });
-
-        av
-    }
-
-    fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
-        let null_h = get_null_hash_value(random_state.clone());
-
-        let mut offset = 0;
-        self.downcast_iter().for_each(|arr| {
-            match arr.null_count() {
-                0 => arr
-                    .values()
-                    .as_slice()
-                    .iter()
-                    .zip(&mut hashes[offset..])
-                    .for_each(|(v, h)| {
-                        let l = T::Native::get_hash(v, &random_state);
-                        *h = _boost_hash_combine(l, *h)
-                    }),
-                _ => {
-                    let validity = arr.validity().unwrap();
-                    let (slice, byte_offset, _) = validity.as_slice();
-                    (0..validity.len())
-                        .map(|i| unsafe { get_bit_unchecked(slice, i + byte_offset) })
-                        .zip(&mut hashes[offset..])
-                        .zip(arr.values().as_slice())
-                        .for_each(|((valid, h), l)| {
-                            *h = _boost_hash_combine(
-                                [null_h, T::Native::get_hash(l, &random_state)][valid as usize],
-                                *h,
-                            )
-                        });
-                }
-            }
-            offset += arr.len();
-        });
-    }
+        }
+        offset += arr.len();
+    });
 }
 
+macro_rules! vec_hash_int {
+    ($ca:ident, $fx_hash:ident) => {
+        impl VecHash for $ca {
+            fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+                // Note that we don't use the no null branch! This can break in unexpected ways.
+                // for instance with threading we split an array in n_threads, this may lead to
+                // splits that have no nulls and splits that have nulls. Then one array is hashed with
+                // Option<T> and the other array with T.
+                // Meaning that they cannot be compared. By always hashing on Option<T> the random_state is
+                // the only deterministic seed.
+                buf.clear();
+                buf.reserve(self.len());
+
+                let k: u64 = 0x517cc1b727220a95;
+                let k = random_state.hash_one(k);
+
+                #[allow(unused_unsafe)]
+                #[allow(clippy::useless_transmute)]
+                self.downcast_iter().for_each(|arr| {
+                    buf.extend(
+                        arr.values()
+                            .as_slice()
+                            .iter()
+                            .copied()
+                            .map(|v| unsafe { $fx_hash!(v, k) }),
+                    );
+                });
+                finish_vec_hash(self, random_state, buf)
+            }
+
+            fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
+                integer_vec_hash_combine(self, random_state, hashes)
+            }
+        }
+    };
+}
+
+vec_hash_int!(Int64Chunked, fx_hash_64_bit);
+vec_hash_int!(Int32Chunked, fx_hash_32_bit);
+vec_hash_int!(Int16Chunked, fx_hash_16_bit);
+vec_hash_int!(Int8Chunked, fx_hash_8_bit);
+vec_hash_int!(UInt64Chunked, fx_hash_64_bit);
+vec_hash_int!(UInt32Chunked, fx_hash_32_bit);
+vec_hash_int!(UInt16Chunked, fx_hash_16_bit);
+vec_hash_int!(UInt8Chunked, fx_hash_8_bit);
+
 impl VecHash for Utf8Chunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        let null_h = get_null_hash_value(random_state.clone());
-        let mut av = Vec::with_capacity(self.len());
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        buf.clear();
+        buf.reserve(self.len());
+        let null_h = get_null_hash_value(random_state);
+
         self.downcast_iter().for_each(|arr| {
-            av.extend(arr.into_iter().map(|opt_v| match opt_v {
-                Some(v) => str::get_hash(v, &random_state),
-                None => null_h,
-            }))
+            if arr.null_count() == 0 {
+                // simply use the null_hash as seed to get a hash determined by `random_state` that is passed
+                buf.extend(
+                    arr.values_iter()
+                        .map(|v| xxh3_64_with_seed(v.as_bytes(), null_h)),
+                )
+            } else {
+                buf.extend(arr.into_iter().map(|opt_v| match opt_v {
+                    Some(v) => xxh3_64_with_seed(v.as_bytes(), null_h),
+                    None => null_h,
+                }))
+            }
         });
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
-        let null_h = get_null_hash_value(random_state.clone());
+        let null_h = get_null_hash_value(random_state);
         self.apply_to_slice(
             |opt_v, h| {
                 let l = match opt_v {
-                    Some(v) => str::get_hash(v, &random_state),
+                    Some(v) => xxh3_64_with_seed(v.as_bytes(), null_h),
+                    None => null_h,
+                };
+                _boost_hash_combine(l, *h)
+            },
+            hashes,
+        )
+    }
+}
+
+#[cfg(feature = "dtype-binary")]
+impl VecHash for BinaryChunked {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        buf.clear();
+        buf.reserve(self.len());
+        let null_h = get_null_hash_value(random_state);
+        self.downcast_iter().for_each(|arr| {
+            buf.extend(arr.into_iter().map(|opt_v| match opt_v {
+                Some(v) => xxh3_64_with_seed(v, null_h),
+                None => null_h,
+            }))
+        });
+    }
+
+    fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
+        let null_h = get_null_hash_value(random_state);
+        self.apply_to_slice(
+            |opt_v, h| {
+                let l = match opt_v {
+                    Some(v) => xxh3_64_with_seed(v, null_h),
                     None => null_h,
                 };
                 _boost_hash_combine(l, *h)
@@ -147,16 +242,12 @@ impl VecHash for Utf8Chunked {
 }
 
 impl VecHash for BooleanChunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        let mut av = Vec::with_capacity(self.len());
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        buf.clear();
+        buf.reserve(self.len());
         self.downcast_iter().for_each(|arr| {
-            av.extend(arr.into_iter().map(|opt_v| {
-                let mut hasher = random_state.build_hasher();
-                opt_v.hash(&mut hasher);
-                hasher.finish()
-            }))
+            buf.extend(arr.into_iter().map(|opt_v| random_state.hash_single(opt_v)))
         });
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -172,8 +263,8 @@ impl VecHash for BooleanChunked {
 }
 
 impl VecHash for Float32Chunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        self.bit_repr_small().vec_hash(random_state)
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        self.bit_repr_small().vec_hash(random_state, buf)
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -181,8 +272,8 @@ impl VecHash for Float32Chunked {
     }
 }
 impl VecHash for Float64Chunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        self.bit_repr_large().vec_hash(random_state)
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        self.bit_repr_large().vec_hash(random_state, buf)
     }
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
         self.bit_repr_large().vec_hash_combine(random_state, hashes)
@@ -194,23 +285,23 @@ impl<T> VecHash for ObjectChunked<T>
 where
     T: PolarsObject,
 {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
         // Note that we don't use the no null branch! This can break in unexpected ways.
         // for instance with threading we split an array in n_threads, this may lead to
         // splits that have no nulls and splits that have nulls. Then one array is hashed with
         // Option<T> and the other array with T.
         // Meaning that they cannot be compared. By always hashing on Option<T> the random_state is
         // the only deterministic seed.
-        let mut av = Vec::with_capacity(self.len());
+        buf.clear();
+        buf.reserve(self.len());
 
         self.downcast_iter().for_each(|arr| {
-            av.extend(arr.into_iter().map(|opt_v| {
+            buf.extend(arr.into_iter().map(|opt_v| {
                 let mut hasher = random_state.build_hasher();
                 opt_v.hash(&mut hasher);
                 hasher.finish()
             }))
         });
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -331,14 +422,14 @@ const BUILD_HASHER: RandomState = RandomState::with_seeds(0, 0, 0, 0);
 impl AsU64 for [u8; 17] {
     #[inline]
     fn as_u64(self) -> u64 {
-        <[u8]>::get_hash(&self, &BUILD_HASHER)
+        BUILD_HASHER.hash_single(self)
     }
 }
 
 impl AsU64 for [u8; 13] {
     #[inline]
     fn as_u64(self) -> u64 {
-        <[u8]>::get_hash(&self, &BUILD_HASHER)
+        BUILD_HASHER.hash_single(self)
     }
 }
 
@@ -360,6 +451,7 @@ impl Hasher for IdHasher {
         self.write_u64(i as u64)
     }
 
+    #[inline]
     fn write_u64(&mut self, i: u64) {
         self.hash = i;
     }
@@ -387,7 +479,7 @@ pub(crate) struct IdxHash {
     // idx in row of Series, DataFrame
     pub(crate) idx: IdxSize,
     // precomputed hash of T
-    hash: u64,
+    pub(crate) hash: u64,
 }
 
 impl Hash for IdxHash {
@@ -407,30 +499,38 @@ impl IdxHash {
 /// During rehashes, we will rehash the hash instead of the string, that makes rehashing
 /// cheap and allows cache coherent small hash tables.
 #[derive(Eq, Copy, Clone, Debug)]
-pub(crate) struct StrHash<'a> {
-    str: Option<&'a str>,
+pub(crate) struct BytesHash<'a> {
+    payload: Option<&'a [u8]>,
     hash: u64,
 }
 
-impl<'a> Hash for StrHash<'a> {
+impl<'a> Hash for BytesHash<'a> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         state.write_u64(self.hash)
     }
 }
 
-impl<'a> StrHash<'a> {
-    pub(crate) fn new(s: Option<&'a str>, hash: u64) -> Self {
-        Self { str: s, hash }
+impl<'a> BytesHash<'a> {
+    #[inline]
+    pub(crate) fn new(s: Option<&'a [u8]>, hash: u64) -> Self {
+        Self { payload: s, hash }
+    }
+    #[inline]
+    pub(crate) fn new_from_str(s: Option<&'a str>, hash: u64) -> Self {
+        Self {
+            payload: s.map(|s| s.as_bytes()),
+            hash,
+        }
     }
 }
 
-impl<'a> PartialEq for StrHash<'a> {
+impl<'a> PartialEq for BytesHash<'a> {
     fn eq(&self, other: &Self) -> bool {
-        (self.hash == other.hash) && (self.str == other.str)
+        (self.hash == other.hash) && (self.payload == other.payload)
     }
 }
 
-impl<'a> AsU64 for StrHash<'a> {
+impl<'a> AsU64 for BytesHash<'a> {
     fn as_u64(self) -> u64 {
         self.hash
     }
@@ -542,34 +642,35 @@ pub fn _boost_hash_combine(l: u64, r: u64) -> u64 {
 pub(crate) fn df_rows_to_hashes_threaded(
     keys: &[DataFrame],
     hasher_builder: Option<RandomState>,
-) -> (Vec<UInt64Chunked>, RandomState) {
+) -> PolarsResult<(Vec<UInt64Chunked>, RandomState)> {
     let hasher_builder = hasher_builder.unwrap_or_default();
 
     let hashes = POOL.install(|| {
         keys.into_par_iter()
             .map(|df| {
                 let hb = hasher_builder.clone();
-                let (ca, _) = df_rows_to_hashes(df, Some(hb));
-                ca
+                let (ca, _) = df_rows_to_hashes(df, Some(hb))?;
+                Ok(ca)
             })
-            .collect()
-    });
-    (hashes, hasher_builder)
+            .collect::<PolarsResult<Vec<_>>>()
+    })?;
+    Ok((hashes, hasher_builder))
 }
 
 pub(crate) fn df_rows_to_hashes(
     keys: &DataFrame,
     build_hasher: Option<RandomState>,
-) -> (UInt64Chunked, RandomState) {
+) -> PolarsResult<(UInt64Chunked, RandomState)> {
     let build_hasher = build_hasher.unwrap_or_default();
 
     let mut iter = keys.iter();
     let first = iter.next().expect("at least one key");
-    let mut hashes = first.vec_hash(build_hasher.clone());
+    let mut hashes = vec![];
+    first.vec_hash(build_hasher.clone(), &mut hashes)?;
     let hslice = hashes.as_mut_slice();
 
     for keys in iter {
-        keys.vec_hash_combine(build_hasher.clone(), hslice);
+        keys.vec_hash_combine(build_hasher.clone(), hslice)?;
     }
 
     let chunks = vec![Box::new(PrimitiveArray::from_data(
@@ -577,5 +678,5 @@ pub(crate) fn df_rows_to_hashes(
         hashes.into(),
         None,
     )) as ArrayRef];
-    (UInt64Chunked::from_chunks("", chunks), build_hasher)
+    Ok((UInt64Chunked::from_chunks("", chunks), build_hasher))
 }
