@@ -1,8 +1,6 @@
 pub use arrow::io::avro::avro_schema::file::Compression;
-use arrow::io::avro::{
-    avro_schema::{self},
-    write,
-};
+use arrow::io::avro::avro_schema::{self};
+use arrow::io::avro::write;
 pub use Compression as AvroCompression;
 
 use super::*;
@@ -17,7 +15,7 @@ use super::*;
 /// use std::fs::File;
 /// use polars_io::SerWriter;
 ///
-/// fn example(df: &mut DataFrame) -> Result<()> {
+/// fn example(df: &mut DataFrame) -> PolarsResult<()> {
 ///     let mut file = File::create("file.avro").expect("could not create file");
 ///
 ///     AvroWriter::new(&mut file)
@@ -52,7 +50,7 @@ where
         }
     }
 
-    fn finish(&mut self, df: &mut DataFrame) -> Result<()> {
+    fn finish(&mut self, df: &mut DataFrame) -> PolarsResult<()> {
         let schema = df.schema().to_arrow();
         let record = write::to_record(&schema)?;
 
