@@ -39,7 +39,7 @@ pub(crate) fn get_file_chunks(
 
         let end_pos = match next_line_position(
             &bytes[search_pos..],
-            expected_fields,
+            Some(expected_fields),
             delimiter,
             quote_char,
             eol_char,
@@ -56,7 +56,9 @@ pub(crate) fn get_file_chunks(
     offsets
 }
 
-pub fn get_reader_bytes<R: Read + MmapBytesReader>(reader: &mut R) -> Result<ReaderBytes<'_>> {
+pub fn get_reader_bytes<R: Read + MmapBytesReader + ?Sized>(
+    reader: &mut R,
+) -> PolarsResult<ReaderBytes<'_>> {
     // we have a file so we can mmap
     if let Some(file) = reader.to_file() {
         let mmap = unsafe { memmap::Mmap::map(file)? };
@@ -145,7 +147,10 @@ fn infer_field_schema(string: &str, parse_dates: bool) -> DataType {
 }
 
 #[inline]
-pub(crate) fn parse_bytes_with_encoding(bytes: &[u8], encoding: CsvEncoding) -> Result<Cow<str>> {
+pub(crate) fn parse_bytes_with_encoding(
+    bytes: &[u8],
+    encoding: CsvEncoding,
+) -> PolarsResult<Cow<str>> {
     let s = match encoding {
         CsvEncoding::Utf8 => simdutf8::basic::from_utf8(bytes)
             .map_err(anyhow::Error::from)?
@@ -160,7 +165,10 @@ pub(crate) fn parse_bytes_with_encoding(bytes: &[u8], encoding: CsvEncoding) ->
 ///
 /// If `max_read_records` is not set, the whole file is read to infer its schema.
 ///
-/// Return inferred schema and number of records used for inference.
+/// Returns
+///     - inferred schema
+///     - number of rows used for inference.
+///     - bytes read
 #[allow(clippy::too_many_arguments)]
 pub fn infer_file_schema(
     reader_bytes: &ReaderBytes,
@@ -171,12 +179,16 @@ pub fn infer_file_schema(
     // we take &mut because we maybe need to skip more rows dependent
     // on the schema inference
     skip_rows: &mut usize,
+    skip_rows_after_header: usize,
     comment_char: Option<u8>,
     quote_char: Option<u8>,
     eol_char: u8,
     null_values: Option<&NullValues>,
     parse_dates: bool,
-) -> Result<(Schema, usize)> {
+) -> PolarsResult<(Schema, usize, usize)> {
+    // keep track so that we can determine the amount of bytes read
+    let start_ptr = reader_bytes.as_ptr() as usize;
+
     // We use lossy utf8 here because we don't want the schema inference to fail on utf8.
     // It may later.
     let encoding = CsvEncoding::LossyUtf8;
@@ -185,7 +197,7 @@ pub fn infer_file_schema(
     if bytes.is_empty() {
         return Err(PolarsError::NoData("empty csv".into()));
     }
-    let mut lines = SplitLines::new(bytes, eol_char).skip(*skip_rows);
+    let mut lines = SplitLines::new(bytes, quote_char.unwrap_or(b'"'), eol_char).skip(*skip_rows);
     // it can be that we have a single line without eol char
     let has_eol = bytes.contains(&eol_char);
 
@@ -233,16 +245,24 @@ pub fn infer_file_schema(
                         slice
                     };
                     let s = parse_bytes_with_encoding(slice_escaped, encoding)?;
-                    Ok(s.into())
+                    Ok(s)
                 })
-                .collect::<Result<Vec<_>>>()?;
+                .collect::<PolarsResult<Vec<_>>>()?;
 
-            if PlHashSet::from_iter(headers.iter()).len() != headers.len() {
-                return Err(PolarsError::ComputeError(
-                    "CSV header contains duplicate column names".into(),
-                ));
+            let mut final_headers = Vec::with_capacity(headers.len());
+
+            let mut header_names = PlHashMap::with_capacity(headers.len());
+
+            for name in &headers {
+                let count = header_names.entry(name.as_ref()).or_insert(0usize);
+                if *count != 0 {
+                    final_headers.push(format!("{}_duplicated_{}", name, *count - 1))
+                } else {
+                    final_headers.push(name.to_string())
+                }
+                *count += 1;
             }
-            headers
+            final_headers
         } else {
             let mut column_names: Vec<String> = byterecord
                 .enumerate()
@@ -269,6 +289,7 @@ pub fn infer_file_schema(
             has_header,
             schema_overwrite,
             skip_rows,
+            skip_rows_after_header,
             comment_char,
             quote_char,
             eol_char,
@@ -280,12 +301,13 @@ pub fn infer_file_schema(
     };
     if !has_header {
         // re-init lines so that the header is included in type inference.
-        lines = SplitLines::new(bytes, eol_char).skip(*skip_rows);
+        lines = SplitLines::new(bytes, quote_char.unwrap_or(b'"'), eol_char).skip(*skip_rows);
     }
 
     let header_length = headers.len();
     // keep track of inferred field types
-    let mut column_types: Vec<PlHashSet<DataType>> = vec![PlHashSet::new(); header_length];
+    let mut column_types: Vec<PlHashSet<DataType>> =
+        vec![PlHashSet::with_capacity(4); header_length];
     // keep track of columns with nulls
     let mut nulls: Vec<bool> = vec![false; header_length];
 
@@ -295,8 +317,14 @@ pub fn infer_file_schema(
     // needed to prevent ownership going into the iterator loop
     let records_ref = &mut lines;
 
-    for mut line in records_ref.take(max_read_lines.unwrap_or(usize::MAX)) {
+    let mut end_ptr = start_ptr;
+    for mut line in records_ref
+        .take(max_read_lines.unwrap_or(usize::MAX))
+        .skip(skip_rows_after_header)
+    {
         rows_count += 1;
+        // keep track so that we can determine the amount of bytes read
+        end_ptr = line.as_ptr() as usize + line.len();
 
         if let Some(c) = comment_char {
             // line is a comment -> skip
@@ -422,6 +450,7 @@ pub fn infer_file_schema(
             has_header,
             schema_overwrite,
             skip_rows,
+            skip_rows_after_header,
             comment_char,
             quote_char,
             eol_char,
@@ -430,7 +459,11 @@ pub fn infer_file_schema(
         );
     }
 
-    Ok((Schema::from(fields), rows_count))
+    Ok((
+        Schema::from(fields.into_iter()),
+        rows_count,
+        end_ptr - start_ptr,
+    ))
 }
 
 // magic numbers
@@ -450,7 +483,6 @@ pub fn is_compressed(bytes: &[u8]) -> bool {
 #[cfg(any(feature = "decompress", feature = "decompress-fast"))]
 fn decompress_impl<R: Read>(
     decoder: &mut R,
-    bytes: &[u8],
     n_rows: Option<usize>,
     delimiter: u8,
     quote_char: Option<u8>,
@@ -459,8 +491,9 @@ fn decompress_impl<R: Read>(
     let chunk_size = 4096;
     Some(match n_rows {
         None => {
-            // decompression will likely be an order of magnitude larger
-            let mut out = Vec::with_capacity(bytes.len() * 10);
+            // decompression in a preallocated buffer does not work with zlib-ng
+            // and will put the original compressed data in the buffer.
+            let mut out = Vec::new();
             decoder.read_to_end(&mut out).ok()?;
             out
         }
@@ -496,7 +529,7 @@ fn decompress_impl<R: Read>(
             while line_count < n_rows {
                 match next_line_position(
                     &out[buf_pos + 1..],
-                    expected_fields,
+                    Some(expected_fields),
                     delimiter,
                     quote_char,
                     eol_char,
@@ -531,10 +564,10 @@ pub(crate) fn decompress(
 ) -> Option<Vec<u8>> {
     if bytes.starts_with(&GZIP) {
         let mut decoder = flate2::read::MultiGzDecoder::new(bytes);
-        decompress_impl(&mut decoder, bytes, n_rows, delimiter, quote_char, eol_char)
+        decompress_impl(&mut decoder, n_rows, delimiter, quote_char, eol_char)
     } else if bytes.starts_with(&ZLIB0) || bytes.starts_with(&ZLIB1) || bytes.starts_with(&ZLIB2) {
         let mut decoder = flate2::read::ZlibDecoder::new(bytes);
-        decompress_impl(&mut decoder, bytes, n_rows, delimiter, quote_char, eol_char)
+        decompress_impl(&mut decoder, n_rows, delimiter, quote_char, eol_char)
     } else {
         None
     }
