@@ -12,6 +12,7 @@ pub struct PartitionGroupByExec {
     maintain_order: bool,
     slice: Option<(i64, usize)>,
     input_schema: SchemaRef,
+    from_partitioned_ds: bool,
 }
 
 impl PartitionGroupByExec {
@@ -22,6 +23,7 @@ impl PartitionGroupByExec {
         maintain_order: bool,
         slice: Option<(i64, usize)>,
         input_schema: SchemaRef,
+        from_partitioned_ds: bool,
     ) -> Self {
         Self {
             input,
@@ -30,14 +32,23 @@ impl PartitionGroupByExec {
             maintain_order,
             slice,
             input_schema,
+            from_partitioned_ds,
         }
     }
 
     fn keys(&self, df: &DataFrame, state: &ExecutionState) -> PolarsResult<Vec<Series>> {
-        self.keys.iter().map(|s| s.evaluate(df, state)).collect()
+        compute_keys(&self.keys, df, state)
     }
 }
 
+fn compute_keys(
+    keys: &[Arc<dyn PhysicalExpr>],
+    df: &DataFrame,
+    state: &ExecutionState,
+) -> PolarsResult<Vec<Series>> {
+    keys.iter().map(|s| s.evaluate(df, state)).collect()
+}
+
 fn run_partitions(
     df: &mut DataFrame,
     exec: &PartitionGroupByExec,
@@ -50,11 +61,12 @@ fn run_partitions(
     // split on several threads. Than the final result we apply the same groupby again.
     let dfs = split_df(df, n_threads)?;
 
+    let phys_aggs = &exec.phys_aggs;
+    let keys = &exec.keys;
     POOL.install(|| {
         dfs.into_par_iter()
             .map(|df| {
-                let keys = exec.keys(&df, state)?;
-                let phys_aggs = &exec.phys_aggs;
+                let keys = compute_keys(keys, &df, state)?;
                 let gb = df.groupby_with_series(keys, false, maintain_order)?;
                 let groups = gb.get_groups();
 
@@ -71,7 +83,7 @@ fn run_partitions(
                             if agg.len() == 1 {
                                 Ok(match groups.len()  {
                                     0 => agg.slice(0, 0),
-                                    len => agg.expand_at_index(0, len)
+                                    len => agg.new_from_index(0, len)
                                 })
                             } else {
                                 Err(PolarsError::ComputeError(
@@ -102,7 +114,6 @@ fn estimate_unique_count(keys: &[Series], mut sample_size: usize) -> PolarsResul
     // u: total unique groups counted in sample
     // ui: groups with single unique value counted in sample
     let set_size = keys[0].len();
-    let offset = (keys[0].len() / 2) as i64;
     if set_size < sample_size {
         sample_size = set_size;
     }
@@ -126,6 +137,7 @@ fn estimate_unique_count(keys: &[Series], mut sample_size: usize) -> PolarsResul
         let groups = s.group_tuples(true, false)?;
         Ok(finish(&groups))
     } else {
+        let offset = (keys[0].len() / 2) as i64;
         let keys = keys
             .iter()
             .map(|s| s.slice(offset, sample_size))
@@ -143,6 +155,7 @@ fn can_run_partitioned(
     keys: &[Series],
     original_df: &DataFrame,
     state: &ExecutionState,
+    from_partitioned_ds: bool,
 ) -> PolarsResult<bool> {
     if std::env::var("POLARS_NO_PARTITION").is_ok() {
         if state.verbose() {
@@ -185,7 +198,16 @@ fn can_run_partitioned(
             eprintln!("{} unique values: {}", sampled_method, unique_estimate);
         }
 
-        if unique_estimate > unique_count_boundary {
+        if from_partitioned_ds {
+            let estimated_cardinality = unique_estimate as f32 / original_df.height() as f32;
+            if estimated_cardinality < 0.4 {
+                eprintln!("PARTITIONED DS");
+                Ok(true)
+            } else {
+                eprintln!("PARTITIONED DS: estimated cardinality: {} exceeded the boundary: 0.4, running default HASH AGGREGATION", estimated_cardinality);
+                Ok(false)
+            }
+        } else if unique_estimate > unique_count_boundary {
             if state.verbose() {
                 eprintln!("estimated unique count: {} exceeded the boundary: {}, running default HASH AGGREGATION",unique_estimate, unique_count_boundary)
             }
@@ -209,7 +231,9 @@ impl PartitionGroupByExec {
             // of groups.
             let keys = self.keys(&original_df, state)?;
 
-            if !can_run_partitioned(&keys, &original_df, state)? {
+            // set it here, because `self.input.execute` will clear the schema cache.
+            state.set_schema(self.input_schema.clone());
+            if !can_run_partitioned(&keys, &original_df, state, self.from_partitioned_ds)? {
                 return groupby_helper(
                     original_df,
                     keys,
@@ -227,8 +251,6 @@ impl PartitionGroupByExec {
             // Run the partitioned aggregations
             let n_threads = POOL.current_num_threads();
 
-            // set it here, because `self.input.execute` will clear the schema cache.
-            state.set_schema(self.input_schema.clone());
             run_partitions(
                 &mut original_df,
                 self,
@@ -288,7 +310,7 @@ impl Executor for PartitionGroupByExec {
         #[cfg(debug_assertions)]
         {
             if state.verbose() {
-                println!("run PartititonGroupbyExec")
+                println!("run PartitionGroupbyExec")
             }
         }
         let original_df = self.input.execute(state)?;
@@ -299,7 +321,7 @@ impl Executor for PartitionGroupByExec {
                 .iter()
                 .map(|s| Ok(s.to_field(&self.input_schema)?.name))
                 .collect::<PolarsResult<Vec<_>>>()?;
-            let name = column_delimited("groupby_paritioned".to_string(), &by);
+            let name = column_delimited("groupby_partitioned".to_string(), &by);
             Cow::Owned(name)
         } else {
             Cow::Borrowed("")
