@@ -5,6 +5,7 @@ use polars_core::frame::groupby::GroupsProxy;
 use polars_core::prelude::*;
 use polars_core::utils::NoNull;
 
+use crate::physical_plan::expression_err;
 use crate::physical_plan::state::ExecutionState;
 use crate::prelude::*;
 
@@ -15,7 +16,12 @@ pub struct TakeExpr {
 }
 
 impl TakeExpr {
-    fn finish(&self, df: &DataFrame, state: &ExecutionState, series: Series) -> Result<Series> {
+    fn finish(
+        &self,
+        df: &DataFrame,
+        state: &ExecutionState,
+        series: Series,
+    ) -> PolarsResult<Series> {
         let idx = self.idx.evaluate(df, state)?.cast(&IDX_DTYPE)?;
         let idx_ca = idx.idx()?;
 
@@ -27,7 +33,7 @@ impl PhysicalExpr for TakeExpr {
     fn as_expression(&self) -> Option<&Expr> {
         Some(&self.expr)
     }
-    fn evaluate(&self, df: &DataFrame, state: &ExecutionState) -> Result<Series> {
+    fn evaluate(&self, df: &DataFrame, state: &ExecutionState) -> PolarsResult<Series> {
         let series = self.phys_expr.evaluate(df, state)?;
         self.finish(df, state, series)
     }
@@ -38,10 +44,15 @@ impl PhysicalExpr for TakeExpr {
         df: &DataFrame,
         groups: &'a GroupsProxy,
         state: &ExecutionState,
-    ) -> Result<AggregationContext<'a>> {
+    ) -> PolarsResult<AggregationContext<'a>> {
         let mut ac = self.phys_expr.evaluate_on_groups(df, groups, state)?;
         let mut idx = self.idx.evaluate_on_groups(df, groups, state)?;
 
+        let oob_err = || {
+            let msg = "Out of bounds.";
+            Err(expression_err!(msg, self.expr, ComputeError))
+        };
+
         let idx =
             match idx.state {
                 AggState::AggregatedFlat(s) => {
@@ -69,7 +80,7 @@ impl PhysicalExpr for TakeExpr {
                                         Some(idx) => idx >= g.len() as IdxSize,
                                     },
                                 ) {
-                                    return Err(PolarsError::ComputeError("out of bounds".into()));
+                                    return oob_err();
                                 }
 
                                 idx.into_iter()
@@ -86,7 +97,7 @@ impl PhysicalExpr for TakeExpr {
                                         Some(idx) => idx >= g[1],
                                     })
                                 {
-                                    return Err(PolarsError::ComputeError("out of bounds".into()));
+                                    return oob_err();
                                 }
 
                                 idx.into_iter()
@@ -125,18 +136,14 @@ impl PhysicalExpr for TakeExpr {
                                 let idx: NoNull<IdxCa> = match groups.as_ref() {
                                     GroupsProxy::Idx(groups) => {
                                         if groups.all().iter().any(|g| idx >= g.len() as IdxSize) {
-                                            return Err(PolarsError::ComputeError(
-                                                "out of bounds".into(),
-                                            ));
+                                            return oob_err();
                                         }
 
                                         groups.first().iter().map(|f| *f + idx).collect_trusted()
                                     }
                                     GroupsProxy::Slice { groups, .. } => {
                                         if groups.iter().any(|g| idx >= g[1]) {
-                                            return Err(PolarsError::ComputeError(
-                                                "out of bounds".into(),
-                                            ));
+                                            return oob_err();
                                         }
 
                                         groups.iter().map(|g| g[0] + idx).collect_trusted()
@@ -165,7 +172,7 @@ impl PhysicalExpr for TakeExpr {
         let s = idx.cast(&DataType::List(Box::new(IDX_DTYPE)))?;
         let idx = s.list().unwrap();
 
-        let taken = ac
+        let mut taken = ac
             .aggregated()
             .list()
             .unwrap()
@@ -180,14 +187,16 @@ impl PhysicalExpr for TakeExpr {
                 })
                 .transpose()
             })
-            .collect::<Result<ListChunked>>()?;
+            .collect::<PolarsResult<ListChunked>>()?;
+
+        taken.rename(ac.series().name());
 
         ac.with_series(taken.into_series(), true);
         ac.with_update_groups(UpdateGroups::WithGroupsLen);
         Ok(ac)
     }
 
-    fn to_field(&self, input_schema: &Schema) -> Result<Field> {
+    fn to_field(&self, input_schema: &Schema) -> PolarsResult<Field> {
         self.phys_expr.to_field(input_schema)
     }
 
