@@ -27,6 +27,22 @@ pub struct DynamicGroupOptions {
     // add the boundaries to the dataframe
     pub include_boundaries: bool,
     pub closed_window: ClosedWindow,
+    pub start_by: StartBy,
+}
+
+impl Default for DynamicGroupOptions {
+    fn default() -> Self {
+        Self {
+            index_column: "".to_string(),
+            every: Duration::new(1),
+            period: Duration::new(1),
+            offset: Duration::new(1),
+            truncate: true,
+            include_boundaries: false,
+            closed_window: ClosedWindow::Left,
+            start_by: Default::default(),
+        }
+    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -202,10 +218,22 @@ impl Wrap<&DataFrame> {
     ) -> PolarsResult<(Series, Vec<Series>, GroupsProxy)> {
         let w = Window::new(options.every, options.period, options.offset);
         let dt = dt.datetime().unwrap();
+        let tz = dt.time_zone();
 
         let mut lower_bound = None;
         let mut upper_bound = None;
 
+        let mut include_lower_bound = false;
+        let mut include_upper_bound = false;
+
+        if options.include_boundaries {
+            include_lower_bound = true;
+            include_upper_bound = true;
+        }
+        if options.truncate {
+            include_lower_bound = true;
+        }
+
         let mut update_bounds =
             |lower: Vec<i64>, upper: Vec<i64>| match (&mut lower_bound, &mut upper_bound) {
                 (None, None) => {
@@ -222,8 +250,15 @@ impl Wrap<&DataFrame> {
         let groups = if by.is_empty() {
             let vals = dt.downcast_iter().next().unwrap();
             let ts = vals.values().as_slice();
-            let (groups, lower, upper) =
-                groupby_windows(w, ts, options.include_boundaries, options.closed_window, tu);
+            let (groups, lower, upper) = groupby_windows(
+                w,
+                ts,
+                options.closed_window,
+                tu,
+                include_lower_bound,
+                include_upper_bound,
+                options.start_by,
+            );
             update_bounds(lower, upper);
             GroupsProxy::Slice {
                 groups,
@@ -237,34 +272,37 @@ impl Wrap<&DataFrame> {
             let groups = groups.into_idx();
 
             // include boundaries cannot be parallel (easily)
-            if options.include_boundaries {
-                let groupsidx = groups
-                    .iter()
-                    // we just flat map, because iterate over groups so we almost always need to reallocate
-                    .flat_map(|base_g| {
-                        let dt = unsafe { dt.take_unchecked(base_g.1.into()) };
+            if include_lower_bound {
+                POOL.install(|| {
+                    let mut ir = groups
+                        .par_iter()
+                        .map(|base_g| {
+                            let dt = unsafe { dt.take_unchecked(base_g.1.into()) };
 
-                        let vals = dt.downcast_iter().next().unwrap();
-                        let ts = vals.values().as_slice();
-                        let (sub_groups, lower, upper) = groupby_windows(
-                            w,
-                            ts,
-                            options.include_boundaries,
-                            options.closed_window,
-                            tu,
-                        );
-                        let _lower = Int64Chunked::new_vec("lower", lower.clone())
-                            .into_datetime(tu, None)
-                            .into_series();
-                        let _higher = Int64Chunked::new_vec("upper", upper.clone())
-                            .into_datetime(tu, None)
-                            .into_series();
+                            let vals = dt.downcast_iter().next().unwrap();
+                            let ts = vals.values().as_slice();
+                            let (sub_groups, lower, upper) = groupby_windows(
+                                w,
+                                ts,
+                                options.closed_window,
+                                tu,
+                                include_lower_bound,
+                                include_upper_bound,
+                                options.start_by,
+                            );
 
-                        update_bounds(lower, upper);
-                        update_subgroups_idx(&sub_groups, base_g)
-                    })
-                    .collect();
-                GroupsProxy::Idx(groupsidx)
+                            (lower, upper, update_subgroups_idx(&sub_groups, base_g))
+                        })
+                        .collect::<Vec<_>>();
+
+                    ir.iter_mut().for_each(|(lower, upper, _)| {
+                        let lower = std::mem::take(lower);
+                        let upper = std::mem::take(upper);
+                        update_bounds(lower, upper)
+                    });
+
+                    GroupsProxy::Idx(ir.into_iter().flat_map(|(_, _, groups)| groups).collect())
+                })
             } else {
                 let groupsidx = POOL.install(|| {
                     groups
@@ -276,9 +314,11 @@ impl Wrap<&DataFrame> {
                             let (sub_groups, _, _) = groupby_windows(
                                 w,
                                 ts,
-                                options.include_boundaries,
                                 options.closed_window,
                                 tu,
+                                include_lower_bound,
+                                include_upper_bound,
+                                options.start_by,
                             );
                             update_subgroups_idx(&sub_groups, base_g)
                         })
@@ -294,25 +334,19 @@ impl Wrap<&DataFrame> {
             *key = unsafe { key.agg_first(&groups) };
         }
 
+        let lower = lower_bound.map(|lower| Int64Chunked::new_vec(LB_NAME, lower));
+
         if options.truncate {
-            let w = Window::new(options.every, options.period, options.offset);
-            let truncate_fn = match tu {
-                TimeUnit::Nanoseconds => Window::truncate_no_offset_ns,
-                TimeUnit::Microseconds => Window::truncate_no_offset_us,
-                TimeUnit::Milliseconds => Window::truncate_no_offset_ms,
-            };
-            dt = dt.apply(|v| truncate_fn(&w, v));
+            let mut lower = lower.clone().unwrap();
+            lower.rename(dt.name());
+            dt = lower;
         }
 
-        if let (true, Some(lower), Some(higher)) =
-            (options.include_boundaries, lower_bound, upper_bound)
+        if let (true, Some(lower), Some(higher)) = (options.include_boundaries, lower, upper_bound)
         {
-            let s = Int64Chunked::new_vec(LB_NAME, lower)
-                .into_datetime(tu, None)
-                .into_series();
-            by.push(s);
+            by.push(lower.into_datetime(tu, tz.clone()).into_series());
             let s = Int64Chunked::new_vec(UP_NAME, higher)
-                .into_datetime(tu, None)
+                .into_datetime(tu, tz.clone())
                 .into_series();
             by.push(s);
         }
@@ -472,7 +506,7 @@ mod test {
                     "2020-01-08 23:16:43",
                 ],
             )
-            .as_datetime(None, tu)?
+            .as_datetime(None, tu, false)?
             .into_series();
             let a = Series::new("a", [3, 7, 5, 9, 2, 1]);
             let df = DataFrame::new(vec![date, a.clone()])?;
@@ -510,7 +544,7 @@ mod test {
                 "2020-01-08 23:16:43",
             ],
         )
-        .as_datetime(None, TimeUnit::Milliseconds)?
+        .as_datetime(None, TimeUnit::Milliseconds, false)?
         .into_series();
         let a = Series::new("a", [3, 7, 5, 9, 2, 1]);
         let df = DataFrame::new(vec![date, a.clone()])?;
@@ -568,11 +602,15 @@ mod test {
 
     #[test]
     fn test_dynamic_groupby_window() {
-        let start = NaiveDate::from_ymd(2021, 12, 16)
-            .and_hms(0, 0, 0)
+        let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap()
             .timestamp_millis();
-        let stop = NaiveDate::from_ymd(2021, 12, 16)
-            .and_hms(3, 0, 0)
+        let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+            .unwrap()
+            .and_hms_opt(3, 0, 0)
+            .unwrap()
             .timestamp_millis();
         let range = date_range_impl(
             "date",
@@ -581,6 +619,7 @@ mod test {
             Duration::parse("30m"),
             ClosedWindow::Both,
             TimeUnit::Milliseconds,
+            None,
         )
         .into_series();
 
@@ -598,6 +637,7 @@ mod test {
                     truncate: true,
                     include_boundaries: true,
                     closed_window: ClosedWindow::Both,
+                    start_by: Default::default(),
                 },
             )
             .unwrap();
@@ -614,11 +654,15 @@ mod test {
         assert_eq!(g, &["a", "a", "a", "a", "b", "b"]);
 
         let upper = out.column("_upper_boundary").unwrap().slice(0, 3);
-        let start = NaiveDate::from_ymd(2021, 12, 16)
-            .and_hms(1, 0, 0)
+        let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+            .unwrap()
+            .and_hms_opt(1, 0, 0)
+            .unwrap()
             .timestamp_millis();
-        let stop = NaiveDate::from_ymd(2021, 12, 16)
-            .and_hms(3, 0, 0)
+        let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+            .unwrap()
+            .and_hms_opt(3, 0, 0)
+            .unwrap()
             .timestamp_millis();
         let range = date_range_impl(
             "_upper_boundary",
@@ -627,16 +671,21 @@ mod test {
             Duration::parse("1h"),
             ClosedWindow::Both,
             TimeUnit::Milliseconds,
+            None,
         )
         .into_series();
         assert_eq!(&upper, &range);
 
         let upper = out.column("_lower_boundary").unwrap().slice(0, 3);
-        let start = NaiveDate::from_ymd(2021, 12, 16)
-            .and_hms(0, 0, 0)
+        let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+            .unwrap()
+            .and_hms_opt(0, 0, 0)
+            .unwrap()
             .timestamp_millis();
-        let stop = NaiveDate::from_ymd(2021, 12, 16)
-            .and_hms(2, 0, 0)
+        let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+            .unwrap()
+            .and_hms_opt(2, 0, 0)
+            .unwrap()
             .timestamp_millis();
         let range = date_range_impl(
             "_lower_boundary",
@@ -645,6 +694,7 @@ mod test {
             Duration::parse("1h"),
             ClosedWindow::Both,
             TimeUnit::Milliseconds,
+            None,
         )
         .into_series();
         assert_eq!(&upper, &range);
@@ -677,7 +727,55 @@ mod test {
                 truncate: true,
                 include_boundaries: true,
                 closed_window: ClosedWindow::Both,
+                start_by: Default::default(),
             },
         );
     }
+
+    #[test]
+    fn test_truncate_offset() {
+        let start = NaiveDate::from_ymd_opt(2021, 3, 1)
+            .unwrap()
+            .and_hms_opt(12, 0, 0)
+            .unwrap()
+            .timestamp_millis();
+        let stop = NaiveDate::from_ymd_opt(2021, 3, 7)
+            .unwrap()
+            .and_hms_opt(12, 0, 0)
+            .unwrap()
+            .timestamp_millis();
+        let range = date_range_impl(
+            "date",
+            start,
+            stop,
+            Duration::parse("1d"),
+            ClosedWindow::Both,
+            TimeUnit::Milliseconds,
+            None,
+        )
+        .into_series();
+
+        let groups = Series::new("groups", ["a", "a", "a", "b", "b", "a", "a"]);
+        let df = DataFrame::new(vec![range, groups.clone()]).unwrap();
+
+        let (mut time_key, mut keys, groups) = df
+            .groupby_dynamic(
+                vec![groups],
+                &DynamicGroupOptions {
+                    index_column: "date".into(),
+                    every: Duration::parse("6d"),
+                    period: Duration::parse("6d"),
+                    offset: Duration::parse("0h"),
+                    truncate: true,
+                    include_boundaries: true,
+                    closed_window: ClosedWindow::Both,
+                    start_by: Default::default(),
+                },
+            )
+            .unwrap();
+        let mut lower_bound = keys[1].clone();
+        time_key.rename("");
+        lower_bound.rename("");
+        assert!(time_key.series_equal(&lower_bound));
+    }
 }
