@@ -2,16 +2,17 @@ use std::cmp::Ordering;
 
 use polars_arrow::trusted_len::TrustedLen;
 use polars_arrow::utils::CustomIterTools;
+use polars_core::export::rayon::prelude::*;
 use polars_core::prelude::*;
+use polars_core::utils::_split_offsets;
 use polars_core::POOL;
-use polars_core::{export::rayon::prelude::*, utils::split_offsets};
 use polars_utils::flatten;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
 use crate::prelude::*;
 
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub enum ClosedWindow {
     Left,
@@ -20,6 +21,21 @@ pub enum ClosedWindow {
     None,
 }
 
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub enum StartBy {
+    WindowBound,
+    DataPoint,
+    /// only useful if periods are weekly
+    Monday,
+}
+
+impl Default for StartBy {
+    fn default() -> Self {
+        Self::WindowBound
+    }
+}
+
 /// Based on the given `Window`, which has an
 /// - every
 /// - period
@@ -33,11 +49,16 @@ pub enum ClosedWindow {
 pub fn groupby_windows(
     window: Window,
     time: &[i64],
-    include_boundaries: bool,
     closed_window: ClosedWindow,
     tu: TimeUnit,
+    include_lower_bound: bool,
+    include_upper_bound: bool,
+    start_by: StartBy,
 ) -> (GroupsSlice, Vec<i64>, Vec<i64>) {
     let start = time[0];
+    // the boundary we define here is not yet correct. It doesn't take 'period' into account
+    // and it doesn't have the proper starting point. This boundary is used as a proxy to find
+    // the proper 'boundary' in  'window.get_overlapping_bounds_iter'.
     let boundary = if time.len() > 1 {
         // +1 because left or closed boundary could match the next window if it is on the boundary
         let stop = time[time.len() - 1] + 1;
@@ -47,7 +68,7 @@ pub fn groupby_windows(
         Bounds::new_checked(start, stop)
     };
 
-    let size = if include_boundaries {
+    let size = if include_lower_bound || include_upper_bound {
         match tu {
             TimeUnit::Nanoseconds => window.estimate_overlapping_bounds_ns(boundary),
             TimeUnit::Microseconds => window.estimate_overlapping_bounds_us(boundary),
@@ -56,8 +77,10 @@ pub fn groupby_windows(
     } else {
         0
     };
-    let mut lower_bound = Vec::with_capacity(size);
-    let mut upper_bound = Vec::with_capacity(size);
+    let size_lower = if include_lower_bound { size } else { 0 };
+    let size_upper = if include_upper_bound { size } else { 0 };
+    let mut lower_bound = Vec::with_capacity(size_lower);
+    let mut upper_bound = Vec::with_capacity(size_upper);
 
     let mut groups = match tu {
         TimeUnit::Nanoseconds => {
@@ -72,7 +95,7 @@ pub fn groupby_windows(
     };
     let mut start_offset = 0;
 
-    for bi in window.get_overlapping_bounds_iter(boundary, tu) {
+    for bi in window.get_overlapping_bounds_iter(boundary, tu, start_by) {
         let mut skip_window = false;
         // find starting point of window
         while start_offset < time.len() {
@@ -105,8 +128,10 @@ pub fn groupby_windows(
         if i == time.len() - 1 {
             let t = time[i];
             if bi.is_member(t, closed_window) {
-                if include_boundaries {
+                if include_lower_bound {
                     lower_bound.push(bi.start);
+                }
+                if include_upper_bound {
                     upper_bound.push(bi.stop);
                 }
                 groups.push([i as IdxSize, 1])
@@ -125,8 +150,10 @@ pub fn groupby_windows(
         }
         let len = (i as IdxSize) - first;
 
-        if include_boundaries {
+        if include_lower_bound {
             lower_bound.push(bi.start);
+        }
+        if include_upper_bound {
             upper_bound.push(bi.stop);
         }
         groups.push([first, len])
@@ -159,7 +186,7 @@ pub(crate) fn groupby_values_iter_full_lookbehind(
         .enumerate()
         .map(move |(mut i, lower)| {
             if *lower < last {
-                panic!("index column of 'groupby_rolling' must be sorted!")
+                panic!("index column of 'groupby_rolling' must be sorted in ascending order!")
             }
             last = *lower;
             i += start_offset;
@@ -386,7 +413,7 @@ pub fn groupby_values(
     tu: TimeUnit,
 ) -> GroupsSlice {
     partially_check_sorted(time);
-    let thread_offsets = split_offsets(time.len(), POOL.current_num_threads());
+    let thread_offsets = _split_offsets(time.len(), POOL.current_num_threads());
 
     // we have a (partial) lookbehind window
     if offset.negative {
