@@ -7,8 +7,14 @@ use crate::prelude::*;
 #[test]
 fn test_date_range() {
     // Test month as interval in date range
-    let start = NaiveDate::from_ymd(2022, 1, 1).and_hms(0, 0, 0);
-    let end = NaiveDate::from_ymd(2022, 4, 1).and_hms(0, 0, 0);
+    let start = NaiveDate::from_ymd_opt(2022, 1, 1)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let end = NaiveDate::from_ymd_opt(2022, 4, 1)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
     let dates = date_range_vec(
         start.timestamp_nanos(),
         end.timestamp_nanos(),
@@ -30,8 +36,14 @@ fn test_date_range() {
 
 #[test]
 fn test_feb_date_range() {
-    let start = NaiveDate::from_ymd(2022, 2, 1).and_hms(0, 0, 0);
-    let end = NaiveDate::from_ymd(2022, 3, 1).and_hms(0, 0, 0);
+    let start = NaiveDate::from_ymd_opt(2022, 2, 1)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let end = NaiveDate::from_ymd_opt(2022, 3, 1)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
     let dates = date_range_vec(
         start.timestamp_nanos(),
         end.timestamp_nanos(),
@@ -65,10 +77,10 @@ fn take_groups_slice<'a>(groups: &'a GroupsSlice, idx: usize, ts: &'a [i64]) ->
 #[test]
 fn test_groups_large_interval() {
     let dates = &[
-        NaiveDate::from_ymd(2020, 1, 1),
-        NaiveDate::from_ymd(2020, 1, 11),
-        NaiveDate::from_ymd(2020, 1, 12),
-        NaiveDate::from_ymd(2020, 1, 13),
+        NaiveDate::from_ymd_opt(2020, 1, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2020, 1, 11).unwrap(),
+        NaiveDate::from_ymd_opt(2020, 1, 12).unwrap(),
+        NaiveDate::from_ymd_opt(2020, 1, 13).unwrap(),
     ];
     let ts = dates
         .iter()
@@ -77,16 +89,40 @@ fn test_groups_large_interval() {
 
     let dur = Duration::parse("2d");
     let w = Window::new(Duration::parse("2d"), dur.clone(), Duration::from_nsecs(0));
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::Both, TimeUnit::Nanoseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Both,
+        TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups.len(), 4);
     assert_eq!(groups[0], [0, 1]);
     assert_eq!(groups[1], [1, 1]);
     assert_eq!(groups[2], [1, 3]);
     assert_eq!(groups[3], [3, 1]);
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::Left, TimeUnit::Nanoseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Left,
+        TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups.len(), 3);
     assert_eq!(groups[2], [3, 1]);
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::Right, TimeUnit::Nanoseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Right,
+        TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups.len(), 2);
     assert_eq!(groups[1], [2, 2]);
 }
@@ -136,8 +172,15 @@ fn test_boundaries() {
     assert_eq!(b.start, start.timestamp_nanos());
 
     // test closed: "both" (includes both ends of the interval)
-    let (groups, lower, higher) =
-        groupby_windows(w, &ts, true, ClosedWindow::Both, TimeUnit::Nanoseconds);
+    let (groups, lower, higher) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Both,
+        TimeUnit::Nanoseconds,
+        true,
+        true,
+        Default::default(),
+    );
 
     // 1st group
     // expected boundary:
@@ -173,9 +216,18 @@ fn test_boundaries() {
     // 2021-12-16 01:30:00
     // 2021-12-16 02:00:00
     let g = take_groups_slice(&groups, 1, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -184,8 +236,14 @@ fn test_boundaries() {
             t2.timestamp_nanos()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[1], higher[1]],
         &[b_start.timestamp_nanos(), b_end.timestamp_nanos()]
@@ -194,19 +252,43 @@ fn test_boundaries() {
     assert_eq!(groups[2], [4, 3]);
 
     // test closed: "left" (should not include right end of interval)
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::Left, TimeUnit::Nanoseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Left,
+        TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups[0], [0, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [2, 2]); // 01:00:00 -> 01:30:00
     assert_eq!(groups[2], [4, 2]); // 02:00:00 -> 02:30:00
 
     // test closed: "right" (should not include left end of interval)
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::Right, TimeUnit::Nanoseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Right,
+        TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups[0], [1, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 2]); // 01:00:00 -> 01:30:00
     assert_eq!(groups[2], [5, 2]); // 02:00:00 -> 02:30:00
 
     // test closed: "none" (should not include left or right end of interval)
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::None, TimeUnit::Nanoseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::None,
+        TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups[0], [1, 1]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 1]); // 01:00:00 -> 01:30:00
     assert_eq!(groups[2], [5, 1]); // 02:00:00 -> 02:30:00
@@ -214,8 +296,14 @@ fn test_boundaries() {
 
 #[test]
 fn test_boundaries_2() {
-    let start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let stop = NaiveDate::from_ymd(2021, 12, 16).and_hms(4, 0, 0);
+    let start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let stop = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(4, 0, 0)
+        .unwrap();
 
     let ts = date_range_vec(
         start.timestamp_nanos(),
@@ -239,8 +327,15 @@ fn test_boundaries_2() {
 
     assert_eq!(b.start, start.timestamp_nanos() + offset.duration_ns());
 
-    let (groups, lower, higher) =
-        groupby_windows(w, &ts, true, ClosedWindow::Left, TimeUnit::Nanoseconds);
+    let (groups, lower, higher) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Left,
+        TimeUnit::Nanoseconds,
+        true,
+        true,
+        Default::default(),
+    );
 
     // 1st group
     // expected boundary:
@@ -307,8 +402,15 @@ fn test_boundaries_ms() {
     assert_eq!(b.start, start.timestamp_millis());
 
     // test closed: "both" (includes both ends of the interval)
-    let (groups, lower, higher) =
-        groupby_windows(w, &ts, true, ClosedWindow::Both, TimeUnit::Milliseconds);
+    let (groups, lower, higher) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Both,
+        TimeUnit::Milliseconds,
+        true,
+        true,
+        Default::default(),
+    );
 
     // 1st group
     // expected boundary:
@@ -318,9 +420,18 @@ fn test_boundaries_ms() {
     // 2021-12-16 00:30:00
     // 2021-12-16 01:00:00
     let g = take_groups_slice(&groups, 0, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -329,8 +440,14 @@ fn test_boundaries_ms() {
             t2.timestamp_millis()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(0, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[0], higher[0]],
         &[b_start.timestamp_millis(), b_end.timestamp_millis()]
@@ -344,9 +461,18 @@ fn test_boundaries_ms() {
     // 2021-12-16 01:30:00
     // 2021-12-16 02:00:00
     let g = take_groups_slice(&groups, 1, &ts);
-    let t0 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let t1 = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 30, 0);
-    let t2 = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let t0 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let t1 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 30, 0)
+        .unwrap();
+    let t2 = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         g,
         &[
@@ -355,8 +481,14 @@ fn test_boundaries_ms() {
             t2.timestamp_millis()
         ]
     );
-    let b_start = NaiveDate::from_ymd(2021, 12, 16).and_hms(1, 0, 0);
-    let b_end = NaiveDate::from_ymd(2021, 12, 16).and_hms(2, 0, 0);
+    let b_start = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(1, 0, 0)
+        .unwrap();
+    let b_end = NaiveDate::from_ymd_opt(2021, 12, 16)
+        .unwrap()
+        .and_hms_opt(2, 0, 0)
+        .unwrap();
     assert_eq!(
         &[lower[1], higher[1]],
         &[b_start.timestamp_millis(), b_end.timestamp_millis()]
@@ -365,20 +497,43 @@ fn test_boundaries_ms() {
     assert_eq!(groups[2], [4, 3]);
 
     // test closed: "left" (should not include right end of interval)
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::Left, TimeUnit::Milliseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Left,
+        TimeUnit::Milliseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups[0], [0, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [2, 2]); // 01:00:00 -> 01:30:00
     assert_eq!(groups[2], [4, 2]); // 02:00:00 -> 02:30:00
 
     // test closed: "right" (should not include left end of interval)
-    let (groups, _, _) =
-        groupby_windows(w, &ts, false, ClosedWindow::Right, TimeUnit::Milliseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::Right,
+        TimeUnit::Milliseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups[0], [1, 2]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 2]); // 01:00:00 -> 01:30:00
     assert_eq!(groups[2], [5, 2]); // 02:00:00 -> 02:30:00
 
     // test closed: "none" (should not include left or right end of interval)
-    let (groups, _, _) = groupby_windows(w, &ts, false, ClosedWindow::None, TimeUnit::Milliseconds);
+    let (groups, _, _) = groupby_windows(
+        w,
+        &ts,
+        ClosedWindow::None,
+        TimeUnit::Milliseconds,
+        false,
+        false,
+        Default::default(),
+    );
     assert_eq!(groups[0], [1, 1]); // 00:00:00 -> 00:30:00
     assert_eq!(groups[1], [3, 1]); // 01:00:00 -> 01:30:00
     assert_eq!(groups[2], [5, 1]); // 02:00:00 -> 02:30:00
@@ -387,8 +542,14 @@ fn test_boundaries_ms() {
 #[test]
 fn test_rolling_lookback() {
     // Test month as interval in date range
-    let start = NaiveDate::from_ymd(1970, 1, 16).and_hms(0, 0, 0);
-    let end = NaiveDate::from_ymd(1970, 1, 16).and_hms(4, 0, 0);
+    let start = NaiveDate::from_ymd_opt(1970, 1, 16)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let end = NaiveDate::from_ymd_opt(1970, 1, 16)
+        .unwrap()
+        .and_hms_opt(4, 0, 0)
+        .unwrap();
     let dates = date_range_vec(
         start.timestamp_millis(),
         end.timestamp_millis(),
@@ -504,9 +665,11 @@ fn test_end_membership() {
     let (groups, _, _) = groupby_windows(
         window,
         &time,
-        false,
         ClosedWindow::Left,
         TimeUnit::Milliseconds,
+        false,
+        false,
+        Default::default(),
     );
     assert_eq!(groups[0], [0, 1]);
     assert_eq!(groups[1], [0, 1]);
@@ -525,9 +688,11 @@ fn test_groupby_windows_membership_2791() {
     let (groups, _, _) = groupby_windows(
         window,
         &dates,
-        false,
         ClosedWindow::Left,
         TimeUnit::Milliseconds,
+        false,
+        false,
+        Default::default(),
     );
     assert_eq!(groups[0], [0, 2]);
     assert_eq!(groups[1], [2, 2]);
@@ -545,9 +710,11 @@ fn test_groupby_windows_duplicates_2931() {
     let (groups, _, _) = groupby_windows(
         window,
         &dates,
-        false,
         ClosedWindow::Left,
         TimeUnit::Milliseconds,
+        false,
+        false,
+        Default::default(),
     );
     assert_eq!(groups, [[0, 1], [1, 2], [3, 2]]);
 }
@@ -555,13 +722,13 @@ fn test_groupby_windows_duplicates_2931() {
 #[test]
 fn test_groupby_windows_offsets_3776() {
     let dates = &[
-        NaiveDate::from_ymd(2020, 12, 1),
-        NaiveDate::from_ymd(2021, 2, 1),
-        NaiveDate::from_ymd(2021, 5, 1),
+        NaiveDate::from_ymd_opt(2020, 12, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2021, 2, 1).unwrap(),
+        NaiveDate::from_ymd_opt(2021, 5, 1).unwrap(),
     ];
     let ts = dates
         .iter()
-        .map(|d| d.and_hms(0, 0, 0).timestamp_nanos())
+        .map(|d| d.and_hms_opt(0, 0, 0).unwrap().timestamp_nanos())
         .collect::<Vec<_>>();
 
     let window = Window::new(
@@ -572,9 +739,11 @@ fn test_groupby_windows_offsets_3776() {
     let (groups, _, _) = groupby_windows(
         window,
         &ts,
-        false,
         ClosedWindow::Right,
         TimeUnit::Nanoseconds,
+        false,
+        false,
+        Default::default(),
     );
     assert_eq!(groups, [[0, 1], [1, 1], [2, 1]]);
 }
