@@ -1,22 +1,30 @@
-use super::*;
 use polars::export::chrono::prelude::*;
 
+use super::*;
+
 #[test]
 #[cfg(all(
     feature = "temporal",
     feature = "dtype-date",
     feature = "dynamic_groupby"
 ))]
-fn test_groupby_dynamic_week_bounds() -> Result<()> {
-    let start = NaiveDate::from_ymd(2022, 2, 1).and_hms(0, 0, 0);
-    let stop = NaiveDate::from_ymd(2022, 2, 14).and_hms(0, 0, 0);
-    let range = date_range(
+fn test_groupby_dynamic_week_bounds() -> PolarsResult<()> {
+    let start = NaiveDate::from_ymd_opt(2022, 2, 1)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let stop = NaiveDate::from_ymd_opt(2022, 2, 14)
+        .unwrap()
+        .and_hms_opt(0, 0, 0)
+        .unwrap();
+    let range = polars_time::date_range(
         "dt",
         start,
         stop,
         Duration::parse("1d"),
         ClosedWindow::Left,
         TimeUnit::Milliseconds,
+        None,
     )
     .into_series();
 
@@ -38,6 +46,7 @@ fn test_groupby_dynamic_week_bounds() -> Result<()> {
                 closed_window: ClosedWindow::Left,
                 truncate: false,
                 include_boundaries: true,
+                ..Default::default()
             },
         )
         .agg([col("a").sum()])
