@@ -3,7 +3,15 @@ from __future__ import annotations
 from typing import TYPE_CHECKING
 
 import polars.internals as pli
-from polars.datatypes import DataType, Date, Datetime, Time
+from polars.datatypes import (
+    DataType,
+    Date,
+    Datetime,
+    TemporalDataType,
+    Time,
+    is_polars_dtype,
+)
+from polars.utils import deprecated_alias
 
 if TYPE_CHECKING:
     from polars.internals.type_aliases import TransferEncoding
@@ -12,15 +20,18 @@ if TYPE_CHECKING:
 class ExprStringNameSpace:
     """Namespace for string related expressions."""
 
+    _accessor = "str"
+
     def __init__(self, expr: pli.Expr):
         self._pyexpr = expr._pyexpr
 
     def strptime(
         self,
-        datatype: type[Date] | type[Datetime] | type[Time],
+        datatype: TemporalDataType,
         fmt: str | None = None,
         strict: bool = True,
         exact: bool = True,
+        cache: bool = True,
     ) -> pli.Expr:
         """
         Parse a Utf8 expression to a Date/Datetime/Time type.
@@ -28,7 +39,7 @@ class ExprStringNameSpace:
         Parameters
         ----------
         datatype
-            Date | Datetime | Time.
+            Date | Datetime | Time
         fmt
             Format to use, refer to the `chrono strftime documentation
             <https://docs.rs/chrono/latest/chrono/format/strftime/index.html>`_
@@ -38,6 +49,14 @@ class ExprStringNameSpace:
         exact
             - If True, require an exact format match.
             - If False, allow the format to match anywhere in the target string.
+        cache
+            Use a cache of unique, converted dates to apply the datetime conversion.
+
+        Notes
+        -----
+        When parsing a Datetime the column precision will be inferred from
+        the format string, if given, eg: "%F %T%.3f" => Datetime("ms"). If
+        no fractional second component is found then the default is "us".
 
         Examples
         --------
@@ -79,41 +98,94 @@ class ExprStringNameSpace:
         └────────────┘
 
         """
-        if not issubclass(datatype, DataType):  # pragma: no cover
+        if not is_polars_dtype(datatype):  # pragma: no cover
             raise ValueError(f"expected: {DataType} got: {datatype}")
+
         if datatype == Date:
-            return pli.wrap_expr(self._pyexpr.str_parse_date(fmt, strict, exact))
+            return pli.wrap_expr(self._pyexpr.str_parse_date(fmt, strict, exact, cache))
         elif datatype == Datetime:
-            return pli.wrap_expr(self._pyexpr.str_parse_datetime(fmt, strict, exact))
+            tu = datatype.tu  # type: ignore[union-attr]
+            dtcol = pli.wrap_expr(
+                self._pyexpr.str_parse_datetime(fmt, strict, exact, cache)
+            )
+            return dtcol if (tu is None) else dtcol.dt.cast_time_unit(tu)
         elif datatype == Time:
-            return pli.wrap_expr(self._pyexpr.str_parse_time(fmt, strict, exact))
+            return pli.wrap_expr(self._pyexpr.str_parse_time(fmt, strict, exact, cache))
         else:  # pragma: no cover
             raise ValueError("dtype should be of type {Date, Datetime, Time}")
 
     def lengths(self) -> pli.Expr:
         """
-        Get the length of the Strings as UInt32.
+        Get length of the strings as UInt32 (as number of bytes).
+
+        Notes
+        -----
+        The returned lengths are equal to the number of bytes in the UTF8 string. If you
+        need the length in terms of the number of characters, use ``n_chars`` instead.
 
         Examples
         --------
-        >>> df = pl.DataFrame({"s": [None, "bears", "110"]})
-        >>> df.select(["s", pl.col("s").str.lengths().alias("len")])
-        shape: (3, 2)
-        ┌───────┬──────┐
-        │ s     ┆ len  │
-        │ ---   ┆ ---  │
-        │ str   ┆ u32  │
-        ╞═══════╪══════╡
-        │ null  ┆ null │
-        ├╌╌╌╌╌╌╌┼╌╌╌╌╌╌┤
-        │ bears ┆ 5    │
-        ├╌╌╌╌╌╌╌┼╌╌╌╌╌╌┤
-        │ 110   ┆ 3    │
-        └───────┴──────┘
+        >>> df = pl.DataFrame({"s": ["Café", None, "345", "東京"]}).with_columns(
+        ...     [
+        ...         pl.col("s").str.lengths().alias("length"),
+        ...         pl.col("s").str.n_chars().alias("nchars"),
+        ...     ]
+        ... )
+        >>> df
+        shape: (4, 3)
+        ┌──────┬────────┬────────┐
+        │ s    ┆ length ┆ nchars │
+        │ ---  ┆ ---    ┆ ---    │
+        │ str  ┆ u32    ┆ u32    │
+        ╞══════╪════════╪════════╡
+        │ Café ┆ 5      ┆ 4      │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ null ┆ null   ┆ null   │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 345  ┆ 3      ┆ 3      │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 東京  ┆ 6      ┆ 2      │
+        └──────┴────────┴────────┘
 
         """
         return pli.wrap_expr(self._pyexpr.str_lengths())
 
+    def n_chars(self) -> pli.Expr:
+        """
+        Get length of the strings as UInt32 (as number of chars).
+
+        Notes
+        -----
+        If you know that you are working with ASCII text, ``lengths`` will be
+        equivalent, and faster (returns length in terms of the number of bytes).
+
+        Examples
+        --------
+        >>> df = pl.DataFrame({"s": ["Café", None, "345", "東京"]}).with_columns(
+        ...     [
+        ...         pl.col("s").str.n_chars().alias("nchars"),
+        ...         pl.col("s").str.lengths().alias("length"),
+        ...     ]
+        ... )
+        >>> df
+        shape: (4, 3)
+        ┌──────┬────────┬────────┐
+        │ s    ┆ nchars ┆ length │
+        │ ---  ┆ ---    ┆ ---    │
+        │ str  ┆ u32    ┆ u32    │
+        ╞══════╪════════╪════════╡
+        │ Café ┆ 4      ┆ 5      │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ null ┆ null   ┆ null   │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 345  ┆ 3      ┆ 3      │
+        ├╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌┤
+        │ 東京  ┆ 2      ┆ 6      │
+        └──────┴────────┴────────┘
+
+        """
+        return pli.wrap_expr(self._pyexpr.str_n_chars())
+
     def concat(self, delimiter: str = "-") -> pli.Expr:
         """
         Vertically concat the values in the Series to a single string value.
@@ -144,27 +216,141 @@ class ExprStringNameSpace:
         return pli.wrap_expr(self._pyexpr.str_concat(delimiter))
 
     def to_uppercase(self) -> pli.Expr:
-        """Transform to uppercase variant."""
+        """
+        Transform to uppercase variant.
+
+        Examples
+        --------
+        >>> df = pl.DataFrame({"foo": ["cat", "dog"]})
+        >>> df.select(pl.col("foo").str.to_uppercase())
+        shape: (2, 1)
+        ┌─────┐
+        │ foo │
+        │ --- │
+        │ str │
+        ╞═════╡
+        │ CAT │
+        ├╌╌╌╌╌┤
+        │ DOG │
+        └─────┘
+
+        """
         return pli.wrap_expr(self._pyexpr.str_to_uppercase())
 
     def to_lowercase(self) -> pli.Expr:
-        """Transform to lowercase variant."""
+        """
+        Transform to lowercase variant.
+
+        Examples
+        --------
+        >>> df = pl.DataFrame({"foo": ["CAT", "DOG"]})
+        >>> df.select(pl.col("foo").str.to_lowercase())
+        shape: (2, 1)
+        ┌─────┐
+        │ foo │
+        │ --- │
+        │ str │
+        ╞═════╡
+        │ cat │
+        ├╌╌╌╌╌┤
+        │ dog │
+        └─────┘
+
+        """
         return pli.wrap_expr(self._pyexpr.str_to_lowercase())
 
-    def strip(self) -> pli.Expr:
-        """Remove leading and trailing whitespace."""
-        return pli.wrap_expr(self._pyexpr.str_strip())
+    def strip(self, matches: None | str = None) -> pli.Expr:
+        """
+        Remove leading and trailing whitespace.
 
-    def lstrip(self) -> pli.Expr:
-        """Remove leading whitespace."""
-        return pli.wrap_expr(self._pyexpr.str_lstrip())
+        Parameters
+        ----------
+        matches
+            An optional single character that should be trimmed
 
-    def rstrip(self) -> pli.Expr:
-        """Remove trailing whitespace."""
-        return pli.wrap_expr(self._pyexpr.str_rstrip())
+        Examples
+        --------
+        >>> df = pl.DataFrame({"foo": [" lead", "trail ", " both "]})
+        >>> df.select(pl.col("foo").str.strip())
+        shape: (3, 1)
+        ┌───────┐
+        │ foo   │
+        │ ---   │
+        │ str   │
+        ╞═══════╡
+        │ lead  │
+        ├╌╌╌╌╌╌╌┤
+        │ trail │
+        ├╌╌╌╌╌╌╌┤
+        │ both  │
+        └───────┘
+
+        """
+        if matches is not None and len(matches) > 1:
+            raise ValueError("matches should contain a single character")
+        return pli.wrap_expr(self._pyexpr.str_strip(matches))
+
+    def lstrip(self, matches: None | str = None) -> pli.Expr:
+        """
+        Remove leading whitespace.
+
+        Examples
+        --------
+        >>> df = pl.DataFrame({"foo": [" lead", "trail ", " both "]})
+        >>> df.select(pl.col("foo").str.lstrip())
+        shape: (3, 1)
+        ┌────────┐
+        │ foo    │
+        │ ---    │
+        │ str    │
+        ╞════════╡
+        │ lead   │
+        ├╌╌╌╌╌╌╌╌┤
+        │ trail  │
+        ├╌╌╌╌╌╌╌╌┤
+        │ both   │
+        └────────┘
+
+        """
+        if matches is not None and len(matches) > 1:
+            raise ValueError("matches should contain a single character")
+        return pli.wrap_expr(self._pyexpr.str_lstrip(matches))
+
+    def rstrip(self, matches: None | str = None) -> pli.Expr:
+        """
+        Remove trailing whitespace.
+
+        Parameters
+        ----------
+        matches
+            An optional single character that should be trimmed
+
+        Examples
+        --------
+        >>> df = pl.DataFrame({"foo": [" lead", "trail ", " both "]})
+        >>> df.select(pl.col("foo").str.rstrip())
+        shape: (3, 1)
+        ┌───────┐
+        │ foo   │
+        │ ---   │
+        │ str   │
+        ╞═══════╡
+        │  lead │
+        ├╌╌╌╌╌╌╌┤
+        │ trail │
+        ├╌╌╌╌╌╌╌┤
+        │  both │
+        └───────┘
+
+        """
+        if matches is not None and len(matches) > 1:
+            raise ValueError("matches should contain a single character")
+        return pli.wrap_expr(self._pyexpr.str_rstrip(matches))
 
     def zfill(self, alignment: int) -> pli.Expr:
         """
+        Fills the string with zeroes.
+
         Return a copy of the string left filled with ASCII '0' digits to make a string
         of length width.
 
@@ -430,6 +616,7 @@ class ExprStringNameSpace:
     def json_path_match(self, json_path: str) -> pli.Expr:
         """
         Extract the first match of json string with provided JSONPath expression.
+
         Throw errors if encounter invalid json strings.
         All return value will be casted to Utf8 regardless of the original value.
 
@@ -604,7 +791,9 @@ class ExprStringNameSpace:
 
     def extract_all(self, pattern: str) -> pli.Expr:
         r"""
-        Extract each successive non-overlapping regex match in an individual string as
+        Extracts all matches for the given regex pattern.
+
+        Extracts each successive non-overlapping regex match in an individual string as
         an array.
 
         Parameters
@@ -713,7 +902,9 @@ class ExprStringNameSpace:
 
     def split_exact(self, by: str, n: int, inclusive: bool = False) -> pli.Expr:
         """
-        Split the string by a substring into a struct of ``n`` fields.
+        Split the string by a substring using ``n`` splits.
+
+        Results in a struct of ``n+1`` fields.
 
         If it cannot make ``n`` splits, the remaining field elements will be null.
 
@@ -728,12 +919,11 @@ class ExprStringNameSpace:
 
         Examples
         --------
-        >>> (
-        ...     pl.DataFrame({"x": ["a_1", None, "c", "d_4"]}).select(
-        ...         [
-        ...             pl.col("x").str.split_exact("_", 1).alias("fields"),
-        ...         ]
-        ...     )
+        >>> df = pl.DataFrame({"x": ["a_1", None, "c", "d_4"]})
+        >>> df.select(
+        ...     [
+        ...         pl.col("x").str.split_exact("_", 1).alias("fields"),
+        ...     ]
         ... )
         shape: (4, 1)
         ┌─────────────┐
@@ -754,7 +944,7 @@ class ExprStringNameSpace:
         Split string values in column x in exactly 2 parts and assign
         each part to a new column.
 
-        >>> pl.DataFrame({"x": ["a_1", None, "c", "d_4"]}).with_columns(
+        >>> df.with_columns(
         ...     [
         ...         pl.col("x")
         ...         .str.split_exact("_", 1)
@@ -786,7 +976,76 @@ class ExprStringNameSpace:
             return pli.wrap_expr(self._pyexpr.str_split_exact_inclusive(by, n))
         return pli.wrap_expr(self._pyexpr.str_split_exact(by, n))
 
-    def replace(self, pattern: str, value: str, literal: bool = False) -> pli.Expr:
+    def splitn(self, by: str, n: int) -> pli.Expr:
+        """
+        Split the string by a substring, restricted to returning at most ``n`` items.
+
+        If the number of possible splits is less than ``n-1``, the remaining field
+        elements will be null. If the number of possible splits is ``n-1`` or greater,
+        the last (nth) substring will contain the remainder of the string.
+
+        Parameters
+        ----------
+        by
+            Substring to split by.
+        n
+            Max number of items to return.
+
+        Examples
+        --------
+        >>> df = pl.DataFrame({"s": ["foo bar", None, "foo-bar", "foo bar baz"]})
+        >>> df.select(pl.col("s").str.splitn(" ", 2).alias("fields"))
+        shape: (4, 1)
+        ┌───────────────────┐
+        │ fields            │
+        │ ---               │
+        │ struct[2]         │
+        ╞═══════════════════╡
+        │ {"foo","bar"}     │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ {null,null}       │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ {"foo-bar",null}  │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ {"foo","bar baz"} │
+        └───────────────────┘
+
+        Split string values in column s in exactly 2 parts and assign
+        each part to a new column.
+
+        >>> df.with_columns(
+        ...     [
+        ...         pl.col("s")
+        ...         .str.splitn(" ", 2)
+        ...         .struct.rename_fields(["first_part", "second_part"])
+        ...         .alias("fields"),
+        ...     ]
+        ... ).unnest("fields")
+        shape: (4, 3)
+        ┌─────────────┬────────────┬─────────────┐
+        │ s           ┆ first_part ┆ second_part │
+        │ ---         ┆ ---        ┆ ---         │
+        │ str         ┆ str        ┆ str         │
+        ╞═════════════╪════════════╪═════════════╡
+        │ foo bar     ┆ foo        ┆ bar         │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ null        ┆ null       ┆ null        │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ foo-bar     ┆ foo-bar    ┆ null        │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ foo bar baz ┆ foo        ┆ bar baz     │
+        └─────────────┴────────────┴─────────────┘
+
+        Returns
+        -------
+        Struct of Utf8 type
+
+        """
+        return pli.wrap_expr(self._pyexpr.str_splitn(by, n))
+
+    def replace(
+        self, pattern: str | pli.Expr, value: str | pli.Expr, literal: bool = False
+    ) -> pli.Expr:
         r"""
         Replace first matching regex/literal substring with a new string value.
 
@@ -821,9 +1080,15 @@ class ExprStringNameSpace:
         └─────┴────────┘
 
         """
-        return pli.wrap_expr(self._pyexpr.str_replace(pattern, value, literal))
+        pattern = pli.expr_to_lit_or_expr(pattern, str_to_lit=True)
+        value = pli.expr_to_lit_or_expr(value, str_to_lit=True)
+        return pli.wrap_expr(
+            self._pyexpr.str_replace(pattern._pyexpr, value._pyexpr, literal)
+        )
 
-    def replace_all(self, pattern: str, value: str, literal: bool = False) -> pli.Expr:
+    def replace_all(
+        self, pattern: str | pli.Expr, value: str | pli.Expr, literal: bool = False
+    ) -> pli.Expr:
         """
         Replace all matching regex/literal substrings with a new string value.
 
@@ -856,24 +1121,29 @@ class ExprStringNameSpace:
         └─────┴─────────┘
 
         """
-        return pli.wrap_expr(self._pyexpr.str_replace_all(pattern, value, literal))
+        pattern = pli.expr_to_lit_or_expr(pattern, str_to_lit=True)
+        value = pli.expr_to_lit_or_expr(value, str_to_lit=True)
+        return pli.wrap_expr(
+            self._pyexpr.str_replace_all(pattern._pyexpr, value._pyexpr, literal)
+        )
 
-    def slice(self, start: int, length: int | None = None) -> pli.Expr:
+    @deprecated_alias(start="offset")
+    def slice(self, offset: int, length: int | None = None) -> pli.Expr:
         """
         Create subslices of the string values of a Utf8 Series.
 
         Parameters
         ----------
-        start
-            Starting index of the slice (zero-indexed). Negative indexing
-            may be used.
+        offset
+            Start index. Negative indexing is supported.
         length
-            Optional length of the slice. If None (default), the slice is taken to the
+            Length of the slice. If set to ``None`` (default), the slice is taken to the
             end of the string.
 
         Returns
         -------
-        Series of Utf8 type
+        Expr
+            Series of dtype Utf8.
 
         Examples
         --------
@@ -917,4 +1187,4 @@ class ExprStringNameSpace:
         └─────────────┴──────────┘
 
         """
-        return pli.wrap_expr(self._pyexpr.str_slice(start, length))
+        return pli.wrap_expr(self._pyexpr.str_slice(offset, length))
