@@ -7,16 +7,21 @@ from typing import TYPE_CHECKING, Any, Callable, Sequence, cast, overload
 
 from polars import internals as pli
 from polars.datatypes import (
+    DTYPE_TEMPORAL_UNITS,
     DataType,
     Date,
     Datetime,
     Duration,
+    Int64,
     PolarsDataType,
     Time,
     UInt32,
     is_polars_dtype,
     py_type_to_dtype,
 )
+from polars.dependencies import _NUMPY_TYPE
+from polars.dependencies import numpy as np
+from polars.internals.type_aliases import EpochTimeUnit
 from polars.utils import (
     _datetime_to_pl_timestamp,
     _time_to_pl_time,
@@ -37,6 +42,7 @@ try:
     from polars.polars import count as _count
     from polars.polars import cov as pycov
     from polars.polars import cumfold as pycumfold
+    from polars.polars import cumreduce as pycumreduce
     from polars.polars import dtype_cols as _dtype_cols
     from polars.polars import first as _first
     from polars.polars import fold as pyfold
@@ -47,6 +53,7 @@ try:
     from polars.polars import min_exprs as _min_exprs
     from polars.polars import pearson_corr as pypearson_corr
     from polars.polars import py_datetime, py_duration
+    from polars.polars import reduce as pyreduce
     from polars.polars import repeat as _repeat
     from polars.polars import spearman_rank_corr as pyspearman_rank_corr
     from polars.polars import sum_exprs as _sum_exprs
@@ -55,20 +62,17 @@ try:
 except ImportError:
     _DOCUMENTING = True
 
-try:
-    import numpy as np
-
-    _NUMPY_AVAILABLE = True
-except ImportError:
-    _NUMPY_AVAILABLE = False
-
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
     from typing_extensions import Literal
 
 if TYPE_CHECKING:
-    from polars.internals.type_aliases import InterpolationMethod, IntoExpr, TimeUnit
+    from polars.internals.type_aliases import (
+        IntoExpr,
+        RollingInterpolationMethod,
+        TimeUnit,
+    )
 
 
 def col(
@@ -732,7 +736,11 @@ def lit(
     tu: TimeUnit
     if isinstance(value, datetime):
         tu = "us"
-        return lit(_datetime_to_pl_timestamp(value, tu)).cast(Datetime(tu))
+        e = lit(_datetime_to_pl_timestamp(value, tu)).cast(Datetime(tu))
+        if value.tzinfo is not None:
+            return e.dt.tz_localize(str(value.tzinfo))
+        else:
+            return e
 
     elif isinstance(value, timedelta):
         tu = "us"
@@ -752,7 +760,7 @@ def lit(
             return e
         return e.alias(name)
 
-    if _NUMPY_AVAILABLE and isinstance(value, np.ndarray):
+    if _NUMPY_TYPE(value) and isinstance(value, np.ndarray):
         return lit(pli.Series("", value))
 
     if dtype:
@@ -795,7 +803,7 @@ def cumsum(
     column: str | Sequence[pli.Expr | str] | pli.Series | pli.Expr,
 ) -> pli.Expr | Any:
     """
-    Cumulatively sum values in a column/Series, or horizontally across list of columns/expressions.  # noqa E501
+    Cumulatively sum values in a column/Series, or horizontally across list of columns/expressions.
 
     ``pl.cumsum(str)`` is syntactic sugar for:
 
@@ -865,7 +873,7 @@ def cumsum(
     │ 2   ┆ 4   ┆ 6   ┆ {2,8}     │
     └─────┴─────┴─────┴───────────┘
 
-    """
+    """  # noqa E501
     if isinstance(column, pli.Series):
         return column.cumsum()
     elif isinstance(column, str):
@@ -973,13 +981,16 @@ def map(
 
     """
     exprs = pli.selection_to_pyexpr_list(exprs)
-    return pli.wrap_expr(_map_mul(exprs, f, return_dtype, apply_groups=False))
+    return pli.wrap_expr(
+        _map_mul(exprs, f, return_dtype, apply_groups=False, returns_scalar=False)
+    )
 
 
 def apply(
     exprs: Sequence[str | pli.Expr],
     f: Callable[[Sequence[pli.Series]], pli.Series | Any],
     return_dtype: type[DataType] | None = None,
+    returns_scalar: bool = True,
 ) -> pli.Expr:
     """
     Apply a custom/user-defined function (UDF) in a GroupBy context.
@@ -1000,6 +1011,8 @@ def apply(
         Function to apply over the input
     return_dtype
         dtype of the output Series
+    returns_scalar
+        If the function returns a single scalar as output.
 
     Returns
     -------
@@ -1007,7 +1020,11 @@ def apply(
 
     """
     exprs = pli.selection_to_pyexpr_list(exprs)
-    return pli.wrap_expr(_map_mul(exprs, f, return_dtype, apply_groups=True))
+    return pli.wrap_expr(
+        _map_mul(
+            exprs, f, return_dtype, apply_groups=True, returns_scalar=returns_scalar
+        )
+    )
 
 
 def fold(
@@ -1029,6 +1046,11 @@ def fold(
     exprs
         Expressions to aggregate over. May also be a wildcard expression.
 
+    Notes
+    -----
+    If you simply want the first encountered expression as accumulator,
+    consider using ``reduce``.
+
     """
     # in case of pl.col("*")
     acc = pli.expr_to_lit_or_expr(acc, str_to_lit=True)
@@ -1039,6 +1061,34 @@ def fold(
     return pli.wrap_expr(pyfold(acc._pyexpr, f, exprs))
 
 
+def reduce(
+    f: Callable[[pli.Series, pli.Series], pli.Series],
+    exprs: Sequence[pli.Expr | str] | pli.Expr,
+) -> pli.Expr:
+    """
+    Accumulate over multiple columns horizontally/ row wise with a left fold.
+
+    Parameters
+    ----------
+    f
+        Function to apply over the accumulator and the value.
+        Fn(acc, value) -> new_value
+    exprs
+        Expressions to aggregate over. May also be a wildcard expression.
+
+    Notes
+    -----
+    See ``fold`` for the version with an explicit accumulator.
+
+    """
+    # in case of pl.col("*")
+    if isinstance(exprs, pli.Expr):
+        exprs = [exprs]
+
+    exprs = pli.selection_to_pyexpr_list(exprs)
+    return pli.wrap_expr(pyreduce(f, exprs))
+
+
 def cumfold(
     acc: IntoExpr,
     f: Callable[[pli.Series, pli.Series], pli.Series],
@@ -1046,7 +1096,7 @@ def cumfold(
     include_init: bool = False,
 ) -> pli.Expr:
     """
-    Cumulatively accumulate over multiple columns horizontally/ row wise with a left fold.  # noqa E501
+    Cumulatively accumulate over multiple columns horizontally/ row wise with a left fold.
 
     Every cumulative result is added as a separate field in a Struct column.
 
@@ -1063,7 +1113,12 @@ def cumfold(
     include_init
         Include the initial accumulator state as struct field.
 
-    """
+    Notes
+    -----
+    If you simply want the first encountered expression as accumulator,
+    consider using ``cumreduce``.
+
+    """  # noqa E501
     # in case of pl.col("*")
     acc = pli.expr_to_lit_or_expr(acc, str_to_lit=True)
     if isinstance(exprs, pli.Expr):
@@ -1073,6 +1128,32 @@ def cumfold(
     return pli.wrap_expr(pycumfold(acc._pyexpr, f, exprs, include_init))
 
 
+def cumreduce(
+    f: Callable[[pli.Series, pli.Series], pli.Series],
+    exprs: Sequence[pli.Expr | str] | pli.Expr,
+) -> pli.Expr:
+    """
+    Cumulatively accumulate over multiple columns horizontally/ row wise with a left fold.
+
+    Every cumulative result is added as a separate field in a Struct column.
+
+    Parameters
+    ----------
+    f
+        Function to apply over the accumulator and the value.
+        Fn(acc, value) -> new_value
+    exprs
+        Expressions to aggregate over. May also be a wildcard expression.
+
+    """  # noqa E501
+    # in case of pl.col("*")
+    if isinstance(exprs, pli.Expr):
+        exprs = [exprs]
+
+    exprs = pli.selection_to_pyexpr_list(exprs)
+    return pli.wrap_expr(pycumreduce(f, exprs))
+
+
 def any(name: str | Sequence[str] | Sequence[pli.Expr] | pli.Expr) -> pli.Expr:
     """Evaluate columnwise or elementwise with a bitwise OR operation."""
     if isinstance(name, str):
@@ -1231,7 +1312,7 @@ def groups(column: str) -> pli.Expr:
 
 
 def quantile(
-    column: str, quantile: float, interpolation: InterpolationMethod = "nearest"
+    column: str, quantile: float, interpolation: RollingInterpolationMethod = "nearest"
 ) -> pli.Expr:
     """
     Syntactic sugar for `pl.col("foo").quantile(..)`.
@@ -1267,6 +1348,7 @@ def arange(
     step: int = ...,
     *,
     eager: Literal[True],
+    dtype: PolarsDataType | None = ...,
 ) -> pli.Series:
     ...
 
@@ -1278,6 +1360,7 @@ def arange(
     step: int = ...,
     *,
     eager: bool = False,
+    dtype: PolarsDataType | None = ...,
 ) -> pli.Expr | pli.Series:
     ...
 
@@ -1288,13 +1371,13 @@ def arange(
     step: int = 1,
     *,
     eager: bool = False,
+    dtype: PolarsDataType | None = None,
 ) -> pli.Expr | pli.Series:
     """
-    Create a range expression.
+    Create a range expression (or Series).
 
-    This can be used in a `select`, `with_column` etc.
-
-    Be sure that the range size is equal to the DataFrame you are collecting.
+    This can be used in a `select`, `with_column` etc. Be sure that the resulting
+    range size is equal to the length of the DataFrame you are collecting.
 
     Examples
     --------
@@ -1310,16 +1393,25 @@ def arange(
         Step size of the range.
     eager
         If eager evaluation is `True`, a Series is returned instead of an Expr.
+    dtype
+        Apply an explicit integer dtype to the resulting expression (default is Int64).
 
     """
     low = pli.expr_to_lit_or_expr(low, str_to_lit=False)
     high = pli.expr_to_lit_or_expr(high, str_to_lit=False)
+    range_expr = pli.wrap_expr(pyarange(low._pyexpr, high._pyexpr, step))
 
-    if eager:
-        df = pli.DataFrame({"a": [1]})
-        return df.select(arange(low, high, step).alias("arange"))["arange"]
-
-    return pli.wrap_expr(pyarange(low._pyexpr, high._pyexpr, step))
+    if dtype is not None and dtype != Int64:
+        range_expr = range_expr.cast(dtype)
+    if not eager:
+        return range_expr
+    else:
+        return (
+            pli.DataFrame()
+            .select(range_expr)
+            .to_series()
+            .rename("arange", in_place=True)
+        )
 
 
 def argsort_by(
@@ -1685,6 +1777,7 @@ def collect_all(
     no_optimization: bool = False,
     slice_pushdown: bool = True,
     common_subplan_elimination: bool = True,
+    allow_streaming: bool = False,
 ) -> list[pli.DataFrame]:
     """
     Collect multiple LazyFrames at the same time.
@@ -1711,6 +1804,8 @@ def collect_all(
         Slice pushdown optimization.
     common_subplan_elimination
         Will try to cache branching subplans that occur on self-joins or unions.
+    allow_streaming
+        Run parts of the query in a streaming fashion (this is in an alpha state)
 
     Returns
     -------
@@ -1733,6 +1828,7 @@ def collect_all(
             simplify_expression,
             slice_pushdown,
             common_subplan_elimination,
+            allow_streaming,
         )
         prepared.append(ldf)
 
@@ -2005,7 +2101,8 @@ def arg_where(
 def coalesce(
     exprs: Sequence[
         pli.Expr | str | date | datetime | timedelta | int | float | bool | pli.Series
-    ],
+    ]
+    | pli.Expr,
 ) -> pli.Expr:
     """
     Folds the expressions from left to right, keeping the first non-null value.
@@ -2045,3 +2142,122 @@ def coalesce(
     """
     exprs = pli.selection_to_pyexpr_list(exprs)
     return pli.wrap_expr(_coalesce_exprs(exprs))
+
+
+@overload
+def from_epoch(
+    column: str | pli.Expr | pli.Series,
+    unit: EpochTimeUnit = ...,
+    *,
+    eager: Literal[False],
+) -> pli.Expr:
+    ...
+
+
+@overload
+def from_epoch(
+    column: str | pli.Expr | pli.Series | Sequence[int],
+    unit: EpochTimeUnit = ...,
+    *,
+    eager: Literal[True],
+) -> pli.Series:
+    ...
+
+
+@overload
+def from_epoch(
+    column: pli.Series | Sequence[int],
+    unit: EpochTimeUnit = ...,
+    *,
+    eager: Literal[True] = ...,
+) -> pli.Series:
+    ...
+
+
+@overload
+def from_epoch(
+    column: str | pli.Expr,
+    unit: EpochTimeUnit = ...,
+    *,
+    eager: Literal[False] = ...,
+) -> pli.Expr:
+    ...
+
+
+@overload
+def from_epoch(
+    column: str | pli.Expr | pli.Series | Sequence[int],
+    unit: EpochTimeUnit = ...,
+    *,
+    eager: bool = ...,
+) -> pli.Expr | pli.Series:
+    ...
+
+
+def from_epoch(
+    column: str | pli.Expr | pli.Series | Sequence[int],
+    unit: EpochTimeUnit = "s",
+    *,
+    eager: bool = False,
+) -> pli.Expr | pli.Series:
+    """
+    Utility function that parses an epoch timestamp (or Unix time) to Polars Date(time).
+
+    Depending on the `unit` provided, this function will return a different dtype:
+    - unit="d" returns pl.Date
+    - unit="s" returns pl.Datetime["us"] (pl.Datetime's default)
+    - unit="ms" returns pl.Datetime["ms"]
+    - unit="us" returns pl.Datetime["us"]
+    - unit="ns" returns pl.Datetime["ns"]
+
+    Parameters
+    ----------
+    column
+        Series or expression to parse integers to pl.Datetime.
+    unit
+        The unit of the timesteps since epoch time.
+    eager
+        If eager evaluation is `True`, a Series is returned instead of an Expr.
+
+    Examples
+    --------
+    >>> df = pl.DataFrame({"timestamp": [1666683077, 1666683099]}).lazy()
+    >>> df.select(pl.from_epoch(pl.col("timestamp"), unit="s")).collect()
+    shape: (2, 1)
+    ┌─────────────────────┐
+    │ timestamp           │
+    │ ---                 │
+    │ datetime[μs]        │
+    ╞═════════════════════╡
+    │ 2022-10-25 07:31:17 │
+    ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+    │ 2022-10-25 07:31:39 │
+    └─────────────────────┘
+
+    """
+    if isinstance(column, str):
+        column = col(column)
+    elif not isinstance(column, (pli.Series, pli.Expr)):
+        column = pli.Series(column)  # Sequence input handled by Series constructor
+
+    if unit == "d":
+        expr = column.cast(Date)
+    elif unit == "s":
+        expr = (column.cast(Int64) * 1_000_000).cast(Datetime("us"))
+    elif unit in DTYPE_TEMPORAL_UNITS:
+        expr = column.cast(Datetime(unit))
+    else:
+        raise ValueError(
+            f"'unit' must be one of {{'ns', 'us', 'ms', 's', 'd'}}, got '{unit}'."
+        )
+
+    if eager:
+        if not isinstance(column, pli.Series):
+            raise ValueError(
+                "expected 'Series or Sequence' in 'from_epoch' if 'eager=True', got"
+                f" {type(column)}"
+            )
+        else:
+            return column.to_frame().select(expr).to_series()
+    else:
+        return expr
