@@ -3,7 +3,7 @@ from __future__ import annotations
 from typing import TYPE_CHECKING
 
 import polars.internals as pli
-from polars.datatypes import Date, Datetime, Time
+from polars.datatypes import TemporalDataType
 from polars.internals.series.utils import expr_dispatch
 from polars.utils import deprecated_alias
 
@@ -23,10 +23,11 @@ class StringNameSpace:
 
     def strptime(
         self,
-        datatype: type[Date] | type[Datetime] | type[Time],
+        datatype: TemporalDataType,
         fmt: str | None = None,
         strict: bool = True,
         exact: bool = True,
+        cache: bool = True,
     ) -> pli.Series:
         """
         Parse a Series of dtype Utf8 to a Date/Datetime Series.
@@ -45,6 +46,8 @@ class StringNameSpace:
         exact
             - If True, require an exact format match.
             - If False, allow the format to match anywhere in the target string.
+        cache
+            Use a cache of unique, converted dates to apply the datetime conversion.
 
         Returns
         -------
@@ -93,7 +96,12 @@ class StringNameSpace:
 
     def lengths(self) -> pli.Series:
         """
-        Get length of the string values in the Series.
+        Get length of the string values in the Series (as number of bytes).
+
+        Notes
+        -----
+        The returned lengths are equal to the number of bytes in the UTF8 string. If you
+        need the length in terms of the number of characters, use ``n_chars`` instead.
 
         Returns
         -------
@@ -101,15 +109,43 @@ class StringNameSpace:
 
         Examples
         --------
-        >>> s = pl.Series(["foo", None, "hello", "world"])
+        >>> s = pl.Series(["Café", None, "345", "東京"])
         >>> s.str.lengths()
         shape: (4,)
         Series: '' [u32]
         [
-            3
+            5
             null
-            5
-            5
+            3
+            6
+        ]
+
+        """
+
+    def n_chars(self) -> pli.Series:
+        """
+        Get length of the string values in the Series (as number of chars).
+
+        Returns
+        -------
+        Series[u32]
+
+        Notes
+        -----
+        If you know that you are working with ASCII text, ``lengths`` will be
+        equivalent, and faster (returns length in terms of the number of bytes).
+
+        Examples
+        --------
+        >>> s = pl.Series(["Café", None, "345", "東京"])
+        >>> s.str.n_chars()
+        shape: (4,)
+        Series: '' [u32]
+        [
+            4
+            null
+            3
+            2
         ]
 
         """
@@ -626,14 +662,38 @@ class StringNameSpace:
 
         """
 
-    def strip(self) -> pli.Series:
-        """Remove leading and trailing whitespace."""
+    def strip(self, matches: None | str = None) -> pli.Series:
+        """
+        Remove leading and trailing whitespace.
 
-    def lstrip(self) -> pli.Series:
-        """Remove leading whitespace."""
+        Parameters
+        ----------
+        matches
+            An optional single character that should be trimmed
 
-    def rstrip(self) -> pli.Series:
-        """Remove trailing whitespace."""
+        """
+
+    def lstrip(self, matches: None | str = None) -> pli.Series:
+        """
+        Remove leading whitespace.
+
+        Parameters
+        ----------
+        matches
+            An optional single character that should be trimmed
+
+        """
+
+    def rstrip(self, matches: None | str = None) -> pli.Series:
+        """
+        Remove trailing whitespace.
+
+        Parameters
+        ----------
+        matches
+            An optional single character that should be trimmed
+
+        """
 
     def zfill(self, alignment: int) -> pli.Series:
         """
