@@ -4,8 +4,20 @@
 extern crate core;
 extern crate polars;
 
+#[cfg(feature = "build_info")]
+#[macro_use]
+extern crate pyo3_built;
+
+#[cfg(feature = "build_info")]
+#[allow(dead_code)]
+mod build {
+    include!(concat!(env!("OUT_DIR"), "/built.rs"));
+}
+
 pub mod apply;
 pub mod arrow_interop;
+#[cfg(feature = "csv-file")]
+mod batched_csv;
 pub mod conversion;
 pub mod dataframe;
 pub mod datatypes;
@@ -14,22 +26,23 @@ pub mod file;
 pub mod lazy;
 mod list_construction;
 pub mod npy;
+mod object;
 pub mod prelude;
 pub(crate) mod py_modules;
 pub mod series;
 mod set;
-#[cfg(feature = "polars-sql")]
+#[cfg(feature = "sql")]
 mod sql;
 pub mod utils;
 
-#[cfg(target_os = "linux")]
+#[cfg(all(target_os = "linux", not(use_mimalloc)))]
 use jemallocator::Jemalloc;
 use lazy::ToExprs;
-#[cfg(not(target_os = "linux"))]
+#[cfg(any(not(target_os = "linux"), use_mimalloc))]
 use mimalloc::MiMalloc;
 use polars::functions::{diag_concat_df, hor_concat_df};
 use polars::prelude::Null;
-use polars_core::datatypes::TimeUnit;
+use polars_core::datatypes::{TimeUnit, TimeZone};
 use polars_core::prelude::{DataFrame, IntoSeries, IDX_DTYPE};
 use polars_core::POOL;
 use pyo3::exceptions::PyValueError;
@@ -54,11 +67,11 @@ use crate::prelude::{
 use crate::series::PySeries;
 
 #[global_allocator]
-#[cfg(target_os = "linux")]
+#[cfg(all(target_os = "linux", not(use_mimalloc)))]
 static ALLOC: Jemalloc = Jemalloc;
 
 #[global_allocator]
-#[cfg(not(target_os = "linux"))]
+#[cfg(any(not(target_os = "linux"), use_mimalloc))]
 static ALLOC: MiMalloc = MiMalloc;
 
 #[pyfunction]
@@ -109,10 +122,25 @@ fn binary_expr(l: dsl::PyExpr, op: u8, r: dsl::PyExpr) -> dsl::PyExpr {
 }
 
 #[pyfunction]
-pub fn fold(acc: PyExpr, lambda: PyObject, exprs: Vec<PyExpr>) -> PyExpr {
+fn fold(acc: PyExpr, lambda: PyObject, exprs: Vec<PyExpr>) -> PyExpr {
     dsl::fold(acc, lambda, exprs)
 }
 
+#[pyfunction]
+fn reduce(lambda: PyObject, exprs: Vec<PyExpr>) -> PyExpr {
+    dsl::reduce(lambda, exprs)
+}
+
+#[pyfunction]
+fn cumfold(acc: PyExpr, lambda: PyObject, exprs: Vec<PyExpr>, include_init: bool) -> PyExpr {
+    dsl::cumfold(acc, lambda, exprs, include_init)
+}
+
+#[pyfunction]
+fn cumreduce(lambda: PyObject, exprs: Vec<PyExpr>) -> PyExpr {
+    dsl::cumreduce(lambda, exprs)
+}
+
 #[pyfunction]
 fn arange(low: PyExpr, high: PyExpr, step: usize) -> PyExpr {
     polars::lazy::dsl::arange(low.inner, high.inner, step).into()
@@ -219,12 +247,12 @@ fn py_datetime(
     hour: Option<dsl::PyExpr>,
     minute: Option<dsl::PyExpr>,
     second: Option<dsl::PyExpr>,
-    millisecond: Option<dsl::PyExpr>,
+    microsecond: Option<dsl::PyExpr>,
 ) -> dsl::PyExpr {
     let hour = hour.map(|e| e.inner);
     let minute = minute.map(|e| e.inner);
     let second = second.map(|e| e.inner);
-    let millisecond = millisecond.map(|e| e.inner);
+    let microsecond = microsecond.map(|e| e.inner);
 
     let args = DatetimeArgs {
         year: year.inner,
@@ -233,17 +261,19 @@ fn py_datetime(
         hour,
         minute,
         second,
-        millisecond,
+        microsecond,
     };
 
     polars::lazy::dsl::datetime(args).into()
 }
 
+#[allow(clippy::too_many_arguments)]
 #[pyfunction]
 fn py_duration(
     days: Option<PyExpr>,
     seconds: Option<PyExpr>,
     nanoseconds: Option<PyExpr>,
+    microseconds: Option<PyExpr>,
     milliseconds: Option<PyExpr>,
     minutes: Option<PyExpr>,
     hours: Option<PyExpr>,
@@ -253,6 +283,7 @@ fn py_duration(
         days: days.map(|e| e.inner),
         seconds: seconds.map(|e| e.inner),
         nanoseconds: nanoseconds.map(|e| e.inner),
+        microseconds: microseconds.map(|e| e.inner),
         milliseconds: milliseconds.map(|e| e.inner),
         minutes: minutes.map(|e| e.inner),
         hours: hours.map(|e| e.inner),
@@ -429,10 +460,18 @@ pub fn map_mul(
     py: Python,
     pyexpr: Vec<PyExpr>,
     lambda: PyObject,
-    output_type: &PyAny,
+    output_type: Option<Wrap<DataType>>,
     apply_groups: bool,
+    returns_scalar: bool,
 ) -> PyExpr {
-    lazy::map_mul(&pyexpr, py, lambda, output_type, apply_groups)
+    lazy::map_mul(
+        &pyexpr,
+        py,
+        lambda,
+        output_type,
+        apply_groups,
+        returns_scalar,
+    )
 }
 
 #[pyfunction]
@@ -443,10 +482,34 @@ fn py_date_range(
     closed: Wrap<ClosedWindow>,
     name: &str,
     tu: Wrap<TimeUnit>,
+    tz: Option<TimeZone>,
 ) -> PySeries {
-    polars::time::date_range_impl(name, start, stop, Duration::parse(every), closed.0, tu.0)
-        .into_series()
-        .into()
+    polars::time::date_range_impl(
+        name,
+        start,
+        stop,
+        Duration::parse(every),
+        closed.0,
+        tu.0,
+        tz,
+    )
+    .into_series()
+    .into()
+}
+
+#[pyfunction]
+fn py_date_range_lazy(
+    start: PyExpr,
+    end: PyExpr,
+    every: &str,
+    closed: Wrap<ClosedWindow>,
+    name: String,
+    tz: Option<TimeZone>,
+) -> PyExpr {
+    let start = start.inner;
+    let end = end.inner;
+    let every = Duration::parse(every);
+    polars::lazy::dsl::functions::date_range(name, start, end, every, closed.0, tz).into()
 }
 
 #[pyfunction]
@@ -515,12 +578,21 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
         py.get_type::<InvalidOperationError>(),
     )
     .unwrap();
+
+    #[cfg(feature = "build_info")]
+    m.add(
+        "_build_info_",
+        pyo3_built!(py, build, "build", "time", "deps", "features", "host", "target", "git"),
+    )?;
+
     m.add_class::<PySeries>().unwrap();
     m.add_class::<PyDataFrame>().unwrap();
     m.add_class::<PyLazyFrame>().unwrap();
     m.add_class::<PyLazyGroupBy>().unwrap();
     m.add_class::<dsl::PyExpr>().unwrap();
-    #[cfg(feature = "polars-sql")]
+    #[cfg(feature = "csv-file")]
+    m.add_class::<batched_csv::PyBatchedCsv>().unwrap();
+    #[cfg(feature = "sql")]
     m.add_class::<sql::PySQLContext>().unwrap();
     m.add_wrapped(wrap_pyfunction!(col)).unwrap();
     m.add_wrapped(wrap_pyfunction!(count)).unwrap();
@@ -531,6 +603,9 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(dtype_str_repr)).unwrap();
     m.add_wrapped(wrap_pyfunction!(lit)).unwrap();
     m.add_wrapped(wrap_pyfunction!(fold)).unwrap();
+    m.add_wrapped(wrap_pyfunction!(cumfold)).unwrap();
+    m.add_wrapped(wrap_pyfunction!(reduce)).unwrap();
+    m.add_wrapped(wrap_pyfunction!(cumreduce)).unwrap();
     m.add_wrapped(wrap_pyfunction!(binary_expr)).unwrap();
     m.add_wrapped(wrap_pyfunction!(arange)).unwrap();
     m.add_wrapped(wrap_pyfunction!(pearson_corr)).unwrap();
@@ -558,6 +633,7 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(py_datetime)).unwrap();
     m.add_wrapped(wrap_pyfunction!(py_duration)).unwrap();
     m.add_wrapped(wrap_pyfunction!(py_date_range)).unwrap();
+    m.add_wrapped(wrap_pyfunction!(py_date_range_lazy)).unwrap();
     m.add_wrapped(wrap_pyfunction!(sum_exprs)).unwrap();
     m.add_wrapped(wrap_pyfunction!(min_exprs)).unwrap();
     m.add_wrapped(wrap_pyfunction!(max_exprs)).unwrap();
