@@ -2,7 +2,7 @@ use numpy::PyArray1;
 use polars_core::prelude::QuantileInterpolOptions;
 use polars_core::series::IsSorted;
 use polars_core::utils::CustomIterTools;
-use pyo3::exceptions::PyRuntimeError;
+use pyo3::exceptions::{PyRuntimeError, PyValueError};
 use pyo3::prelude::*;
 use pyo3::types::{PyBytes, PyList, PyTuple};
 use pyo3::Python;
@@ -494,6 +494,14 @@ impl PySeries {
         Ok(())
     }
 
+    pub fn new_from_index(&self, index: usize, length: usize) -> PyResult<Self> {
+        if index >= self.series.len() {
+            Err(PyValueError::new_err("index is out of bounds"))
+        } else {
+            Ok(self.series.new_from_index(index, length).into())
+        }
+    }
+
     pub fn filter(&self, filter: &PySeries) -> PyResult<Self> {
         let filter_series = &filter.series;
         if let Ok(ca) = filter_series.bool() {
@@ -558,34 +566,6 @@ impl PySeries {
         self.series.has_validity()
     }
 
-    pub fn sample_n(
-        &self,
-        n: usize,
-        with_replacement: bool,
-        shuffle: bool,
-        seed: Option<u64>,
-    ) -> PyResult<Self> {
-        let s = self
-            .series
-            .sample_n(n, with_replacement, shuffle, seed)
-            .map_err(PyPolarsErr::from)?;
-        Ok(s.into())
-    }
-
-    pub fn sample_frac(
-        &self,
-        frac: f64,
-        with_replacement: bool,
-        shuffle: bool,
-        seed: Option<u64>,
-    ) -> PyResult<Self> {
-        let s = self
-            .series
-            .sample_frac(frac, with_replacement, shuffle, seed)
-            .map_err(PyPolarsErr::from)?;
-        Ok(s.into())
-    }
-
     pub fn series_equal(&self, other: &PySeries, null_equal: bool, strict: bool) -> bool {
         if strict {
             self.series.eq(&other.series)
@@ -713,7 +693,13 @@ impl PySeries {
                         let ca = series.duration().unwrap();
                         return Wrap(ca).to_object(py);
                     }
-                    dt => panic!("to_list() not implemented for {:?}", dt),
+                    DataType::Binary => {
+                        let ca = series.binary().unwrap();
+                        return Wrap(ca).to_object(py);
+                    }
+                    DataType::Null | DataType::Unknown => {
+                        panic!("to_list not implemented for null/unknown")
+                    }
                 };
                 pylist.to_object(py)
             }
@@ -773,12 +759,30 @@ impl PySeries {
         &self,
         lambda: &PyAny,
         output_type: Option<Wrap<DataType>>,
+        skip_nulls: bool,
     ) -> PyResult<PySeries> {
         Python::with_gil(|py| {
             let series = &self.series;
 
             let output_type = output_type.map(|dt| dt.0);
 
+            macro_rules! dispatch_apply {
+                ($self:expr, $method:ident, $($args:expr),*) => {
+                    if matches!($self.dtype(), DataType::Object(_)) {
+                        let ca = $self.0.unpack::<ObjectType<ObjectValue>>().unwrap();
+                        ca.$method($($args),*)
+                    } else {
+                        apply_method_all_arrow_series2!(
+                            $self,
+                            $method,
+                            $($args),*
+                        )
+                    }
+
+                }
+
+            }
+
             if matches!(
                 self.series.dtype(),
                 DataType::Datetime(_, _)
@@ -786,7 +790,8 @@ impl PySeries {
                     | DataType::Duration(_)
                     | DataType::Categorical(_)
                     | DataType::Time
-            ) {
+            ) || !skip_nulls
+            {
                 let mut avs = Vec::with_capacity(self.series.len());
                 let iter = self.series.iter().map(|av| {
                     let input = Wrap(av);
@@ -800,7 +805,7 @@ impl PySeries {
 
             let out = match output_type {
                 Some(DataType::Int8) => {
-                    let ca: Int8Chunked = apply_method_all_arrow_series2!(
+                    let ca: Int8Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -811,7 +816,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Int16) => {
-                    let ca: Int16Chunked = apply_method_all_arrow_series2!(
+                    let ca: Int16Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -822,7 +827,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Int32) => {
-                    let ca: Int32Chunked = apply_method_all_arrow_series2!(
+                    let ca: Int32Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -833,7 +838,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Int64) => {
-                    let ca: Int64Chunked = apply_method_all_arrow_series2!(
+                    let ca: Int64Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -844,7 +849,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::UInt8) => {
-                    let ca: UInt8Chunked = apply_method_all_arrow_series2!(
+                    let ca: UInt8Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -855,7 +860,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::UInt16) => {
-                    let ca: UInt16Chunked = apply_method_all_arrow_series2!(
+                    let ca: UInt16Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -866,7 +871,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::UInt32) => {
-                    let ca: UInt32Chunked = apply_method_all_arrow_series2!(
+                    let ca: UInt32Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -877,7 +882,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::UInt64) => {
-                    let ca: UInt64Chunked = apply_method_all_arrow_series2!(
+                    let ca: UInt64Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -888,7 +893,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Float32) => {
-                    let ca: Float32Chunked = apply_method_all_arrow_series2!(
+                    let ca: Float32Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -899,7 +904,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Float64) => {
-                    let ca: Float64Chunked = apply_method_all_arrow_series2!(
+                    let ca: Float64Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -910,7 +915,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Boolean) => {
-                    let ca: BooleanChunked = apply_method_all_arrow_series2!(
+                    let ca: BooleanChunked = dispatch_apply!(
                         series,
                         apply_lambda_with_bool_out_type,
                         py,
@@ -921,7 +926,7 @@ impl PySeries {
                     ca.into_series()
                 }
                 Some(DataType::Date) => {
-                    let ca: Int32Chunked = apply_method_all_arrow_series2!(
+                    let ca: Int32Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -932,7 +937,7 @@ impl PySeries {
                     ca.into_date().into_series()
                 }
                 Some(DataType::Datetime(tu, tz)) => {
-                    let ca: Int64Chunked = apply_method_all_arrow_series2!(
+                    let ca: Int64Chunked = dispatch_apply!(
                         series,
                         apply_lambda_with_primitive_out_type,
                         py,
@@ -943,7 +948,7 @@ impl PySeries {
                     ca.into_datetime(tu, tz).into_series()
                 }
                 Some(DataType::Utf8) => {
-                    let ca: Utf8Chunked = apply_method_all_arrow_series2!(
+                    let ca = dispatch_apply!(
                         series,
                         apply_lambda_with_utf8_out_type,
                         py,
@@ -951,11 +956,12 @@ impl PySeries {
                         0,
                         None
                     )?;
+
                     ca.into_series()
                 }
                 #[cfg(feature = "object")]
                 Some(DataType::Object(_)) => {
-                    let ca: ObjectChunked<ObjectValue> = apply_method_all_arrow_series2!(
+                    let ca = dispatch_apply!(
                         series,
                         apply_lambda_with_object_out_type,
                         py,
@@ -965,16 +971,9 @@ impl PySeries {
                     )?;
                     ca.into_series()
                 }
-                None => {
-                    return apply_method_all_arrow_series2!(
-                        series,
-                        apply_lambda_unknown,
-                        py,
-                        lambda
-                    );
-                }
+                None => return dispatch_apply!(series, apply_lambda_unknown, py, lambda),
 
-                _ => return apply_method_all_arrow_series2!(series, apply_lambda, py, lambda),
+                _ => return dispatch_apply!(series, apply_lambda_unknown, py, lambda),
             };
 
             Ok(PySeries::new(out))
@@ -1156,7 +1155,11 @@ impl_ufuncs!(apply_ufunc_i64, Int64Type, unsafe_from_ptr_i64);
 
 macro_rules! impl_set_with_mask {
     ($name:ident, $native:ty, $cast:ident, $variant:ident) => {
-        fn $name(series: &Series, filter: &PySeries, value: Option<$native>) -> Result<Series> {
+        fn $name(
+            series: &Series,
+            filter: &PySeries,
+            value: Option<$native>,
+        ) -> PolarsResult<Series> {
             let mask = filter.series.bool()?;
             let ca = series.$cast()?;
             let new = ca.set(mask, value)?;
