@@ -12,8 +12,12 @@ import pyarrow as pa
 import pytest
 
 import polars as pl
+from polars.datatypes import DTYPE_TEMPORAL_UNITS
+from polars.dependencies import zoneinfo
 from polars.exceptions import NoRowsReturned, TooManyRowsReturned
-from polars.testing import assert_frame_equal, assert_series_equal, columns
+from polars.internals.construction import iterable_to_pydf
+from polars.testing import assert_frame_equal, assert_series_equal
+from polars.testing.parametric import columns
 
 if TYPE_CHECKING:
     from polars.internals.type_aliases import JoinStrategy
@@ -260,14 +264,14 @@ def test_dataclasses_and_namedtuple() -> None:
             data=trades,
             columns=[  # type: ignore[arg-type]
                 ("ts", pl.Datetime("ms")),
-                ("tk", pl.Utf8),
+                ("tk", pl.Categorical),
                 ("pc", pl.Float32),
                 ("sz", pl.UInt16),
             ],
         )
         assert df.schema == {
             "ts": pl.Datetime("ms"),
-            "tk": pl.Utf8,
+            "tk": pl.Categorical,
             "pc": pl.Float32,
             "sz": pl.UInt16,
         }
@@ -279,6 +283,7 @@ def test_dataframe_membership_operator() -> None:
     df = pl.DataFrame({"name": ["Jane", "John"], "age": [20, 30]})
     assert "name" in df
     assert "phone" not in df
+    assert df._ipython_key_completions_() == ["name", "age"]
 
 
 def test_sort() -> None:
@@ -484,11 +489,15 @@ def test_groupby() -> None:
         df.groupby("a", "b")  # type: ignore[arg-type]
 
 
-BAD_AGG_PARAMETERS = [[("b", "sum")], [("b", ["sum"])], {"b": "sum"}, {"b": ["sum"]}]
-GOOD_AGG_PARAMETERS: list[pl.Expr | list[pl.Expr]] = [
-    [pl.col("b").sum()],
-    pl.col("b").sum(),
-]
+def bad_agg_parameters() -> Any:
+    return [[("b", "sum")], [("b", ["sum"])], {"b": "sum"}, {"b": ["sum"]}]
+
+
+def good_agg_parameters() -> list[pl.Expr | list[pl.Expr]]:
+    return [
+        [pl.col("b").sum()],
+        pl.col("b").sum(),
+    ]
 
 
 @pytest.mark.parametrize("lazy", [True, False])
@@ -496,15 +505,15 @@ def test_groupby_agg_input_types(lazy: bool) -> None:
     df = pl.DataFrame({"a": [1, 1, 2, 2], "b": [1, 2, 3, 4]})
     df_or_lazy: pl.DataFrame | pl.LazyFrame = df.lazy() if lazy else df
 
-    for bad_param in BAD_AGG_PARAMETERS:
+    for bad_param in bad_agg_parameters():
         with pytest.raises(TypeError):
-            result = df_or_lazy.groupby("a").agg(bad_param)  # type: ignore[arg-type]
+            result = df_or_lazy.groupby("a").agg(bad_param)
             if lazy:
                 result.collect()  # type: ignore[union-attr]
 
     expected = pl.DataFrame({"a": [1, 2], "b": [3, 7]})
 
-    for good_param in GOOD_AGG_PARAMETERS:
+    for good_param in good_agg_parameters():
         result = df_or_lazy.groupby("a", maintain_order=True).agg(good_param)
         if lazy:
             result = result.collect()  # type: ignore[union-attr]
@@ -516,19 +525,17 @@ def test_groupby_rolling_agg_input_types(lazy: bool) -> None:
     df = pl.DataFrame({"index_column": [0, 1, 2, 3], "b": [1, 3, 1, 2]})
     df_or_lazy: pl.DataFrame | pl.LazyFrame = df.lazy() if lazy else df
 
-    for bad_param in BAD_AGG_PARAMETERS:
+    for bad_param in bad_agg_parameters():
         with pytest.raises(TypeError):
             result = df_or_lazy.groupby_rolling(
                 index_column="index_column", period="2i"
-            ).agg(
-                bad_param  # type: ignore[arg-type]
-            )
+            ).agg(bad_param)
             if lazy:
                 result.collect()  # type: ignore[union-attr]
 
     expected = pl.DataFrame({"index_column": [0, 1, 2, 3], "b": [1, 4, 4, 3]})
 
-    for good_param in GOOD_AGG_PARAMETERS:
+    for good_param in good_agg_parameters():
         result = df_or_lazy.groupby_rolling(
             index_column="index_column", period="2i"
         ).agg(good_param)
@@ -542,19 +549,17 @@ def test_groupby_dynamic_agg_input_types(lazy: bool) -> None:
     df = pl.DataFrame({"index_column": [0, 1, 2, 3], "b": [1, 3, 1, 2]})
     df_or_lazy: pl.DataFrame | pl.LazyFrame = df.lazy() if lazy else df
 
-    for bad_param in BAD_AGG_PARAMETERS:
+    for bad_param in bad_agg_parameters():
         with pytest.raises(TypeError):
             result = df_or_lazy.groupby_dynamic(
                 index_column="index_column", every="2i", closed="right"
-            ).agg(
-                bad_param  # type: ignore[arg-type]
-            )
+            ).agg(bad_param)
             if lazy:
                 result.collect()  # type: ignore[union-attr]
 
-    expected = pl.DataFrame({"index_column": [0, 0, 2], "b": [1, 4, 2]})
+    expected = pl.DataFrame({"index_column": [-2, 0, 2], "b": [1, 4, 2]})
 
-    for good_param in GOOD_AGG_PARAMETERS:
+    for good_param in good_agg_parameters():
         result = df_or_lazy.groupby_dynamic(
             index_column="index_column", every="2i", closed="right"
         ).agg(good_param)
@@ -673,9 +678,6 @@ def test_extend() -> None:
 
 
 def test_drop() -> None:
-    df = pl.DataFrame({"a": [2, 1, 3], "b": ["a", "b", "c"], "c": [1, 2, 3]})
-    df = df.drop(name="a")  # type: ignore[call-arg]
-    assert df.shape == (3, 2)
     df = pl.DataFrame({"a": [2, 1, 3], "b": ["a", "b", "c"], "c": [1, 2, 3]})
     df = df.drop(columns="a")
     assert df.shape == (3, 2)
@@ -1037,7 +1039,7 @@ def test_describe() -> None:
             "f": ["3", "0", None, None, "2020-01-01", "2022-01-01", None],
         }
     )
-    pl.testing.assert_frame_equal(df.describe(), expected)
+    assert_frame_equal(df.describe(), expected)
 
 
 def test_duration_arithmetic() -> None:
@@ -1081,6 +1083,7 @@ def test_string_cache_eager_lazy() -> None:
         df1 = pl.DataFrame(
             {"region_ids": ["reg1", "reg2", "reg3", "reg4", "reg5"]}
         ).select([pl.col("region_ids").cast(pl.Categorical)])
+
         df2 = pl.DataFrame(
             {"seq_name": ["reg4", "reg2", "reg1"], "score": [3.0, 1.0, 2.0]}
         ).select([pl.col("seq_name").cast(pl.Categorical), pl.col("score")])
@@ -1096,6 +1099,14 @@ def test_string_cache_eager_lazy() -> None:
             df2, left_on="region_ids", right_on="seq_name", how="left"
         ).frame_equal(expected, null_equal=True)
 
+        # also check row-wise categorical insert.
+        # (column-wise is preferred, but this shouldn't fail)
+        df3 = pl.DataFrame(
+            data=[["reg1"], ["reg2"], ["reg3"], ["reg4"], ["reg5"]],
+            columns=[("region_ids", pl.Categorical)],
+        )
+        assert_frame_equal(df1, df3)
+
 
 def test_assign() -> None:
     # check if can assign in case of a single column
@@ -1209,6 +1220,66 @@ def test_write_csv() -> None:
     assert s == expected
 
 
+def test_from_generator_or_iterable() -> None:
+    # generator function
+    def gen(n: int, strkey: bool = True) -> Iterator[Any]:
+        for i in range(n):
+            yield (str(i) if strkey else i), 1 * i, 2**i, 3**i
+
+    # iterable object
+    class Rows:
+        def __init__(self, n: int, strkey: bool = True):
+            self._n = n
+            self._strkey = strkey
+
+        def __iter__(self) -> Iterator[Any]:
+            yield from gen(self._n, self._strkey)
+
+    # check init from column-oriented generator
+    assert_frame_equal(
+        pl.DataFrame(data=gen(4, False), orient="col"),
+        pl.DataFrame(
+            data=[(0, 0, 1, 1), (1, 1, 2, 3), (2, 2, 4, 9), (3, 3, 8, 27)], orient="col"
+        ),
+    )
+    # check init from row-oriented generators (more common)
+    expected = pl.DataFrame(
+        data=list(gen(4)), columns=["a", "b", "c", "d"], orient="row"
+    )
+    for generated_frame in (
+        pl.DataFrame(data=gen(4), columns=["a", "b", "c", "d"]),
+        pl.DataFrame(data=Rows(4), columns=["a", "b", "c", "d"]),
+        pl.DataFrame(data=(x for x in Rows(4)), columns=["a", "b", "c", "d"]),
+    ):
+        assert_frame_equal(expected, generated_frame)
+        assert generated_frame.schema == {
+            "a": pl.Utf8,
+            "b": pl.Int64,
+            "c": pl.Int64,
+            "d": pl.Int64,
+        }
+
+    # test 'iterable_to_pydf' directly to validate 'chunk_size' behaviour
+    cols = ["a", "b", ("c", pl.Int8), "d"]
+
+    d1 = iterable_to_pydf(gen(4), columns=cols, chunk_size=2)  # type: ignore[arg-type]
+    d2 = iterable_to_pydf(Rows(4), columns=cols, chunk_size=3)  # type: ignore[arg-type]
+    d3 = iterable_to_pydf(Rows(4), columns=cols)  # type: ignore[arg-type]
+
+    expected_data = [("0", 0, 1, 1), ("1", 1, 2, 3), ("2", 2, 4, 9), ("3", 3, 8, 27)]
+    expected_schema = [("a", pl.Utf8), ("b", pl.Int64), ("c", pl.Int8), ("d", pl.Int64)]
+
+    for d in (d1, d2, d3):
+        assert expected_data == d.row_tuples()
+        assert expected_schema == list(zip(d1.columns(), d1.dtypes()))
+
+    # empty iterator
+    assert_frame_equal(
+        pl.DataFrame(data=gen(0), columns=["a", "b", "c", "d"]),
+        pl.DataFrame(columns=["a", "b", "c", "d"]),
+    )
+
+
 def test_from_rows() -> None:
     df = pl.from_records([[1, 2, "foo"], [2, 3, "bar"]], orient="row")
     assert df.frame_equal(
@@ -1216,7 +1287,6 @@ def test_from_rows() -> None:
             {"column_0": [1, 2], "column_1": [2, 3], "column_2": ["foo", "bar"]}
         )
     )
-
     df = pl.from_records(
         [[1, datetime.fromtimestamp(100)], [2, datetime.fromtimestamp(2398754908)]],
         orient="row",
@@ -1311,18 +1381,20 @@ def test_reproducible_hash_with_seeds() -> None:
     the same seeds.
 
     """
+    import platform
+
     df = pl.DataFrame({"s": [1234, None, 5678]})
     seeds = (11, 22, 33, 44)
+
+    # TODO: introduce a platform-stable string hash...
+    #  in the meantime, account for arm64 (mac) hash values to reduce noise
     expected = pl.Series(
         "s",
-        [
-            15801072432137883943,
-            988796329533502010,
-            9604537446374444741,
-        ],
+        [8823051245921001677, 988796329533502010, 7528667241828618484]
+        if platform.mac_ver()[-1] == "arm64"
+        else [6629530352159708028, 988796329533502010, 6048298245521876612],
         dtype=pl.UInt64,
     )
-
     result = df.hash_rows(*seeds)
     assert_series_equal(expected, result, check_names=False, check_exact=True)
     result = df["s"].hash(*seeds)
@@ -1374,8 +1446,12 @@ def test_hashing_on_python_objects() -> None:
 
 
 def test_unique_unit_rows() -> None:
-    # simply test if we don't panic.
-    pl.DataFrame({"a": [1], "b": [None]}).unique(subset="a")
+    df = pl.DataFrame({"a": [1], "b": [None]})
+
+    # 'unique' one-row frame should be equal to the original frame
+    assert df.frame_equal(df.unique(subset="a"))
+    for col in df.columns:
+        assert df.n_unique(subset=[col]) == 1
 
 
 def test_panic() -> None:
@@ -1734,7 +1810,30 @@ def test_is_duplicated() -> None:
 
 def test_is_unique() -> None:
     df = pl.DataFrame({"foo": [1, 2, 2], "bar": [6, 7, 7]})
+
     assert df.is_unique().series_equal(pl.Series("", [True, False, False]))
+    assert df.unique(maintain_order=True).rows() == [(1, 6), (2, 7)]
+    assert df.n_unique() == 2
+
+
+def test_n_unique_subsets() -> None:
+    df = pl.DataFrame(
+        {
+            "a": [1, 1, 2, 3, 4, 5],
+            "b": [0.5, 0.5, 1.0, 2.0, 3.0, 3.0],
+            "c": [True, True, True, False, True, True],
+        }
+    )
+    # omitting 'subset' counts unique rows
+    assert df.n_unique() == 5
+
+    # providing it counts unique col/expr subsets
+    assert df.n_unique(subset=["b", "c"]) == 4
+    assert df.n_unique(subset=pl.col("c")) == 2
+    assert (
+        df.n_unique(subset=[(pl.col("a") // 2), (pl.col("c") | (pl.col("b") >= 2))])
+        == 3
+    )
 
 
 def test_sample() -> None:
@@ -1755,18 +1854,18 @@ def test_shrink_to_fit() -> None:
 def test_arithmetic() -> None:
     df = pl.DataFrame({"a": [1.0, 2.0], "b": [3.0, 4.0]})
 
-    df_mul = df * 2
-    expected = pl.DataFrame({"a": [2, 4], "b": [6, 8]})
-    assert df_mul.frame_equal(expected)
+    for df_mul in (df * 2, 2 * df):
+        expected = pl.DataFrame({"a": [2, 4], "b": [6, 8]})
+        assert df_mul.frame_equal(expected)
+
+    for df_plus in (df + 2, 2 + df):
+        expected = pl.DataFrame({"a": [3, 4], "b": [5, 6]})
+        assert df_plus.frame_equal(expected)
 
     df_div = df / 2
     expected = pl.DataFrame({"a": [0.5, 1.0], "b": [1.5, 2.0]})
     assert df_div.frame_equal(expected)
 
-    df_plus = df + 2
-    expected = pl.DataFrame({"a": [3, 4], "b": [5, 6]})
-    assert df_plus.frame_equal(expected)
-
     df_minus = df - 2
     expected = pl.DataFrame({"a": [-1, 0], "b": [1, 2]})
     assert df_minus.frame_equal(expected)
@@ -1814,11 +1913,15 @@ def test_arithmetic() -> None:
 
 def test_add_string() -> None:
     df = pl.DataFrame({"a": ["hi", "there"], "b": ["hello", "world"]})
-    result = df + " hello"
     expected = pl.DataFrame(
         {"a": ["hi hello", "there hello"], "b": ["hello hello", "world hello"]}
     )
-    assert result.frame_equal(expected)
+    assert (df + " hello").frame_equal(expected)
+
+    expected = pl.DataFrame(
+        {"a": ["hello hi", "hello there"], "b": ["hello hello", "hello world"]}
+    )
+    assert ("hello " + df).frame_equal(expected)
 
 
 def test_get_item() -> None:
@@ -1913,6 +2016,18 @@ def test_get_item() -> None:
     with pytest.raises(ValueError):
         df[pl.Series([True, False, True]), "b"]
 
+    # 5343
+    df = pl.DataFrame(
+        {
+            f"fo{col}": [n**col for n in range(5)]  # 5 rows
+            for col in range(12)  # 12 columns
+        }
+    )
+    assert df[4, 4] == 256
+    assert df[4, 5] == 1024
+    assert df[4, [2]].frame_equal(pl.DataFrame({"fo2": [16]}))
+    assert df[4, [5]].frame_equal(pl.DataFrame({"fo5": [1024]}))
+
 
 @pytest.mark.parametrize("as_series,inner_dtype", [(True, pl.Series), (False, list)])
 def test_to_dict(as_series: bool, inner_dtype: Any) -> None:
@@ -2336,7 +2451,7 @@ def test_set() -> None:
 
     # needs to be a 2 element tuple
     with pytest.raises(ValueError):
-        df[(1, 2, 3)] = 1  # type: ignore[index]
+        df[(1, 2, 3)] = 1
 
     # we cannot index with any type, such as bool
     with pytest.raises(ValueError):
@@ -2359,3 +2474,48 @@ def test_union_with_aliases_4770() -> None:
     ).filter(pl.col("x").is_not_null())
 
     assert lf.collect()["x"].to_list() == [1, 3, 4]
+
+
+def test_init_datetimes_with_timezone() -> None:
+    tz_us = "America/New_York"
+    tz_europe = "Europe/Amsterdam"
+
+    dtm = datetime(2022, 10, 12, 12, 30, tzinfo=zoneinfo.ZoneInfo("UTC"))
+    for tu in DTYPE_TEMPORAL_UNITS | frozenset([None]):
+        df = pl.DataFrame(
+            data={"d1": [dtm], "d2": [dtm]},
+            columns=[
+                ("d1", pl.Datetime(tu, tz_us)),
+                ("d2", pl.Datetime(tu, tz_europe)),
+            ],
+        )
+        assert (df["d1"].to_physical() == df["d2"].to_physical()).all()
+        assert df.rows() == [
+            (
+                datetime(2022, 10, 12, 8, 30, tzinfo=zoneinfo.ZoneInfo(tz_us)),
+                datetime(2022, 10, 12, 14, 30, tzinfo=zoneinfo.ZoneInfo(tz_europe)),
+            )
+        ]
+
+
+def test_init_physical_with_timezone() -> None:
+    tz_uae = "Asia/Dubai"
+    tz_asia = "Asia/Tokyo"
+
+    dtm_us = 1665577800000000
+    for tu in DTYPE_TEMPORAL_UNITS | frozenset([None]):
+        dtm = {"ms": dtm_us // 1_000, "ns": dtm_us * 1_000}.get(str(tu), dtm_us)
+        df = pl.DataFrame(
+            data={"d1": [dtm], "d2": [dtm]},
+            columns=[
+                ("d1", pl.Datetime(tu, tz_uae)),
+                ("d2", pl.Datetime(tu, tz_asia)),
+            ],
+        )
+        assert (df["d1"].to_physical() == df["d2"].to_physical()).all()
+        assert df.rows() == [
+            (
+                datetime(2022, 10, 12, 16, 30, tzinfo=zoneinfo.ZoneInfo(tz_uae)),
+                datetime(2022, 10, 12, 21, 30, tzinfo=zoneinfo.ZoneInfo(tz_asia)),
+            )
+        ]
