@@ -276,3 +276,32 @@ def test_lazy_concat_err() -> None:
             ValueError, match="Lazy only allows 'vertical' concat strategy."
         ):
             pl.concat([df1.lazy(), df2.lazy()], how=how).collect()
+
+
+def test_invalid_sort_by() -> None:
+    df = pl.DataFrame(
+        {
+            "a": ["bill", "bob", "jen", "allie", "george"],
+            "b": ["M", "M", "F", "F", "M"],
+            "c": [32, 40, 20, 19, 39],
+        }
+    )
+
+    # `select a where b order by c desc`
+    with pytest.raises(
+        pl.ComputeError,
+        match="The sortby operation produced a different length than the Series that has to be sorted.",  # noqa: E501
+    ):
+        df.select(pl.col("a").filter(pl.col("b") == "M").sort_by("c", True))
+
+
+def test_concat_list_err_supertype() -> None:
+    df = pl.DataFrame({"nums": [1, 2, 3, 4], "letters": ["a", "b", "c", "d"]}).select(
+        [
+            pl.col("nums"),
+            pl.struct(["letters", "nums"]).alias("combo"),
+            pl.struct(["nums", "letters"]).alias("reverse_combo"),
+        ]
+    )
+    with pytest.raises(pl.ComputeError, match="Failed to determine supertype"):
+        df.select(pl.concat_list(["combo", "reverse_combo"]))
