@@ -1,8 +1,8 @@
 from __future__ import annotations
 
 import math
-from datetime import date, datetime, time
-from typing import TYPE_CHECKING, Any, cast
+from datetime import date, datetime, time, timedelta
+from typing import TYPE_CHECKING, Any, Iterator, cast
 
 import numpy as np
 import pandas as pd
@@ -10,8 +10,23 @@ import pyarrow as pa
 import pytest
 
 import polars as pl
-from polars.datatypes import Date, Datetime, Float64, Int32, Int64, Time, UInt32, UInt64
-from polars.testing import assert_series_equal, verify_series_and_expr_api
+from polars.datatypes import (
+    Date,
+    Datetime,
+    Field,
+    Float64,
+    Int32,
+    Int64,
+    PolarsDataType,
+    Time,
+    UInt32,
+    UInt64,
+)
+from polars.exceptions import ShapeError
+from polars.internals.construction import iterable_to_pyseries
+from polars.internals.type_aliases import EpochTimeUnit
+from polars.testing import assert_frame_equal, assert_series_equal
+from polars.testing._private import verify_series_and_expr_api
 
 if TYPE_CHECKING:
     from polars.internals.type_aliases import TimeUnit
@@ -43,7 +58,7 @@ def test_init_inputs(monkeypatch: Any) -> None:
         assert pl.Series(dtype_if_empty=pl.Utf8).dtype == pl.Utf8
         assert pl.Series([], dtype_if_empty=pl.UInt16).dtype == pl.UInt16
         # "== []" will be cast to empty Series with Utf8 dtype.
-        pl.testing.assert_series_equal(
+        assert_series_equal(
             pl.Series([], dtype_if_empty=pl.Utf8) == [], pl.Series("", dtype=pl.Boolean)
         )
         assert pl.Series(values=[True, False]).dtype == pl.Boolean
@@ -57,16 +72,16 @@ def test_init_inputs(monkeypatch: Any) -> None:
         assert pl.Series("a", [10000, 20000, 30000], dtype=pl.Time).dtype == pl.Time
 
         # 2d numpy array
-        res = pl.Series(name="a", values=np.array([[1, 2], [3, 4]]))
-        assert all(res[0] == np.array([1, 2]))
-        assert all(res[1] == np.array([3, 4]))
-        assert (
-            pl.Series(values=np.array([["foo", "bar"], ["foo2", "bar2"]])).dtype
-            == pl.Object
-        )
+        res = pl.Series(name="a", values=np.array([[1, 2], [3, 4]], dtype=np.int64))
+        assert res.dtype == pl.List(pl.Int64)
+        assert res[0].to_list() == [1, 2]
+        assert res[1].to_list() == [3, 4]
+        assert pl.Series(
+            values=np.array([["foo", "bar"], ["foo2", "bar2"]])
+        ).dtype == pl.List(pl.Utf8)
 
         # lists
-        assert pl.Series("a", [[1, 2], [3, 4]]).dtype == pl.List
+        assert pl.Series("a", [[1, 2], [3, 4]]).dtype == pl.List(pl.Int64)
 
     # datetime64: check timeunit (auto-detect, implicit/explicit) and NaT
     d64 = pd.date_range(date(2021, 8, 1), date(2021, 8, 3)).values
@@ -95,11 +110,58 @@ def test_init_inputs(monkeypatch: Any) -> None:
         pl.Series("bigint", [2**64])
 
     # numpy not available
-    monkeypatch.setattr(pl.internals.series.series, "_NUMPY_AVAILABLE", False)
+    monkeypatch.setattr(pl.internals.series.series, "_NUMPY_TYPE", lambda x: False)
     with pytest.raises(ValueError):
         pl.DataFrame(np.array([1, 2, 3]), columns=["a"])
 
 
+def test_init_dataclass_namedtuple() -> None:
+    from dataclasses import dataclass
+    from typing import NamedTuple
+
+    @dataclass
+    class TeaShipmentDC:
+        exporter: str
+        importer: str
+        product: str
+        tonnes: int | None
+
+    class TeaShipmentNT(NamedTuple):
+        exporter: str
+        importer: str
+        product: str
+        tonnes: None | int
+
+    for Tea in (TeaShipmentDC, TeaShipmentNT):
+        t0 = Tea(exporter="Sri Lanka", importer="USA", product="Ceylon", tonnes=10)
+        t1 = Tea(exporter="India", importer="UK", product="Darjeeling", tonnes=25)
+
+        s = pl.Series("t", [t0, t1])
+
+        assert isinstance(s, pl.Series)
+        assert s.dtype.fields == [  # type: ignore[attr-defined]
+            Field("exporter", pl.Utf8),
+            Field("importer", pl.Utf8),
+            Field("product", pl.Utf8),
+            Field("tonnes", pl.Int64),
+        ]
+        assert s.to_list() == [
+            {
+                "exporter": "Sri Lanka",
+                "importer": "USA",
+                "product": "Ceylon",
+                "tonnes": 10,
+            },
+            {
+                "exporter": "India",
+                "importer": "UK",
+                "product": "Darjeeling",
+                "tonnes": 25,
+            },
+        ]
+        assert_frame_equal(s.to_frame(), pl.DataFrame({"t": [t0, t1]}))
+
+
 def test_concat() -> None:
     s = pl.Series("a", [2, 1, 3])
 
@@ -209,9 +271,10 @@ def test_arithmetic(s: pl.Series) -> None:
     assert ((a / 1) == [1.0, 2.0]).sum() == 2
     assert ((a // 2) == [0, 1]).sum() == 2
     assert ((a * 2) == [2, 4]).sum() == 2
-    assert ((1 + a) == [2, 3]).sum() == 2
+    assert ((2 + a) == [3, 4]).sum() == 2
     assert ((1 - a) == [0, -1]).sum() == 2
-    assert ((1 * a) == [1, 2]).sum() == 2
+    assert ((2 * a) == [2, 4]).sum() == 2
+
     # integer division
     assert_series_equal(1 / a, pl.Series([1.0, 0.5]))
     if s.dtype == Int64:
@@ -284,6 +347,9 @@ def test_add_string() -> None:
     result = s + " world"
     assert_series_equal(result, pl.Series(["hello world", "weird world"]))
 
+    result = "pfx:" + s
+    assert_series_equal(result, pl.Series(["pfx:hello", "pfx:weird"]))
+
 
 def test_append_extend() -> None:
     a = pl.Series("a", [1, 2])
@@ -327,12 +393,13 @@ def test_various() -> None:
 
 def test_filter_ops() -> None:
     a = pl.Series("a", range(20))
-    assert a[a > 1].len() == 18
-    assert a[a < 1].len() == 1
-    assert a[a <= 1].len() == 2
-    assert a[a >= 1].len() == 19
-    assert a[a == 1].len() == 1
-    assert a[a != 1].len() == 19
+    with pytest.deprecated_call(match="passing a boolean mask to Series.__getitem__"):
+        assert a[a > 1].len() == 18
+        assert a[a < 1].len() == 1
+        assert a[a <= 1].len() == 2
+        assert a[a >= 1].len() == 19
+        assert a[a == 1].len() == 1
+        assert a[a != 1].len() == 19
 
 
 def test_cast() -> None:
@@ -350,6 +417,27 @@ def test_cast() -> None:
         pl.Series(["1", "2", "3", "4", "foobar"]).cast(int)
 
 
+def test_to_pandas() -> None:
+    for test_data in (
+        [1, None, 2],
+        ["abc", None, "xyz"],
+        [None, datetime.now()],
+        [[1, 2], [3, 4], None],
+    ):
+        a = pl.Series("s", test_data)
+        b = a.to_pandas()
+
+        assert a.name == b.name
+        assert b.isnull().sum() == 1
+
+        if a.dtype == pl.List:
+            vals = [(None if x is None else x.tolist()) for x in b]
+        else:
+            vals = b.replace({np.nan: None}).values.tolist()  # type: ignore[union-attr]
+
+        assert vals == test_data
+
+
 def test_to_python() -> None:
     a = pl.Series("a", range(20))
     b = a.to_list()
@@ -457,6 +545,10 @@ def test_ufunc() -> None:
         cast(pl.Series, np.power(s_uint8, 2.0)),
         pl.Series("a", [1.0, 4.0, 9.0, 16.0], dtype=pl.Float64),
     )
+    assert_series_equal(
+        cast(pl.Series, np.power(s_uint8, 2, dtype=np.uint16)),
+        pl.Series("a", [1, 4, 9, 16], dtype=pl.UInt16),
+    )
 
     s_int8 = pl.Series("a", [1, -2, 3, -4], dtype=pl.Int8)
     assert_series_equal(
@@ -467,6 +559,10 @@ def test_ufunc() -> None:
         cast(pl.Series, np.power(s_int8, 2.0)),
         pl.Series("a", [1.0, 4.0, 9.0, 16.0], dtype=pl.Float64),
     )
+    assert_series_equal(
+        cast(pl.Series, np.power(s_int8, 2, dtype=np.int16)),
+        pl.Series("a", [1, 4, 9, 16], dtype=pl.Int16),
+    )
 
     s_uint32 = pl.Series("a", [1, 2, 3, 4], dtype=pl.UInt32)
     assert_series_equal(
@@ -641,6 +737,58 @@ def test_fill_null() -> None:
     assert a.fill_null(strategy="backward").to_list() == [0.0, 1.0, 2.0, 2.0, 3.0, 3.0]
     assert a.fill_null(strategy="mean").to_list() == [0.0, 1.0, 1.5, 2.0, 1.5, 3.0]
 
+    df = pl.DataFrame(
+        [
+            pl.Series("i32", [1, 2, None], dtype=pl.Int32),
+            pl.Series("i64", [1, 2, None], dtype=pl.Int64),
+            pl.Series("f32", [1, 2, None], dtype=pl.Float32),
+            pl.Series("cat", ["a", "b", None], dtype=pl.Categorical),
+            pl.Series("str", ["a", "b", None], dtype=pl.Utf8),
+            pl.Series("bool", [True, True, None], dtype=pl.Boolean),
+        ]
+    )
+
+    assert df.fill_null(0, matches_supertype=False).fill_null("bar").fill_null(
+        False
+    ).to_dict(False) == {
+        "i32": [1, 2, None],
+        "i64": [1, 2, 0],
+        "f32": [1.0, 2.0, None],
+        "cat": ["a", "b", "bar"],
+        "str": ["a", "b", "bar"],
+        "bool": [True, True, False],
+    }
+
+    assert df.fill_null(0, matches_supertype=True).fill_null("bar").fill_null(
+        False
+    ).to_dict(False) == {
+        "i32": [1, 2, 0],
+        "i64": [1, 2, 0],
+        "f32": [1.0, 2.0, 0.0],
+        "cat": ["a", "b", "bar"],
+        "str": ["a", "b", "bar"],
+        "bool": [True, True, False],
+    }
+    df = pl.DataFrame({"a": [1, None, 2, None]})
+
+    out = df.with_columns(
+        [
+            pl.col("a").cast(pl.UInt8).alias("u8"),
+            pl.col("a").cast(pl.UInt16).alias("u16"),
+            pl.col("a").cast(pl.UInt32).alias("u32"),
+            pl.col("a").cast(pl.UInt64).alias("u64"),
+        ]
+    ).fill_null(3)
+
+    assert out.to_dict(False) == {
+        "a": [1, 3, 2, 3],
+        "u8": [1, 3, 2, 3],
+        "u16": [1, 3, 2, 3],
+        "u32": [1, 3, 2, 3],
+        "u64": [1, 3, 2, 3],
+    }
+    assert out.dtypes == [pl.Int64, pl.UInt8, pl.UInt16, pl.UInt32, pl.UInt64]
+
 
 def test_fill_nan() -> None:
     nan = float("nan")
@@ -854,11 +1002,38 @@ def test_describe() -> None:
     date_s = pl.Series([date(2021, 1, 1), date(2021, 1, 2), date(2021, 1, 3)])
     empty_s = pl.Series(np.empty(0))
 
-    assert num_s.describe().shape == (6, 2)
-    assert float_s.describe().shape == (6, 2)
-    assert str_s.describe().shape == (3, 2)
-    assert bool_s.describe().shape == (3, 2)
-    assert date_s.describe().shape == (4, 2)
+    assert {k: v for k, v in num_s.describe().rows()} == {
+        "count": 3.0,
+        "max": 3.0,
+        "mean": 2.0,
+        "min": 1.0,
+        "null_count": 0.0,
+        "std": 1.0,
+    }
+    assert {k: v for k, v in float_s.describe().rows()} == {
+        "count": 3.0,
+        "max": 8.9,
+        "mean": 4.933333333333334,
+        "min": 1.3,
+        "null_count": 0.0,
+        "std": 3.8109491381194442,
+    }
+    assert {k: v for k, v in str_s.describe().rows()} == {
+        "count": 3,
+        "null_count": 0,
+        "unique": 3,
+    }
+    assert {k: v for k, v in bool_s.describe().rows()} == {
+        "count": 5,
+        "null_count": 1,
+        "sum": 3,
+    }
+    assert {k: v for k, v in date_s.describe().rows()} == {
+        "count": "3",
+        "max": "2021-01-03",
+        "min": "2021-01-01",
+        "null_count": "0",
+    }
 
     with pytest.raises(ValueError):
         assert empty_s.describe()
@@ -1227,6 +1402,46 @@ def test_to_numpy(monkeypatch: Any) -> None:
                 assert np_array_with_missing_values.flags.writeable == writable
 
 
+def test_from_generator_or_iterable() -> None:
+    # generator function
+    def gen(n: int) -> Iterator[int]:
+        yield from range(n)
+
+    # iterable object
+    class Data:
+        def __init__(self, n: int):
+            self._n = n
+
+        def __iter__(self) -> Iterator[int]:
+            yield from gen(self._n)
+
+    expected = pl.Series("s", range(10))
+    assert expected.dtype == pl.Int64
+
+    for generated_series in (
+        pl.Series("s", values=gen(10)),
+        pl.Series("s", values=Data(10)),
+        pl.Series("s", values=(x for x in gen(10))),
+    ):
+        assert_series_equal(expected, generated_series)
+
+    # test 'iterable_to_pyseries' directly to validate 'chunk_size' behaviour
+    ps1 = iterable_to_pyseries("s", gen(10), dtype=pl.UInt8)
+    ps2 = iterable_to_pyseries("s", gen(10), dtype=pl.UInt8, chunk_size=3)
+    ps3 = iterable_to_pyseries("s", Data(10), dtype=pl.UInt8, chunk_size=6)
+
+    expected = pl.Series("s", range(10), dtype=pl.UInt8)
+    assert expected.dtype == pl.UInt8
+
+    for ps in (ps1, ps2, ps3):
+        generated_series = pl.Series("s")
+        generated_series._s = ps
+        assert_series_equal(expected, generated_series)
+
+    # empty generator
+    assert_series_equal(pl.Series("s", []), pl.Series("s", values=gen(0)))
+
+
 def test_from_sequences(monkeypatch: Any) -> None:
     # test int, str, bool, flt
     values = [
@@ -1247,6 +1462,7 @@ def test_from_sequences(monkeypatch: Any) -> None:
 
 def test_comparisons_int_series_to_float() -> None:
     srs_int = pl.Series([1, 2, 3, 4])
+
     assert_series_equal(srs_int - 1.0, pl.Series([0.0, 1.0, 2.0, 3.0]))
     assert_series_equal(srs_int + 1.0, pl.Series([2.0, 3.0, 4.0, 5.0]))
     assert_series_equal(srs_int * 2.0, pl.Series([2.0, 4.0, 6.0, 8.0]))
@@ -1254,6 +1470,9 @@ def test_comparisons_int_series_to_float() -> None:
     assert_series_equal(srs_int % 2.0, pl.Series([1.0, 0.0, 1.0, 0.0]))
     assert_series_equal(4.0 % srs_int, pl.Series([0.0, 0.0, 1.0, 0.0]))
 
+    assert_series_equal(srs_int - pl.lit(1.0), pl.Series([0.0, 1.0, 2.0, 3.0]))
+    assert_series_equal(srs_int + pl.lit(1.0), pl.Series([2.0, 3.0, 4.0, 5.0]))
+
     assert_series_equal(srs_int // 2.0, pl.Series([0.0, 1.0, 1.0, 2.0]))
     assert_series_equal(srs_int < 3.0, pl.Series([True, True, False, False]))
     assert_series_equal(srs_int <= 3.0, pl.Series([True, True, True, False]))
@@ -1265,6 +1484,7 @@ def test_comparisons_int_series_to_float() -> None:
 
 def test_comparisons_float_series_to_int() -> None:
     srs_float = pl.Series([1.0, 2.0, 3.0, 4.0])
+
     assert_series_equal(srs_float - 1, pl.Series([0.0, 1.0, 2.0, 3.0]))
     assert_series_equal(srs_float + 1, pl.Series([2.0, 3.0, 4.0, 5.0]))
     assert_series_equal(srs_float * 2, pl.Series([2.0, 4.0, 6.0, 8.0]))
@@ -1272,6 +1492,9 @@ def test_comparisons_float_series_to_int() -> None:
     assert_series_equal(srs_float % 2, pl.Series([1.0, 0.0, 1.0, 0.0]))
     assert_series_equal(4 % srs_float, pl.Series([0.0, 0.0, 1.0, 0.0]))
 
+    assert_series_equal(srs_float - pl.lit(1), pl.Series([0.0, 1.0, 2.0, 3.0]))
+    assert_series_equal(srs_float + pl.lit(1), pl.Series([2.0, 3.0, 4.0, 5.0]))
+
     assert_series_equal(srs_float // 2, pl.Series([0.0, 1.0, 1.0, 2.0]))
     assert_series_equal(srs_float < 3, pl.Series([True, True, False, False]))
     assert_series_equal(srs_float <= 3, pl.Series([True, True, True, False]))
@@ -1381,6 +1604,13 @@ def test_arg_min_and_arg_max() -> None:
     assert s.arg_min() == 3
     assert s.arg_max() == 0
 
+    s = pl.Series([None, True, False, True])
+    assert s.arg_min() == 2
+    assert s.arg_max() == 1
+    s = pl.Series([None, None], dtype=pl.Boolean)
+    assert s.arg_min() is None
+    assert s.arg_max() is None
+
 
 def test_is_null_is_not_null() -> None:
     s = pl.Series("a", [1.0, 2.0, 3.0, None])
@@ -1412,9 +1642,21 @@ def test_is_duplicated() -> None:
 
 
 def test_dot() -> None:
-    s = pl.Series("a", [1, 2, 3])
+    s1 = pl.Series("a", [1, 2, 3])
     s2 = pl.Series("b", [4.0, 5.0, 6.0])
-    assert s.dot(s2) == 32
+
+    assert np.array([1, 2, 3]) @ np.array([4, 5, 6]) == 32
+
+    for dot_result in (
+        s1.dot(s2),
+        s1 @ s2,
+        [1, 2, 3] @ s2,
+        s1 @ np.array([4, 5, 6]),
+    ):
+        assert dot_result == 32
+
+    with pytest.raises(ShapeError, match="length mismatch"):
+        s1 @ [4, 5, 6, 7, 8]
 
 
 def test_sample() -> None:
@@ -1461,11 +1703,17 @@ def test_str_concat() -> None:
 
 
 def test_str_lengths() -> None:
-    s = pl.Series(["messi", "ronaldo", None])
-    expected = pl.Series([5, 7, None], dtype=UInt32)
+    s = pl.Series(["Café", None, "345", "東京"])
+    expected = pl.Series([5, None, 3, 6], dtype=UInt32)
     verify_series_and_expr_api(s, expected, "str.lengths")
 
 
+def test_str_n_chars() -> None:
+    s = pl.Series(["Café", None, "345", "東京"])
+    expected = pl.Series([4, None, 3, 2], dtype=UInt32)
+    verify_series_and_expr_api(s, expected, "str.n_chars")
+
+
 def test_str_contains() -> None:
     s = pl.Series(["messi", "ronaldo", "ibrahimovic"])
     expected = pl.Series([True, False, False])
@@ -1528,11 +1776,26 @@ def test_str_rstrip() -> None:
     expected = pl.Series([" hello", "world"])
     assert_series_equal(s.str.rstrip(), expected)
 
+    s = pl.Series([" hello ", "world\t "])
+    expected = pl.Series([" hell", "world"])
+    assert_series_equal(s.str.rstrip().str.rstrip("o"), expected)
+
+
+def test_str_strip() -> None:
+    s = pl.Series([" hello ", "world\t "])
+    expected = pl.Series(["hello", "world"])
+    assert_series_equal(s.str.strip(), expected)
+
+    expected = pl.Series(["hello", "worl"])
+    assert_series_equal(s.str.strip().str.strip("d"), expected)
+
 
 def test_str_lstrip() -> None:
     s = pl.Series([" hello ", "\t world"])
     expected = pl.Series(["hello ", "world"])
     assert_series_equal(s.str.lstrip(), expected)
+    expected = pl.Series(["ello ", "world"])
+    assert_series_equal(s.str.lstrip().str.lstrip("h"), expected)
 
 
 def test_str_strptime() -> None:
@@ -1566,7 +1829,7 @@ def test_dt_year_month_week_day_ordinal_day() -> None:
     verify_series_and_expr_api(a, exp, "dt.year")
 
     verify_series_and_expr_api(a, pl.Series("a", [5, 10, 2], dtype=UInt32), "dt.month")
-    verify_series_and_expr_api(a, pl.Series("a", [0, 4, 1], dtype=UInt32), "dt.weekday")
+    verify_series_and_expr_api(a, pl.Series("a", [1, 5, 2], dtype=UInt32), "dt.weekday")
     verify_series_and_expr_api(a, pl.Series("a", [21, 40, 8], dtype=UInt32), "dt.week")
     verify_series_and_expr_api(a, pl.Series("a", [19, 4, 20], dtype=UInt32), "dt.day")
     verify_series_and_expr_api(
@@ -1856,6 +2119,32 @@ def test_ewm_std_var() -> None:
     assert np.allclose(var, std**2, rtol=1e-16)
 
 
+def test_ewm_param_validation() -> None:
+    s = pl.Series("values", range(10))
+
+    with pytest.raises(ValueError, match="mutually exclusive"):
+        s.ewm_std(com=0.5, alpha=0.5)
+
+    with pytest.raises(ValueError, match="mutually exclusive"):
+        s.ewm_mean(span=1.5, half_life=0.75)
+
+    with pytest.raises(ValueError, match="mutually exclusive"):
+        s.ewm_var(alpha=0.5, span=1.5)
+
+    with pytest.raises(ValueError, match="Require 'com' >= 0"):
+        s.ewm_std(com=-0.5)
+
+    with pytest.raises(ValueError, match="Require 'span' >= 1"):
+        s.ewm_mean(span=0.5)
+
+    with pytest.raises(ValueError, match="Require 'half_life' > 0"):
+        s.ewm_var(half_life=0)
+
+    for alpha in (-0.5, -0.0000001, 0.0, 1.0000001, 1.5):
+        with pytest.raises(ValueError, match="Require 0 < 'alpha' <= 1"):
+            s.ewm_std(alpha=alpha)
+
+
 def test_extend_constant() -> None:
     a = pl.Series("a", [1, 2, 3])
     expected = pl.Series("a", [1, 2, 3, 1, 1, 1])
@@ -1907,6 +2196,23 @@ def test_ceil() -> None:
     verify_series_and_expr_api(a, expected, "ceil")
 
 
+def test_duration_arithmetic() -> None:
+    # apply some basic duration math to series
+    s = pl.Series([datetime(2022, 1, 1, 10, 20, 30), datetime(2022, 1, 2, 20, 40, 50)])
+    d1 = pl.duration(days=5, microseconds=123456)
+    d2 = timedelta(days=5, microseconds=123456)
+
+    expected_values = [
+        datetime(2022, 1, 6, 10, 20, 30, 123456),
+        datetime(2022, 1, 7, 20, 40, 50, 123456),
+    ]
+    for d in (d1, d2):
+        df1 = pl.select((s + d).alias("d_offset"))
+        df2 = pl.select((d + s).alias("d_offset"))
+        assert df1["d_offset"].to_list() == expected_values
+        assert df1["d_offset"].series_equal(df2["d_offset"])
+
+
 def test_duration_extract_times() -> None:
     a = pl.Series("a", [datetime(2021, 1, 1)])
     b = pl.Series("b", [datetime(2021, 1, 2)])
@@ -2091,6 +2397,34 @@ def test_repr() -> None:
 
 def test_builtin_abs() -> None:
     s = pl.Series("s", [-1, 0, 1, None])
-    a = abs(s)
+    assert abs(s).to_list() == [1, 0, 1, None]
 
-    assert a.to_list() == [1, 0, 1, None]
+
+@pytest.mark.parametrize(
+    "value, unit, exp, exp_type",
+    [
+        (13285, "d", date(2006, 5, 17), pl.Date),
+        (1147880044, "s", datetime(2006, 5, 17, 15, 34, 4), pl.Datetime),
+        (1147880044 * 1_000, "ms", datetime(2006, 5, 17, 15, 34, 4), pl.Datetime("ms")),
+        (
+            1147880044 * 1_000_000,
+            "us",
+            datetime(2006, 5, 17, 15, 34, 4),
+            pl.Datetime("us"),
+        ),
+        (
+            1147880044 * 1_000_000_000,
+            "ns",
+            datetime(2006, 5, 17, 15, 34, 4),
+            pl.Datetime("ns"),
+        ),
+    ],
+)
+def test_from_epoch_expr(
+    value: int, unit: EpochTimeUnit, exp: date | datetime, exp_type: PolarsDataType
+) -> None:
+    s = pl.Series("timestamp", [value, None])
+    result = pl.from_epoch(s, unit=unit)
+
+    expected = pl.Series("timestamp", [exp, None]).cast(exp_type)
+    assert_series_equal(result, expected)
